#!/usr/bin/env python

from distutils.core import setup
from os.path import join
from glob import glob

src_png = glob(join('winkirip', 'glade', 'pixmaps', '*png'))
src_svg = glob(join('winkirip', 'glade', 'pixmaps', '*svg'))
src_flags = glob(join('winkirip', 'glade', 'pixmaps', 'flags', '*png'))
src_glade = glob(join('winkirip', 'glade', '*glade*'))
src_po = glob(join('winkirip', 'po', '*po*'))
src_i18n_bg = glob(join('winkirip', 'po', 'i18n', 'bg_BG', 'LC_MESSAGES', 'winki.mo'))
src_i18n_de = glob(join('winkirip', 'po', 'i18n', 'de_DE', 'LC_MESSAGES', 'winki.mo'))
src_i18n_es = glob(join('winkirip', 'po', 'i18n', 'es_ES', 'LC_MESSAGES', 'winki.mo'))
src_i18n_uk = glob(join('winkirip', 'po', 'i18n', 'uk_UA', 'LC_MESSAGES', 'winki.mo'))
src_i18n_fr = glob(join('winkirip', 'po', 'i18n', 'fr_FR', 'LC_MESSAGES', 'winki.mo'))
#src_omf = glob(join('winkirip', 'omf', '*omf'))
src_flag_descr = glob(join('winkirip', 'iso_flags.csv'))
#src_xml = glob(join('winkirip', 'help', 'C', '*xml'))
#src_figures = glob(join('winkirip', 'help', 'C', 'figures', '*png'))
src_icon = glob(join('winkirip', 'glade', 'pixmaps', 'winki*.png'))
src_desktop = glob(join('winkirip', 'desktop', '*desktop'))
#src_winkirc = [join('winkirip', 'example', 'winkirc')]
src_script = join('winkirip', 'winki')
src_authors = join('winkirip', 'AUTHORS')
src_gpl = join('winkirip', 'GPL')
src_install = join('winkirip', 'INSTALL')
src_readme = join('winkirip', 'README')
src_changelog = join('winkirip', 'CHANGELOG')
src_todo = join('winkirip', 'TODO')

# winki_path is set here _and_ in lib.py (i18n stuff in winkirip/install)
winki_path = join('/', 'usr', 'share', 'winkirip')
dest_glade = join(winki_path, 'glade')
dest_flags = join(winki_path, 'glade', 'flags')
dest_doc = join(winki_path, 'doc')
dest_po = join(winki_path, 'po')
dest_i18n_bg = join(winki_path, 'i18n', 'bg_BG', 'LC_MESSAGES')
dest_i18n_de = join(winki_path, 'i18n', 'de_DE', 'LC_MESSAGES')
dest_i18n_es = join(winki_path, 'i18n', 'es_ES', 'LC_MESSAGES')
dest_i18n_uk = join(winki_path, 'i18n', 'uk_UA', 'LC_MESSAGES')
dest_i18n_fr = join(winki_path, 'i18n', 'fr_FR', 'LC_MESSAGES')
#dest_omf = join('/', 'usr', 'share', 'omf', 'winki')
#dest_help = join('/', 'usr', 'share', 'gnome', 'help', 'winki', 'C')
#dest_figures = join('/', 'usr', 'share', 'gnome', 'help', 'winki', 'C', 'figures')
dest_icon = join('/', 'usr', 'share', 'pixmaps')
dest_desktop = join('/', 'usr', 'share', 'applications')

winkiDescription = \
"""Winki is an easy to use frontend for various command line video conversion tools.
Supported input formats:
- DVD (PAL only)
Supported output formats:
- DVD (PAL only)
- matroska or AVI multimedia file (XviD, DivX or MPEG-2 video; OGG, MP3, MP2 or AC3 audio)"""

winkiClassifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: X11 Applications',
    'Intended Audience :: End Users/Desktop',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Operating System :: POSIX',
    'Programming Language :: Python',
    'Natural Language :: English',
    'Topic :: Desktop Environment :: Gnome',
    'Topic :: Desktop Environment :: XFCE'
    ]

setup(  name = 'winki',
    version = '0.4.5',
    description = 'GTK frontend to various command line video encoding tools',
    long_description = winkiDescription,
    author = 'Velko Hristov',
    author_email = 'velko@winki-the-ripper.de',
    url = 'http://www.winki-the-ripper.de',
    license = 'GPL',
    keywords = "dvd encoding mpeg2 mpeg4 ac3 mp2 mp3 ogg mkv avi",
    #classifiers = [ 'Development Status :: 5 - Alpha', 'Intended Audience :: End Users/Desktop'],
    classifiers = winkiClassifiers,
    scripts = [src_script],
    data_files = [(dest_glade, src_png),
              (dest_glade, src_svg),
              (dest_glade, src_glade),
              (dest_doc, [src_authors,
                          src_gpl,
                          src_install,
                          src_readme,
                          src_changelog,
                          src_todo]),
              (dest_flags, src_flags),
              (dest_po, src_po),
              (dest_i18n_bg, src_i18n_bg),
              (dest_i18n_de, src_i18n_de),
              (dest_i18n_es, src_i18n_es),
              (dest_i18n_uk, src_i18n_uk),
              (dest_i18n_fr, src_i18n_fr),
#             (dest_omf, src_omf),
              (winki_path, src_flag_descr),
              (dest_icon, src_icon),
              (dest_desktop, src_desktop)],
    packages = ['winkirip'])

