# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import events_pending, main_iteration, WIN_POS_CENTER_ON_PARENT
from gtk.glade import XML
from time import sleep
from winkirip.lib import build_path, winki_path, check_widgets, set_button_data

class BaseAlert:
    def __init__(self, widget_name):
        self.xml = None
        self.state = 'unknown'
        self.__glade_file = build_path([winki_path, 'glade', 'alert.glade'])

        try:
            self.xml = XML(self.__glade_file, widget_name)
        except:
            print 'BaseAlert: Cannot load %s from %s' % (widget_name, self.__glade_file)
            raise SystemExit

        dic = self.get_dictionary()
        self.xml.signal_autoconnect(dic)

        check_widgets(self.xml, self.get_widgets())

        self.__dialog = self.xml.get_widget(widget_name)
        self.__dialog.connect('delete_event', self.no_quit)
        self.__dialog.set_position(WIN_POS_CENTER_ON_PARENT)
        return

    def get_dictionary(self):
        return {}

    def get_widgets(self):
        return []

    def no_quit(self, button, event = None):
        "disallow close"
        return True

    def execute(self, modal):
        self.__dialog.set_modal(True)
        self.__dialog.show()

        if not modal:
            return

        while self.state == 'unknown':
            sleep(0.001)
            while events_pending():
                main_iteration(False)
        return

    def hide(self):
        self.__dialog.hide()
        return True

    def reset(self):
        return

class AlertWarning(BaseAlert):
    def __init__(self):
        BaseAlert.__init__(self, 'alert_warning')
        return

    def get_dictionary(self):
        return {'on_button_alert_cancel_clicked' : self.on_cancel,
                'on_button_alert_affirmative_clicked' : self.on_ok}

    def get_widgets(self):
        return ['label_alert',
                'button_alert_cancel',
                'button_alert_affirmative']

    def show(self, primary_text, secondary_text, button_text):
        self.state = 'unknown'

        text = ('<span weight="bold" size="larger">%s</span>\n\n%s' % (primary_text, secondary_text))

        label = self.xml.get_widget('label_alert')
        label.set_markup(text)

        button = self.xml.get_widget('button_alert_affirmative')
        set_button_data(button, button_text)
        button.grab_focus()

        self.execute(True)
        return True

    def on_ok(self, widget, event = None):
        self.state = 'ok'
        self.hide()
        return True

    def on_cancel(self, widget, event = None):
        self.state = 'cancel'
        self.hide()
        return True

class AlertError(BaseAlert):
    def __init__(self):
        BaseAlert.__init__(self, 'alert_error')
        return

    def get_dictionary(self):
        return {'on_button_ok_clicked' : self.on_ok}

    def get_widgets(self):
        return ['label_error',
                'button_ok']

    def show(self, primary_text, secondary_text, modal = True):
        self.state = 'unknown'

        text = ('<span weight="bold" size="larger">%s</span>\n\n%s' % (primary_text, secondary_text))

        label = self.xml.get_widget('label_error')
        label.set_markup(text)

        button = self.xml.get_widget('button_ok')
        button.grab_focus()

        self.execute(modal)
        return True

    def on_ok(self, widget, event = None):
        self.state = 'ok'
        self.hide()
        return True

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
