# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from ConfigParser import SafeConfigParser
from os.path import basename
from winkirip.rc_version import RCVersion
from winkirip.lib import normalize_path, to_int, to_float, debug, error
from winkirip.container_audio import AUDIO_UNKNOWN, AUDIO_PCM, AUDIO_AC3
from winkirip.container_audio import AUDIO_OGG, AUDIO_MP2, AUDIO_MP3
from winkirip.container_audio import AUDIO_AAC, AUDIO_FLAC, AUDIO_MOBILE
from winkirip.container_subtitle import SUBTITLE_UNKNOWN, SUBTITLE_VOBSUB
from winkirip.container_subtitle import SUBTITLE_SRT, SUBTITLE_SUB
from winkirip.container_video import VIDEO_MPEG1, VIDEO_MPEG2, VIDEO_MPEG4
from winkirip.container_video import VIDEO_XVID, VIDEO_H264, VIDEO_MOBILE
from winkirip.container_video import VIDEO_DIVX
from winkirip.container import CONTAINER_DVD, CONTAINER_VCD, CONTAINER_SVCD
from winkirip.container import CONTAINER_FILE
from winkirip.container import FILE_MATROSKA, FILE_AVI, FILE_MP4, FILE_MOBILE
from winkirip.select_settings import PRESET_FILE_NONE, PRESET_FILE_QUALITY
from winkirip.select_settings import PRESET_FILE_COMPRESSION, PRESET_FILE_SPEED
from winkirip.select_settings import PRESET_FILE_IPOD, PRESET_FILE_MOBILE
from winkirip.select_settings import PRESET_DVD_NONE, PRESET_DVD_QUALITY
from winkirip.select_settings import PRESET_DVD_SPEED

class Config:
    def __init__(self):
        self.__config_file = '~/.winkirc'
        self.__values = None

        self.__audio_dvd = {AUDIO_AC3 : 'AC3',
                            AUDIO_MP2 : 'MP2',
                            AUDIO_PCM : 'PCM'}
        self.__audio_file = {AUDIO_OGG : 'OGG',
                             AUDIO_MP3 : 'MP3',
                             AUDIO_MP2 : 'MP2',
                             AUDIO_AC3 : 'AC3',
                             AUDIO_PCM : 'PCM',
                             AUDIO_AAC : 'AAC',
                             AUDIO_FLAC : 'Flac',
                             AUDIO_MOBILE : 'mobile'}
        self.__video_dvd = {VIDEO_MPEG2 : 'MPEG-2',
                            VIDEO_MPEG1 : 'MPEG-1'}
        self.__video_file = {VIDEO_MPEG2:'MPEG-2',
                             VIDEO_MPEG1 : 'MPEG-1',
                             VIDEO_MPEG4:'MPEG-4',
                             VIDEO_XVID : 'XviD',
                             VIDEO_H264 : 'x264',
                             VIDEO_DIVX : 'DivX',
                             VIDEO_MOBILE : 'mobile'}
        self.__file_format = {FILE_MATROSKA :'matroska',
                              FILE_AVI:'avi',
                              FILE_MP4:'mp4',
                              FILE_MOBILE : 'mobile'}
        self.__file_preset = {PRESET_FILE_NONE : 'none',
                              PRESET_FILE_QUALITY : 'quality',
                              PRESET_FILE_COMPRESSION : 'compression',
                              PRESET_FILE_SPEED : 'speed',
                              PRESET_FILE_IPOD : 'ipod',
                              PRESET_FILE_MOBILE : 'mobile'}
        self.__dvd_preset = {PRESET_DVD_NONE : 'none',
                             PRESET_DVD_QUALITY : 'quality',
                             PRESET_DVD_SPEED : 'speed'}
        self.__ogg_quality = {64 : '64',
                              96 : '96',
                              128 : '128',
                              160 : '160',
                              192 : '192',
                              224 : '224',
                              256 : '256',
                              320 : '320'}
        self.__mp2_quality = {64 : '64',
                              96 : '96',
                              128 : '128',
                              160 : '160',
                              192 : '192',
                              224 : '224',
                              256 : '256',
                              320 : '320'}
        self.__mp3_quality = {32 : '32',
                              40 : '40',
                              48 : '48',
                              56 : '56',
                              64 : '64',
                              96 : '96',
                              128 : '128',
                              160 : '160',
                              192 : '192',
                              224 : '224',
                              256 : '256',
                              320 : '320'}
        self.__ac3_quality = {64 : '64',
                              96 : '96',
                              112 : '112',
                              128 : '128',
                              224 : '224',
                              384 : '384',
                              512 : '512'}
        self.__aac_quality = {40 : '40',
                              48 : '48',
                              56 : '56',
                              64 : '64',
                              80 : '80',
                              96 : '96',
                              112 : '112',
                              128 : '128',
                              160 : '160',
                              192 : '192'}
        return

    # -- public interface --

    def load(self):
        parser = SafeConfigParser()

        fixed_name = normalize_path(self.__config_file)
        if not fixed_name or not len(fixed_name):
            return False

        try:
            parser.read([fixed_name])
        except:
            return False

        versioner = RCVersion()
        self.__values = versioner.update_from_parser(parser)

        return True

    def write(self):
        parser = SafeConfigParser()
        versioner = RCVersion()
        parser = versioner.update_parser(parser, self.__values)

        fixed_name = normalize_path(self.__config_file)
        if not fixed_name or not len(fixed_name):
            return False

        try:
            f = open(fixed_name, 'w')
            parser.write(f)
            f.close()
        except:
            return False

        return True

    # -- get/set functions --

    # the whole point of this duplicated effort is to pass only predictable
    # values into the application

    def get_output_dir(self):
        return self.__get_string_value('output folder', 'settings')

    def set_output_dir(self, value):
        return self.__set_string_value(value, 'output folder','settings')

    def get_source(self):
        source = self.__get_string_value('default source', 'settings')
        if not source in ['dvd', 'file', 'vcd']:
            debug('Config.get_source()', 'invalid source "%s"' % source, error)
            return 'dvd'

        return source

    def set_source(self, value):
        return self.__set_string_value(value, 'default source','settings')

    def get_encoding(self):
        encoding = self.__get_string_value('default encoding', 'settings')
        if not encoding in ['multi pass', 'copy', 'vcd', 'svcd', 'dvd']:
            func = 'Config.get_encoding()'
            msg = 'invalid encoding "%s"' % encoding
            debug(func, msg, error)
            return 'multi pass'

        return encoding

    def set_encoding(self, value):
        return self.__set_string_value(value, 'default encoding','settings')

    def get_file_encoding(self):
        value = self.__get_string_value('default file encoding', 'settings')
        return self.__find_key(value, self.__file_format, FILE_MATROSKA)

    def set_file_encoding(self, value):
        value = self.__find_value(value, self.__file_format, 'matroska')
        return self.__set_string_value(value, 'default file encoding','settings')

    def get_size(self, ctype):
        csize = 'unknown container type'
        if ctype == CONTAINER_DVD:
            csize = self.__get_string_value('default size dvd', 'settings')
        elif ctype == CONTAINER_FILE:
            csize = self.__get_string_value('default size file', 'settings')
        elif ctype == CONTAINER_VCD:
            csize = self.__get_string_value('default size vcd', 'settings')
        elif ctype == CONTAINER_SVCD:
            csize = self.__get_string_value('default size svcd', 'settings')

        sz = to_int(csize)
        if sz < 0:
            debug('Config.get_size()', 'not a number [%s]' % csize, error)
            return 700

        return sz

    def set_size(self, ctype, value):
        option = None
        if ctype == CONTAINER_DVD:
            option = 'default size dvd'
        elif ctype == CONTAINER_FILE:
            option = 'default size file'
        elif ctype == CONTAINER_VCD:
            option = 'default size vcd'
        elif ctype == CONTAINER_SVCD:
            option = 'default size svcd'

        if not option:
            debug('Config.set_size()', 'unknown type [%s]' % ctype, error)
            return

        return self.__set_string_value(value, option, 'settings')

    def get_user_mode(self):
        mode = self.__get_string_value('user mode','settings')
        if mode != 'default' and mode != 'advanced':
            func = 'Config.get_user_mode()'
            msg = 'unknown user mode "%s"' % mode
            debug(func, msg, error)
            mode = 'default'
        return mode

    def set_user_mode(self, value):
        return self.__set_string_value(value, 'user mode','settings')

    def get_dvdrom_devices(self):
        return self.__get_device_list('dvd-rom devices')

    def set_dvdrom_devices(self, dev_list):
        new_list = []
        for device in dev_list:
            if device.is_dvd_reader():
                new_list.append(device)

        return self.__set_device_list('dvd-rom devices', new_list)

    def get_cdrom_devices(self):
        return self.__get_device_list('cd-rom devices')

    def set_cdrom_devices(self, dev_list):
        return self.__set_device_list('cd-rom devices', dev_list)

    def get_device_info(self, section):
        result = {}

        if not self.__values or not self.__values.has_section(section):
            return result

        result['speed'] = self.__get_string_value('speed', section)
        result['manufacturer'] = self.__get_string_value('manufacturer', section)
        result['caps'] = self.__get_string_value('capabilities', section)
        result['model'] = self.__get_string_value('model', section)
        result['version'] = self.__get_string_value('version', section)
        return result

    def get_video_format_dvd(self):
        value = self.__get_string_value('dvd format','video')
        return self.__find_key(value, self.__video_dvd, VIDEO_MPEG2)

    def set_video_format_dvd(self, value):
        value = self.__find_value(value, self.__video_dvd, 'MPEG-2')
        return self.__set_string_value(value, 'dvd format','video')

    def get_video_resolution_dvd(self):
        value = self.__get_string_value('dvd resolution','video')

        if not value in ['720x576', '704x576', '352x576', '352x288', 'original']:
            return 'original'

        return value

    def set_video_resolution_dvd(self, value):
        return self.__set_string_value(value, 'dvd resolution','video')

    def get_video_aspect_dvd(self):
        value = self.__get_string_value('dvd aspect','video')
        if not value in ['16/9', '4/3', 'original']:
            return '16/9'
        return value

    def set_video_aspect_dvd(self, value):
        return self.__set_string_value(value, 'dvd aspect','video')

    def get_video_shrink_method_dvd(self):
        value = self.__get_string_value('dvd shrink method','video')
        if not value in ['requantize', '1 pass', '2 pass']:
            return 'requantize'
        return value

    def set_video_shrink_method_dvd(self, value):
        return self.__set_string_value(value, 'dvd shrink method','video')

    def get_audio_format_dvd(self):
        value = self.__get_string_value('dvd format','audio')
        return self.__find_key(value, self.__audio_dvd, AUDIO_AC3)

    def set_audio_format_dvd(self, value):
        value = self.__find_value(value, self.__audio_dvd, 'AC3')
        return self.__set_string_value(value, 'dvd format','audio')

    def get_dvd_preset(self):
        value = self.__get_string_value('dvd preset','video')
        return self.__find_key(value, self.__dvd_preset, PRESET_DVD_NONE)

    def set_dvd_preset(self, value):
        value = self.__find_value(value, self.__dvd_preset, 'none')
        return self.__set_string_value(value, 'dvd preset','video')

    def get_file_preset(self):
        value = self.__get_string_value('file preset','video')
        return self.__find_key(value, self.__file_preset, PRESET_FILE_NONE)

    def set_file_preset(self, value):
        value = self.__find_value(value, self.__file_preset, 'none')
        return self.__set_string_value(value, 'file preset','video')

    def get_video_format_file(self):
        value = self.__get_string_value('file format','video')
        return self.__find_key(value, self.__video_file, VIDEO_MPEG4)

    def set_video_format_file(self, value):
        value = self.__find_value(value, self.__video_file, 'MPEG-4')
        return self.__set_string_value(value, 'file format','video')

    def get_video_resolution_file(self):
        value = self.__get_string_value('file resolution','video')

        if not value in ['original', '720x576', '704x576', '352x576', '352x288', '384x288', '512x288', '320x240', '176x144']:
            return 'original'

        return value

    def set_video_resolution_file(self, value):
        return self.__set_string_value(value, 'file resolution','video')

    def get_video_aspect_file(self):
        value = self.__get_string_value('file aspect','video')

        if not value in ['16/9', '4/3', '2.11', 'original']:
            return 'original'

        return value

    def set_video_aspect_file(self, value):
        return self.__set_string_value(value, 'file aspect','video')

    def get_video_shrink_method_file(self):
        value = self.__get_string_value('file shrink method','video')

        if not value in ['1 pass', '2 pass']:
            return '1 pass'

        return value

    def set_video_shrink_method_file(self, value):
        return self.__set_string_value(value, 'file shrink method','video')

    def get_audio_format_file(self):
        value = self.__get_string_value('file format','audio')
        return self.__find_key(value, self.__audio_file, AUDIO_OGG)

    def set_audio_format_file(self, value):
        value = self.__find_value(value, self.__audio_file, 'OGG')
        return self.__set_string_value(value, 'file format','audio')

    def get_ogg_quality(self):
        value = self.__get_string_value('ogg quality','audio')
        return self.__find_key(value, self.__ogg_quality, 128)

    def set_ogg_quality(self, value):
        value = self.__find_value(value, self.__ogg_quality, '128')
        return self.__set_string_value(value, 'ogg quality','audio')

    def get_ogg_values(self):
        return self.__ogg_quality.keys()

    def get_mp2_quality(self):
        value = self.__get_string_value('mp2 quality','audio')
        return self.__find_key(value, self.__mp2_quality, 224)

    def set_mp2_quality(self, value):
        value = self.__find_value(value, self.__mp2_quality, '224')
        return self.__set_string_value(value, 'mp2 quality','audio')

    def get_mp2_values(self):
        return self.__mp2_quality.keys()

    def get_mp3_quality(self):
        value = self.__get_string_value('mp3 quality','audio')
        return self.__find_key(value, self.__mp3_quality, 128)

    def set_mp3_quality(self, value):
        value = self.__find_value(value, self.__mp3_quality, '128')
        return self.__set_string_value(value, 'mp3 quality','audio')

    def get_mp3_values(self):
        return self.__mp3_quality.keys()

    def get_ac3_quality(self):
        value = self.__get_string_value('ac3 quality','audio')
        return self.__find_key(value, self.__ac3_quality, 224)

    def set_ac3_quality(self, value):
        value = self.__find_value(value, self.__ac3_quality, '224')
        return self.__set_string_value(value, 'ac3 quality','audio')

    def get_ac3_values(self):
        return self.__ac3_quality.keys()

    def get_aac_quality(self):
        value = self.__get_string_value('aac quality','audio')
        return self.__find_key(value, self.__aac_quality, 128)

    def set_aac_quality(self, value):
        value = self.__find_value(value, self.__aac_quality, '128')
        return self.__set_string_value(value, 'aac quality','audio')

    def get_aac_values(self):
        return self.__aac_quality.keys()

    def get_nice_value(self):
        nice_str = self.__get_string_value('nice value', 'misc')
        value = to_int(nice_str)
        if value > -1:
            return value
        return 19

    def get_lower_threshold(self):
        threshold = self.__get_string_value('lower threshold', 'misc')

        value = 0
        try:
            value = int(threshold)
        except:
            func = 'Config.get_lower_threshold()'
            msg = 'not a number %s' % threshold
            debug(func, msg, error)
            value = 700

        if value < 0:
            func = 'Config.get_lower_threshold()'
            msg = 'invalid value %s' % threshold
            debug(func, msg, error)
            value = 700

        if value > 16000:
            value = 16000

        return value

    def set_lower_treshold(self, value):
        return self.__set_string_value(value, 'lower threshold','misc')

    def get_upper_threshold(self):
        threshold = self.__get_string_value('upper threshold', 'misc')

        value = 0
        try:
            value = int(threshold)
        except:
            func = 'Config.get_upper_threshold()'
            msg = 'not a number %s' % threshold
            debug(func, msg, error)
            value = 2000

        if value < 0:
            func = 'Config.get_upper_threshold()'
            msg = 'invalid value %s' % threshold
            debug(func, msg, error)
            value = 2000

        if value > 16000:
            value = 16000

        return value

    def set_upper_treshold(self, value):
        return self.__set_string_value(value, 'upper threshold','misc')

    def get_last_used_path(self, device):
        result = '.'

        if device == CONTAINER_DVD:
            result = self.__get_string_value('last used dvd path', 'misc')

        elif device == CONTAINER_VCD:
            result = self.__get_string_value('last used vcd path', 'misc')

        elif device == CONTAINER_FILE:
            result = self.__get_string_value('last used file path', 'misc')

        return result

    def set_last_used_path(self, device, value):
        if device == CONTAINER_DVD:
            return self.__set_string_value(value, 'last used dvd path', 'misc')

        elif device == CONTAINER_VCD:
            return self.__set_string_value(value, 'last used vcd path', 'misc')

        elif device == CONTAINER_FILE:
            return self.__set_string_value(value, 'last used file path', 'misc')

        return False

    def get_mplayer_preview_args(self):
        return self.__get_string_value('mplayer preview args', 'misc')

    def get_tool_location(self, tool):
        return self.__get_string_value(tool, 'program locations')

    def set_tool_location(self, tool, value):
        return self.__set_string_value(value, tool,'program locations')

    def get_audio_bytes_per_second(self, format):
        str_value = 'unknown'
        str_format = 'unknown'
        if format == AUDIO_AC3:
            str_value = self.get_ac3_quality()
            str_format = 'ac3'

        elif format == AUDIO_PCM:
            str_value = self.__get_string_value('bytes per pcm second', 'estimation')
            str_format = 'pcm'

        elif format == AUDIO_OGG:
            str_value = self.__get_string_value('bytes per ogg second', 'estimation')
            str_format = 'ogg'

        elif format == AUDIO_MP2:
            str_value = self.get_mp2_quality()
            str_format = 'mp2'

        elif format == AUDIO_MP3:
            str_value = self.get_mp3_quality()
            str_format = 'mp3'

        elif format == AUDIO_AAC:
            str_value = self.get_aac_quality()
            str_format = 'aac'

        value = to_int(str_value)
        if value < 0:
            func = 'Config.get_audio_bytes_per_second()'
            msg = 'cannot estimate size for audio %s (value = %s)'
            debug(func, msg % (str_format, str_value), error)
            return 2000

        if format == AUDIO_AC3:
            value = value * 125.32
        elif format == AUDIO_MP3:
            value = value * 125.36
        elif format == AUDIO_MP2:
            value = value * 125.31

        return value

    def set_audio_bytes_per_second(self, value):
        #return self.__set_string_value(value, 'Use warnings','Misc')
        return False

    def get_subtitle_bytes_per_second(self, format = SUBTITLE_UNKNOWN):
        if format != SUBTITLE_VOBSUB:
            func = 'Config.get_subtitle_bytes_per_second()'
            msg = 'unimplemented format requested: %d'
            debug(func, msg % format, error)

        bps = self.__get_string_value('bytes per vobsub second', 'estimation')
        value = to_int(bps)
        if value < 0:
            func = 'Config.__get_sub_bytes_per_second()'
            msg = 'not a number %s' % bps
            debug(func, msg, error)
            value = 3500
        return value

    def set_subtitle_bytes_per_second(self, value):
        str_value = '%d' % value
        return self.__set_string_value(str_value, 'bytes per vobsub second','estimation')

    def get_preferred_bitrate(self, format):
        bps = 'unknown format'
        if format == VIDEO_MPEG4:
            bps = self.__get_string_value('preferred mpeg-4 bitrate', 'estimation')
        elif format == VIDEO_MPEG2:
            bps = self.__get_string_value('preferred mpeg-2 bitrate', 'estimation')
        elif format == VIDEO_XVID:
            bps = self.__get_string_value('preferred xvid bitrate', 'estimation')
        elif format == VIDEO_H264:
            bps = self.__get_string_value('preferred x264 bitrate', 'estimation')
        else:
            func = 'Config.get_preferred_bitrate()'
            msg = 'unimplemented format requested: %d'
            debug(func, msg % format, error)

        bps = self.__get_string_value('preferred mpeg-4 bitrate', 'estimation')
        value = to_int(bps)
        if value < 0:
            func = 'Config.get_preferred_bitrate()'
            msg = 'not a number %s' % bps
            debug(func, msg, error)
            value = 1200

        return value

    def set_preferred_bitrate(self, value, format = VIDEO_MPEG4):
        str_value = '%d' % value

        if format == VIDEO_MPEG4:
            return self.__set_string_value(str_value, 'preferred mpeg-4 bitrate','estimation')
        elif format == VIDEO_MPEG2:
            return self.__set_string_value(str_value, 'preferred mpeg-2 bitrate','estimation')
        elif format == VIDEO_XVID:
            return self.__set_string_value(str_value, 'preferred xvid bitrate','estimation')
        elif format == VIDEO_H264:
            return self.__set_string_value(str_value, 'preferred x264 bitrate','estimation')
        else:
            func = 'Config.set_preferred_bitrate()'
            msg = 'unimplemented format requested: %d'
            debug(func, msg % format, error)

        return False

    def get_minimum_bitrate(self, format):
        bps = 'unknown format'
        if format == VIDEO_MPEG4:
            bps = self.__get_string_value('minimum mpeg-4 bitrate', 'estimation')
        elif format == VIDEO_MPEG2:
            bps = self.__get_string_value('minimum mpeg-2 bitrate', 'estimation')
        elif format == VIDEO_XVID:
            bps = self.__get_string_value('minimum xvid bitrate', 'estimation')
        elif format == VIDEO_H264:
            bps = self.__get_string_value('minimum x264 bitrate', 'estimation')
        else:
            func = 'Config.get_minimum_bitrate()'
            msg = 'unimplemented format requested: [%d]'
            debug(func, msg % format, error)

        value = to_int(bps)
        if value < 0:
            func = 'Config.get_minimum_bitrate()'
            msg = 'not a number: [%s]' % bps
            debug(func, msg, error)
            value = 800

        return value

    def set_minimum_bitrate(self, value, format = VIDEO_MPEG4):
        str_value = '%d' % value

        if format == VIDEO_MPEG4:
            return self.__set_string_value(str_value, 'minimum mpeg-4 bitrate','estimation')
        elif format == VIDEO_MPEG2:
            return self.__set_string_value(str_value, 'minimum mpeg-2 bitrate','estimation')
        elif format == VIDEO_XVID:
            return self.__set_string_value(str_value, 'minimum xvid bitrate','estimation')
        elif format == VIDEO_H264:
            return self.__set_string_value(str_value, 'minimum x264 bitrate','estimation')
        else:
            func = 'Config.set_minimum_bitrate()'
            msg = 'unimplemented format requested: %d'
            debug(func, msg % format, error)

        return False

    def get_container_penalty(self):
        penalty = self.__get_string_value('matroska container penalty', 'estimation')
        value = to_float(penalty)
        if value < 0.0:
            func = 'Config.get_container_penalty()'
            msg = 'invalid value %s' % penalty
            debug(func, msg, error)
            value = 0.015

        return value

    def set_container_penalty(self, value):
        return self.__set_string_value(value, 'matroska container penalty','estimation')

    def get_svcd_container_penalty(self):
        penalty = self.__get_string_value('svcd container penalty', 'estimation')

        value = to_float(penalty)
        if value < 0.0:
            func = 'Config.get_svcd_container_penalty()'
            msg = 'invalid value %f' % value
            debug(func, msg, error)
            value = 0.14

        return value

    def set_svcd_container_penalty(self, value):
        return self.__set_string_value(value, 'svcd container penalty','estimation')

    def get_vcd_bytes_per_second(self):
        coef = self.__get_string_value('bytes per vcd second', 'estimation')

        value = to_float(coef)
        if value < 0:
            func = 'Config.get_vcd_bytes_per_second()'
            msg = 'invalid value %f' % value
            debug(func, msg, error)
            value = 144500

        return value

    def set_vcd_bytes_per_second(self, value):
        return self.__set_string_value(value, 'bytes per vcd second','estimation')

    def get_window_position(self, win_name):
        win_x = 0
        win_y = 0
        win_w = 800
        win_h = 600

        if win_name == 'settings':
            win_x = 100
            win_y = 100
            win_w = 600
            win_h = 500
        elif win_name == 'properties':
            win_x = 100
            win_y = 100
            win_w = 600
            win_h = 400

        pos_string = self.__get_string_value('%s position' % win_name, 'settings')
        if pos_string and len(pos_string) > 0 and len(pos_string.split()) == 4:
            strx, stry, strw, strh = pos_string.split()
            if to_int(strx) > -1 and to_int(stry) > -1 and to_int(strw) > -1 and to_int(strh) > -1:
                win_x = to_int(strx)
                win_y = to_int(stry)
                win_w = to_int(strw)
                win_h = to_int(strh)
        return win_x, win_y, win_w, win_h

    def set_window_position(self, win_name, win_x, win_y, win_w, win_h):
        value = '%d %d %d %d' % (win_x, win_y, win_w, win_h)
        return self.__set_string_value(value, '%s position' % win_name, 'settings')

    def get_chapter_rip(self):
        value = self.__get_string_value('chapter rip', 'settings')
        if value.lower() == 'true':
            return True
        return False

    def set_chapter_rip(self, value):
        str_value = 'false'
        if value:
            str_value = 'true'
        return self.__set_string_value(str_value, 'chapter rip','settings')

    def get_exit_winki(self):
        value = self.__get_string_value('exit winki', 'settings')
        if value.lower() == 'true':
            return True
        return False

    def set_exit_winki(self, value):
        str_value = 'false'
        if value:
            str_value = 'true'
        return self.__set_string_value(str_value, 'exit winki','settings')

    def get_show_popups(self):
        value = self.__get_string_value('show popups', 'settings')
        if value.lower() == 'true':
            return True
        return False

    def set_show_popups(self, value):
        str_value = 'false'
        if value:
            str_value = 'true'
        return self.__set_string_value(str_value, 'show popups','settings')

    def get_update_rip_ui(self):
        value = self.__get_string_value('update rip ui', 'settings')
        if value.lower() == 'true':
            return True
        return False

    def set_update_rip_ui(self, value):
        str_value = 'false'
        if value:
            str_value = 'true'
        return self.__set_string_value(str_value, 'update rip ui','settings')

    def get_test_rip(self):
        value = self.__get_string_value('test rip', 'settings')
        if value.lower() == 'true':
            return True
        return False

    def set_test_rip(self, value):
        str_value = 'false'
        if value:
            str_value = 'true'
        return self.__set_string_value(str_value, 'test rip','settings')

    # -- helpers --

    def __set_string_value(self, value, option, section):
        if not self.__values:
            return False

        if not self.__values.has_section(section):
            self.__values.add_section(section)

        self.__values.set_option(section, option, value)
        return True

    def __get_string_value(self, key, section):
        value = ''
        if not self.__values:
            func = 'Config.__get_string_value()'
            msg = 'config not loaded'
            debug(func, msg, error)
            return value

        if not self.__values.has_section(section):
            func = 'Config.__get_string_value()'
            msg = 'section [%s] not found' % section
            debug(func, msg, error)
            return value

        sect = self.__values.get_section(section)
        if not sect.has_option(key):
            func = 'Config.__get_string_value()'
            msg = 'option [%s] not found' % key
            debug(func, msg, error)
            return value

        return sect.get_option(key)

    def __get_device_list(self, option_name):
        devices = self.__get_string_value(option_name, 'settings')
        if not devices or not len(devices):
            return ['hdc']

        items = devices.split()
        if not len(items):
            return ['hdc']

        return items

    def __set_device_list(self, option_name, dev_list):
        value = ''
        if not self.__values:
            return False

        for item in dev_list:
            value += ('%s ' % item.get_device_file())

        self.__set_string_value(value, option_name,'settings')

        for item in dev_list:
            section = 'device %s' % item.get_device_file()

            if not self.__values.has_section(section):
                self.__values.add_section(section)

            self.__set_string_value(item.get_speed_string(), 'speed', section)
            self.__set_string_value(item.get_manufacturer(), 'manufacturer', section)
            self.__set_string_value(item.get_capabilities_string(), 'capabilities', section)
            self.__set_string_value(item.get_model(), 'model', section)
            self.__set_string_value(item.get_version(), 'version', section)

        return True

    def __find_value(self, key, dic, default):
        result = default
        try:
            result = dic[key]
        except KeyError:
            pass
        return result

    def __find_key(self, value, dic, default):
        result = default

        for key in dic:
            if dic[key] == value:
                result = key
                break

        return result


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
