# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.lib import set_quotes

from winkirip import lib

class Constructor:
    def __init__(self):
        self.__src_number = None
        self.__src_device = None
        self.__aspect = '16/9'
        self.__test_rip = False
        self.__title_len = 0
        return

    # -- public interface --

    def set_source(self, src_dev = None, src_nr = None):
        self.__src_device = src_dev
        self.__src_number = src_nr
        return

    def set_test_rip(self, test_rip, title_len):
        self.__test_rip = test_rip
        self.__title_len = title_len
        return

    def construct_dvd_source(self, chapt):
        if not self.__src_number:
            return None

        result = 'dvd://%d' % self.__src_number

        if self.__src_device:
            result += ' -dvd-device %s' % lib.quote_path(self.__src_device)

        result += self.__get_chapter_start_end(chapt)

        return result

    def construct_file_source(self, link_exists = True):
        if not link_exists:
            return set_quotes(self.__src_device)

        start_pos, end_pos = self.__get_start_end_pos()
        if self.__test_rip and start_pos > 0 and end_pos > 0:
            return '%s -ss %d -endpos %d' % (lib.quote_path(self.__src_device),
                                             start_pos, end_pos)

        return self.__src_device

    def set_aspect(self, aspect):
        self.__aspect = aspect
        return

    def get_aspect(self):
        if not self.__aspect or len(self.__aspect) < 1 or self.__aspect == 'original':
            return 'autoaspect'

        return 'aspect=%s' % self.__aspect

    # -- helpers --

    def __get_chapter_start_end(self, chapt):
        start_pos, end_pos = self.__get_start_end_pos()
        result = ''

        if self.__test_rip and start_pos > 0 and end_pos > 0:
            result = ' -ss %d -endpos %d' % (start_pos, end_pos)

        else:
            if chapt and len(chapt) == 2:
                result = ' -chapter %d-%d' % (chapt[0], chapt[1])

        return result

    def __get_start_end_pos(self):
        # 20 seconds test rip length
        test_rip_length = 20
        start_pos = 0
        end_pos = 0

        if self.__title_len > 0 and self.__test_rip:
            start_pos = self.__title_len / 2.
            if (self.__title_len / 2. + test_rip_length) < self.__title_len:
                end_pos = start_pos + test_rip_length
            else:
                end_pos = self.__title_len

        return int(start_pos), int(end_pos)



# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
