# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.constructor import Constructor

class ConstructorAudio(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__outname = None
        self.__bitrate = None
        self.__inname = None
        self.__resample = None
        return

    # -- methods inherited from Constructor --

    def construct(self):
        # overwrite in derived classes
        assert(False)
        return False, ''

    # -- public interface --

    def get_inname(self):
        return self.__inname

    def get_outname(self):
        return self.__outname

    def get_bitrate(self):
        return self.__bitrate

    def get_resample(self):
        return self.__resample

    def get_tool_checker(self):
        return self.__tools

    def set_inname(self, inname):
        self.__inname = inname
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

    def set_bitrate(self, bitrate):
        self.__bitrate = bitrate
        return

    def set_resample(self, resample):
        self.__resample = resample
        return


class ConstructorOgg(ConstructorAudio):
    def __init__(self, tools):
        ConstructorAudio.__init__(self, tools)
        return

    # -- methods inherited from Constructor --

    def construct(self):
        tool_checker = self.get_tool_checker()
        if not tool_checker.has_tool('oggenc'):
            return False, ''

        program = tool_checker.get_tool_location('oggenc')
        bitrate = self.get_bitrate()
        infile = self.get_inname()
        outfile = self.get_outname()

        if not program or not infile or not outfile:
            return False, ''

        resample = self.get_resample()
        downsample = ''
        if resample:
            downsample = '--resample %s' % resample

        bitrate_str = ''
        if bitrate > 0:
            bitrate_str = '-b %d' % bitrate

        cmd = '%s -q 4 %s %s %s -o %s' % (program, bitrate_str, downsample, infile, outfile)
        return True, cmd

class ConstructorMp3(ConstructorAudio):
    def __init__(self, tools):
        ConstructorAudio.__init__(self, tools)
        return

    # -- methods inherited from Constructor --

    def construct(self):
        tool_checker = self.get_tool_checker()
        if not tool_checker.has_tool('lame'):
            return False, ''

        program = tool_checker.get_tool_location('lame')
        bitrate = self.get_bitrate()
        infile = self.get_inname()
        outfile = self.get_outname()

        if not program or not infile or not outfile:
            return False, ''

        resample = self.get_resample()
        downsample = ''
        if resample:
            downsample = '--resample %s' % resample

        bitrate_str = ''
        if bitrate > 0:
            bitrate_str = '-b %d' % bitrate

        cmd = '%s %s %s %s %s' % (program, bitrate_str, downsample, infile, outfile)
        return True, cmd

class ConstructorMp2(ConstructorAudio):
    def __init__(self, tools):
        ConstructorAudio.__init__(self, tools)
        return

    # -- methods inherited from Constructor --

    def construct(self):
        tool_checker = self.get_tool_checker()
        if not tool_checker.has_tool('mp2enc'):
            return False, ''

        program = tool_checker.get_tool_location('mp2enc')
        bitrate = self.get_bitrate()
        infile = self.get_inname()
        outfile = self.get_outname()

        if not program or not infile or not outfile:
            return False, ''

        resample = self.get_resample()
        downsample = ''
        if resample:
            downsample = '-r %s' % resample

        bitrate_str = ''
        if bitrate > 0:
            bitrate_str = '-b %d' % bitrate

        cmd = 'cat %s | %s %s %s -v 2 -o %s' % (infile, program, bitrate_str, downsample, outfile)
        return True, cmd

class ConstructorAc3(ConstructorAudio):
    def __init__(self, tools):
        ConstructorAudio.__init__(self, tools)
        return

    # -- methods inherited from Constructor --

    def construct(self):
        tool_checker = self.get_tool_checker()
        if not tool_checker.has_tool('ffmpeg'):
            return False, ''

        program = tool_checker.get_tool_location('ffmpeg')
        bitrate = self.get_bitrate()
        infile = self.get_inname()
        outfile = self.get_outname()

        if not program or not infile or not outfile:
            return False, ''

        resample = self.get_resample()
        downsample = ''
        if resample:
            downsample = '-ar %s' % resample

        bitrate_str = ''
        if bitrate > 0:
            bitrate_str = '-ab %d' % bitrate

        cmd = '%s -i %s %s %s %s' % (program, infile, bitrate_str, downsample, outfile)
        return True, cmd


class ConstructorAac(ConstructorAudio):
    def __init__(self, tools):
        ConstructorAudio.__init__(self, tools)
        return

    # -- methods inherited from Constructor --

    def construct(self):
        tool_checker = self.get_tool_checker()
        if not tool_checker.has_tool('faac'):
            return False, ''

        program = tool_checker.get_tool_location('faac')
        bitrate = self.get_bitrate()
        infile = self.get_inname()

        if not program or not infile:
            return False, ''

        bitrate_str = ''
        if bitrate > 0:
            bitrate_str = '-b %d' % bitrate

        cmd = '%s %s %s' % (program, bitrate_str, infile)
        return True, cmd


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
