# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_VCD, CONTAINER_SVCD, CONTAINER_FILE, CONTAINER_STREAMS

from winkirip.constructor import Constructor

class ConstructorMplayerAudio(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__srctype = CONTAINER_UNKNOWN
        self.__chapt = None
        self.__outname = None
        self.__aid = -1
        self.__resample = None
        return

    # -- methods inherited from Constructor --

    def construct(self):
        # overwrite in derived class
        assert(False)
        return False, ''

    # -- public interface --

    def set_source_type(self, srctype):
        self.__srctype = srctype
        return

    def set_chapters(self, ch_start, ch_end):
        self.__chapt = [ch_start, ch_end]
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

    def set_aid(self, aid):
        self.__aid = aid
        return

    def set_resample(self, resample):
        self.__resample = resample
        return

    def get_tools(self):
        return self.__tools

    def get_srctype(self):
        return self.__srctype

    def get_chapters(self):
        return self.__chapt

    def get_outname(self):
        return self.__outname

    def get_aid(self):
        return self.__aid

    def get_resample(self):
        return self.__resample

class ConstructorAudioCopy(ConstructorMplayerAudio):
    def __init__(self, tools):
        ConstructorMplayerAudio.__init__(self, tools)
        return

    # -- methods inherited from ConstructorMplayerAudio --

    def construct(self):
        program = None
        if self.get_tools().has_tool('mplayer'):
            program = self.get_tools().get_tool_location('mplayer')

        src = None
        if self.get_srctype() == CONTAINER_DVD:
            src = self.construct_dvd_source(self.get_chapters())
        elif self.get_srctype() == CONTAINER_FILE:
            src = self.construct_file_source()

        if not program or not src or not self.get_outname():
            return False, ''

        aid = ''
        if self.get_aid() > -1:
            aid = '-aid %d' % self.get_aid()

        result = '%s %s %s -dumpaudio -dumpfile %s' % (program, src, aid, self.get_outname())
        return True, result


class ConstructorAudioPcm(ConstructorMplayerAudio):
    def __init__(self, tools):
        ConstructorMplayerAudio.__init__(self, tools)
        return

    # -- methods inherited from ConstructorMplayerAudio --

    def construct(self):
        program = None
        if self.get_tools().has_tool('mplayer'):
            program = self.get_tools().get_tool_location('mplayer')

        src = None
        if self.get_srctype() == CONTAINER_DVD:
            src = self.construct_dvd_source(self.get_chapters())
        elif self.get_srctype() == CONTAINER_FILE:
            src = self.construct_file_source()

        if not program or not src or not self.get_outname():
            return False, ''

        aid = ''
        if self.get_aid() > -1:
            aid = '-aid %d' % self.get_aid()

        downsample = ''
        dest = '-ao pcm:fast:file=%s' % self.get_outname()

        resample = self.get_resample()
        if resample:
            if resample == 'mobile':
                downsample = '-af resample=8000,volume=+4db:sc'
                dest = '-ao pcm:file=%s' % self.get_outname()
            else:
                downsample = '-af resample=%d' % resample

        result = '%s %s -vo null -vc null %s %s %s' % (program, src, aid, downsample, dest)
        return True, result

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
