# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_VCD
from container import CONTAINER_SVCD, CONTAINER_FILE, CONTAINER_STREAMS
from winkirip.constructor import Constructor
from winkirip.lib import debug, error

class ConstructorFFmpeg4(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__srctype = CONTAINER_UNKNOWN
        self.__chapt = None
        self.__amethod = 'none'
        self.__vbitrate = -1
        self.__outname = None
        self.__pass_nr = 1
        self.__pass_count = 1
        self.__scale_expand = ''
        return

    # -- methods inherited from Constructor --

    def construct(self):
        program = None
        if self.__tools.has_tool('mencoder'):
            program = self.__tools.get_tool_location('mencoder')

        src = None
        if self.__srctype == CONTAINER_DVD:
            src = self.construct_dvd_source(self.__chapt)
        elif self.__srctype == CONTAINER_FILE:
            src = self.construct_file_source()

        audio = None
        if self.__amethod == 'copy':
            audio = '-oac copy'
        elif self.__amethod == 'none':
            audio = '-nosound'

        video = '-ovc lavc -lavcopts vcodec=mpeg4:mbd=2:vqmin=2:trell'
        vfilter = '-vf %shqdn3d=2:1:2' % self.__scale_expand

        video += ':%s' % self.get_aspect()

        if self.__vbitrate > 0:
            video += ':vbitrate=%d' % self.__vbitrate

        outname = self.__outname
        if self.__pass_count > 1:
            if self.__pass_nr == 1:
                video += ':vpass=1:turbo'
                outname = '/dev/null'
            else:
                video += ':vpass=2'

        subs = '-forcedsubsonly -noautosub'
        # sigh... probably most of the user will want this
        fourcc = '-ffourcc DX50'

        if not program or not src or not audio or not outname:
            if not program:
                debug('ConstructorFFmpeg4.construct()', 'cannot find mencoder', error)
            if not src:
                debug('ConstructorFFmpeg4.construct()', 'cannot find source video', error)
            if not audio:
                debug('ConstructorFFmpeg4.construct()', 'audio method not set', error)
            if not outname:
                debug('ConstructorFFmpeg4.construct()', 'no output name', error)
            return False, ''

        result = '%s %s %s %s %s %s %s -o %s' % (program, src, subs, audio, video, vfilter, fourcc, outname)
        return True, result

    # -- public interface --

    def set_source_type(self, srctype):
        self.__srctype = srctype
        return

    def set_chapters(self, ch_start, ch_end):
        self.__chapt = [ch_start, ch_end]
        return

    def set_audio_encoding(self, amethod):
        self.__amethod = amethod
        return

    def set_vbitrate(self, vbitrate):
        self.__vbitrate = vbitrate
        return

    def set_scale_expand(self, scale_expand):
        self.__scale_expand = scale_expand
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

    def set_pass_nr(self, pass_nr):
        self.__pass_nr = pass_nr
        return

    def set_pass_count(self, pass_count):
        self.__pass_count = pass_count
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
