# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_VCD
from container import CONTAINER_SVCD, CONTAINER_FILE, CONTAINER_STREAMS
from winkirip.constructor import Constructor
from winkirip.lib import debug, error

class ConstructorMpeg2(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__srctype = CONTAINER_UNKNOWN
        self.__chapt = None
        self.__amethod = 'none'
        self.__vbitrate = -1
        self.__abitrate = -1
        self.__outname = None
        self.__pass_nr = 1
        self.__pass_count = 1
        self.__scale_expand = ''
        return

    # -- methods inherited from Constructor --

    def construct(self):
        program = None
        if self.__tools.has_tool('mencoder'):
            program = self.__tools.get_tool_location('mencoder')

        src = None
        if self.__srctype == CONTAINER_DVD:
            src = self.construct_dvd_source(self.__chapt)
        elif self.__srctype == CONTAINER_FILE:
            src = self.construct_file_source()

        audio = None
        afilter = ''
        aopts = ''
        srate = ''
        if self.__amethod == 'copy':
            audio = '-oac copy'
        elif self.__amethod == 'none':
            audio = '-nosound'
        elif self.__amethod == 'ac3':
            audio = '-oac lavc'
            afilter = '-af lavcresample=%d' %  self.get_audio_sampling_freq()
            aopts = ':acodec=ac3:abitrate=%d' % self.get_abitrate()
            srate = '-srate %d' % self.get_audio_sampling_freq()
        elif self.__amethod == 'mp2':
            audio = '-oac lavc'
            afilter = '-af lavcresample=%d' %  self.get_audio_sampling_freq()
            aopts = ':acodec=mp2:abitrate=%d' % self.get_abitrate()
            srate = '-srate %d' % self.get_audio_sampling_freq()

        oformat = '-of mpeg -mpegopts format=%s' % self.get_mpegformat()

        vrc_max_rate = self.get_vrc_max_rate()
        video = '-ovc lavc -lavcopts vcodec=%s:vrc_buf_size=%d:vrc_maxrate=%d:trell:mbd=2:dc=10:keyint=15' % (self.get_vcodec(), self.get_vrc_buf_size(), vrc_max_rate)
        if self.get_vrc_min_rate() > -1:
            video += ':vrc_min_rate=%d' % self.get_vrc_min_rate()
        video += ':%s' % self.get_aspect()
        video += aopts

        vfilter = '-vf %sharddup' % self.__scale_expand

        fps = '-ofps 25'

        vbitrate = self.get_vbitrate()
        if vbitrate > 0:
            if vbitrate > vrc_max_rate:
                vbitrate = vrc_max_rate
            video += ':vbitrate=%d' % vbitrate

        outname = self.__outname
        if self.__pass_count > 1:
            if self.__pass_nr == 1:
                video += ':vpass=1:turbo'
                outname = '/dev/null'
            else:
                video += ':vpass=2'

        subs = '-forcedsubsonly -noautosub'

        if not program or not src or not audio or not outname:
            if not program:
                debug('ConstructorMpeg2.construct()', 'cannot find mencoder', error)
            if not src:
                debug('ConstructorMpeg2.construct()', 'cannot find source video', error)
            if not audio:
                debug('ConstructorMpeg2.construct()', 'audio method not set', error)
            if not outname:
                debug('ConstructorMpeg2.construct()', 'no output name', error)
            return False, ''

        result = '%s %s %s %s %s %s %s %s %s %s -o %s' % (program, src, subs, audio, oformat, vfilter, srate, afilter, video, fps, outname)
        return True, result

    # -- public interface --

    def set_source_type(self, srctype):
        self.__srctype = srctype
        return

    def set_chapters(self, ch_start, ch_end):
        self.__chapt = [ch_start, ch_end]
        return

    def set_audio_encoding(self, amethod):
        self.__amethod = amethod
        return

    def set_vbitrate(self, vbitrate):
        self.__vbitrate = vbitrate
        return

    def set_abitrate(self, abitrate):
        self.__abitrate = abitrate
        return

    def set_scale_expand(self, scale_expand):
        self.__scale_expand = scale_expand
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

    def set_pass_nr(self, pass_nr):
        self.__pass_nr = pass_nr
        return

    def set_pass_count(self, pass_count):
        self.__pass_count = pass_count
        return

    def get_abitrate(self):
        return self.__abitrate

    def get_vbitrate(self):
        return self.__vbitrate

    def get_vcodec(self):
        return 'mpeg2video'

    def get_mpegformat(self):
        # to be overwritten in derived classes
        assert(False)
        return ''

    def get_audio_sampling_freq(self):
        # to be overwritten in derived classes
        assert(False)
        return -1

    def get_vrc_buf_size(self):
        # to be overwritten in derived classes
        assert(False)
        return -1

    def get_vrc_max_rate(self):
        # to be overwritten in derived classes
        assert(False)
        return -1

    def get_vrc_min_rate(self):
        return -1

class ConstructorMobile(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__scale_expand = None
        self.__outname = None
        self.__srctype = CONTAINER_UNKNOWN
        self.__chapt = None
        self.__pass_nr = 1
        self.__pass_count = 1
        self.__vbitrate = -1 # ignored. default 800 should be ok for a phone
        return

    # -- methods inherited from Constructor --

    def construct(self):
        program = None
        if self.__tools.has_tool('mencoder'):
            program = self.__tools.get_tool_location('mencoder')

        src = None
        if self.__srctype == CONTAINER_DVD:
            src = self.construct_dvd_source(self.__chapt)
        elif self.__srctype == CONTAINER_FILE:
            src = self.construct_file_source()

        if not program or not src or not self.__scale_expand or not self.__outname:
            return False, ''

        output = '-o %s' % self.__outname
        video_opt = ''
        if self.__pass_count > 1:
            if self.__pass_nr == 1:
                video_opt += ':vpass=1:turbo'
                output = '-o /dev/null'
            else:
                video_opt += ':vpass=2'

        opts = '-nosound -ovc lavc -lavcopts vcodec=mpeg4%s' % video_opt
        vfilter = '-vf %s' % self.__scale_expand
        fps = '-ofps 12'
        
        result = '%s %s %s %s %s %s' % (program, src, opts, vfilter, fps, output)

        return True, result

    # -- public interface ---

    def set_scale_expand(self, scale_expand):
        self.__scale_expand = scale_expand
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

    def set_source_type(self, srctype):
        self.__srctype = srctype
        return

    def set_chapters(self, ch_start, ch_end):
        self.__chapt = [ch_start, ch_end]
        return

    def set_pass_nr(self, pass_nr):
        self.__pass_nr = pass_nr
        return

    def set_pass_count(self, pass_count):
        self.__pass_count = pass_count
        return

    def set_vbitrate(self, vbitrate):
        self.__vbitrate = vbitrate
        return


class ConstructorMpeg2DVD(ConstructorMpeg2):
    def __init__(self, tools):
        ConstructorMpeg2.__init__(self, tools)
        return

    def get_mpegformat(self):
        return 'dvd'

    def get_audio_sampling_freq(self):
        return 48000

    def get_abitrate(self):
        # up to 1536
        return 384

    def get_vrc_buf_size(self):
        return 1835

    def get_vrc_max_rate(self):
        return 9800

class ConstructorMpeg2SVCD(ConstructorMpeg2):
    def __init__(self, tools):
        ConstructorMpeg2.__init__(self, tools)
        return

    def get_mpegformat(self):
        return 'xsvcd'

    def get_audio_sampling_freq(self):
        return 44100

    def get_abitrate(self):
        # up to 384
        return 192

    def get_vrc_buf_size(self):
        return 917

    def get_vrc_max_rate(self):
        return 2500

class ConstructorMpeg2VCD(ConstructorMpeg2):
    def __init__(self, tools):
        ConstructorMpeg2.__init__(self, tools)
        return

    def get_mpegformat(self):
        return 'xvcd'

    def get_audio_sampling_freq(self):
        return 44100

    def get_abitrate(self):
        # fixed
        return 224

    def get_vcodec(self):
        return 'mpeg1video'

    def get_vrc_buf_size(self):
        return 327

    def get_vrc_min_rate(self):
        return 1152

    def get_vrc_max_rate(self):
        return 1125

    def get_vbitrate(self):
        return 1152

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
