# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_VCD
from container import CONTAINER_SVCD, CONTAINER_FILE, CONTAINER_STREAMS
from winkirip.constructor import Constructor

class ConstructorMpegVideo(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__srctype = CONTAINER_UNKNOWN
        self.__chapt = None
        self.__outname = None
        return

    # -- methods inherited from Constructor --

    def construct(self):
        program = None
        if self.__tools.has_tool('mencoder'):
            program = self.__tools.get_tool_location('mencoder')

        src = None
        if self.__srctype == CONTAINER_DVD:
            src = self.construct_dvd_source(self.__chapt)
        elif self.__srctype == CONTAINER_FILE:
            src = self.construct_file_source()

        if not program or not src or not self.__outname:
            return False, ''

        dump_opt = '-forcedsubsonly -noautosub -ovc copy -nosound -of mpeg -mpegopts format=dvd -o '

        result = '%s %s %s %s' % (program, src, dump_opt, self.__outname)
        return True, result

    # -- public interface --

    def set_source_type(self, srctype):
        self.__srctype = srctype
        return

    def set_chapters(self, ch_start, ch_end):
        self.__chapt = [ch_start, ch_end]
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
