# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.constructor import Constructor
from winkirip.lib import debug, error

class ConstructorMplex(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__outname = None
        self.__vfile = None
        self.__afiles = []
        return

    # -- methods inherited from Constructor --

    def construct(self):
        if not self.__tools.has_tool('mplex'):
            debug('ConstructorMplex.construct()', 'no mplex installation found', error)
            return False, ''

        prog = self.__tools.get_tool_location('mplex')

        if not self.__outname:
            debug('ConstructorMplex.construct()', 'no output name specified', error)
            return False, ''

        if not self.__vfile:
            debug('ConstructorMplex.construct()', 'no video stream specified', error)
            return False, ''

        if len(self.__afiles) < 1:
            debug('ConstructorMplex.construct()', 'no audio streams for multiplexing', error)
            return False, ''

        result = '%s -v 2 -f 8 -o %s %s' % (prog, self.__outname, self.__vfile)

        for afile in self.__afiles:
            result += ' %s' % afile

        return True, result

    # -- public interface --

    def set_outname(self, outname):
        self.__outname = outname
        return

    def set_vfile(self, vfile):
        self.__vfile = vfile
        return

    def add_afile(self, afile):
        self.__afiles.append(afile)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
