# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.constructor import Constructor
from winkirip.lib import debug, error

class ConstructorSpumux(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__inname = None
        self.__outname = None
        self.__subname = None
        self.__subpos = -1
        return

    # -- methods inherited from Constructor --

    def construct(self):
        if not self.__tools.has_tool('spumux'):
            debug('ConstructorSpumux.construct()', 'no spumux installation found', error)
            return False, ''

        prog = self.__tools.get_tool_location('spumux')

        if not self.__inname:
            debug('ConstructorSpumux.construct()', 'no input name specified', error)
            return False, ''

        if not self.__outname:
            debug('ConstructorSpumux.construct()', 'no output name specified', error)
            return False, ''

        if not self.__subname:
            debug('ConstructorSpumux.construct()', 'no subtitle name specified', error)
            return False, ''

        if self.__subpos < 0:
            debug('ConstructorSpumux.construct()', 'no subtitle position specified', error)
            return False, ''

        result = '%s -m dvd -s %d %s < %s > %s' % (prog, self.__subpos, self.__subname, self.__inname, self.__outname)
        return True, result


    # -- helpers --

    def set_inname(self, inname):
        self.__inname = inname
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

    def set_subname(self, subname):
        self.__subname = subname
        return

    def set_subpos(self, subpos):
        self.__subpos = subpos
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
