# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_VCD
from container import CONTAINER_SVCD, CONTAINER_FILE, CONTAINER_STREAMS
from winkirip.constructor import Constructor

class ConstructorSubtitle(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__srctype = CONTAINER_UNKNOWN
        self.__chapt = None
        self.__outname = None
        self.__sid = -1
        return

    # -- methods inherited from Constructor --

    def construct(self):
        # overwrite in derived class
        assert(False)
        return False, ''

    # -- public interface --

    def set_source_type(self, srctype):
        self.__srctype = srctype
        return

    def set_chapters(self, ch_start, ch_end):
        self.__chapt = [ch_start, ch_end]
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

    def set_sid(self, sid):
        self.__sid = sid
        return

    def get_tools(self):
        return self.__tools

    def get_srctype(self):
        return self.__srctype

    def get_chapters(self):
        return self.__chapt

    def get_outname(self):
        return self.__outname

    def get_sid(self):
        return self.__sid

class ConstructorSubtitleDvd(ConstructorSubtitle):
    def __init__(self, tools):
        ConstructorSubtitle.__init__(self, tools)
        self.__palette = None
        return

    # -- methods inherited from ConstructorSubtitle --

    def construct(self):
        mplayer = None
        if self.get_tools().has_tool('mplayer'):
            mplayer = self.get_tools().get_tool_location('mplayer')
        if self.get_tools().has_tool('spuunmux'):
            spuunmux = self.get_tools().get_tool_location('spuunmux')

        src = None
        if self.get_srctype() == CONTAINER_DVD:
            src = self.construct_dvd_source(self.get_chapters())
        elif self.get_srctype() == CONTAINER_FILE:
            src = self.construct_file_source()

        sid = self.get_sid()
        outname = self.get_outname()
        palette = self.get_palette()

        if not mplayer or not spuunmux or not src or not outname or not palette or sid < 0:
            return False, ''

        templ = '%s %s -dumpstream -dumpfile /dev/stdout | %s -o %s -s %d -p %s /dev/stdin'
        result = templ % (mplayer, src, spuunmux, outname, sid, palette)
        return True, result


    # -- public interface --

    def get_palette(self):
        return self.__palette

    def set_palette(self, palette):
        self.__palette = palette
        return

class ConstructorSubtitleVobsub(ConstructorSubtitle):
    def __init__(self, tools):
        ConstructorSubtitle.__init__(self, tools)
        self.__spos = -1
        return

    # -- methods inherited from ConstructorSubtitle --

    def construct(self):
        program = None
        if self.get_tools().has_tool('mencoder'):
            program = self.get_tools().get_tool_location('mencoder')

        src = None
        if self.get_srctype() == CONTAINER_DVD:
            src = self.construct_dvd_source(self.get_chapters())
        elif self.get_srctype() == CONTAINER_FILE:
            src = self.construct_file_source()

        outname = self.get_outname()
        sid = self.get_sid()
        spos = self.get_spos()

        if not program or not src or not outname or sid < 0 or spos < 0:
            return False, ''

        templ = '%s %s -sid %d -vobsubout %s -vobsuboutindex %d -ovc copy -oac pcm -o /dev/null'
        result = templ % (program, src, sid, outname, spos)
        return True, result

    # -- public interface --

    def set_spos(self, spos):
        self.__spos = spos
        return

    def get_spos(self):
        return self.__spos

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
