# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.lib import debug
from winkirip.container_title import Title
from winkirip.device_info import DeviceInfo

CONTAINER_UNKNOWN = -1
CONTAINER_DVD = 0
CONTAINER_VCD = 1
CONTAINER_SVCD = 2
CONTAINER_FILE = 3
CONTAINER_STREAMS = 4

FILE_UNKNOWN = -1
FILE_MATROSKA = 0
FILE_AVI = 1
FILE_MPEG = 2
FILE_OGM = 3
FILE_WMV = 4
FILE_MP4 = 5
FILE_SUBTITLE = 6
FILE_MOBILE = 7 # quick hack for mbile phones

class Container:
    def __init__(self):
        self.__id = -1
        self.__titles = []
        self.__name = None
        self.__type = CONTAINER_UNKNOWN
        self.__ftype = FILE_UNKNOWN
        self.__size = -1
        self.__tags = {}
        self.__device = None
        self.__device_link = None
        self.__chapter_file = None
        self.__tag_file = None

        self.__iter_index = 0
        return

    # -- get/set --

    def get_id(self):
        return self.__id

    def get_name(self):
        return self.__name

    def get_type(self):
        return self.__type

    def get_file_type(self):
        return self.__ftype

    def get_size(self):
        return self.__size

    def get_tags(self):
        return self.__tags

    def get_device(self, preferre_link = True):
        if preferre_link and self.__device_link:
            return self.__device_link
        return self.__device

    def get_chapter_file(self):
        return self.__chapter_file

    def get_tag_file(self):
        return self.__tag_file

    def set_id(self, id):
        self.__id = id
        for title in self.__titles:
            title.set_container_id(self.__id)
        return

    def set_name(self, name):
        self.__name = name
        return

    def set_type(self, ctype):
        self.__type = ctype
        return

    def set_file_type(self, ftype):
        self.__ftype = ftype
        return

    def set_size(self, size):
        self.__size = size
        return

    def set_tags(self, tags):
        self.__tags = tags
        return

    def set_device(self, device):
        if isinstance(device, DeviceInfo):
            self.__device = device.get_full_device_file()
        else:
            self.__device = device
        return

    def set_device_link(self, device):
        self.__device_link = device
        return

    def set_chapter_file(self, chapter_file):
        self.__chapter_file = chapter_file
        return

    def set_tag_file(self, tag_file):
        self.__tag_file = tag_file
        return

    # -- public interface --

    def remove_titles(self):
        self.__titles = []
        return

    def add_title(self, title, auto_tid = True):
        if not isinstance(title, Title):
            debug('container.add_title()', 'title of wrong type passed')
            return False

        title.set_container_id(self.__id)
        if auto_tid:
            title.set_id(len(self.__titles) + 1)

        self.__titles.append(title)
        return True

    def add_video(self, video):
        cur_title = len(self.__titles) - 1
        if cur_title < 0:
            return False

        return self.__titles[cur_title].add_video(video)

    def set_palette(self, palette):
        cur_title = len(self.__titles) - 1
        if cur_title < 0:
            return False

        return self.__titles[cur_title].set_palette(palette)

    def add_audio(self, audio):
        cur_title = len(self.__titles) - 1
        if cur_title < 0:
            return False

        return self.__titles[cur_title].add_audio(audio)

    def add_subtitle(self, subtitle):
        cur_title = len(self.__titles) - 1
        if cur_title < 0:
            return False

        return self.__titles[cur_title].add_subtitle(subtitle)

    def add_chapter(self, chapter):
        cur_title = len(self.__titles) - 1
        if cur_title < 0:
            return False

        return self.__titles[cur_title].add_chapter(chapter)

    def __repr__(self):
        result = ('Container(name: %s, id: %d, type: %d, size: %d)\n' % (self.__name, self.__id, self.__type, self.__size))
        for title in self.__titles:
            result += ('%s' % title)
        result += ('\ntags: %s' % self.__tags)
        return result

    # -- iterator interface --

    def __iter__(self):
        self.__iter_index = 0
        return self

    def next(self):
        if self.__iter_index == len(self.__titles):
            raise StopIteration

        result = self.__titles[self.__iter_index]
        self.__iter_index += 1
        return result

    def __len__(self):
        return len(self.__titles)

    def __getitem__(self, index):
        return self.__titles[index]

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
