# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from container_stream import Stream, TYPE_AUDIO
from winkirip.lib import sec2str
from copy import deepcopy

AUDIO_UNKNOWN = -1
AUDIO_PCM = 0
AUDIO_AC3 = 1
AUDIO_OGG = 2
AUDIO_MP2 = 3
AUDIO_MP3 = 4
AUDIO_FLAC = 5
AUDIO_WMA = 6
AUDIO_AAC = 7
AUDIO_QTIME = 8
AUDIO_DTS = 9
AUDIO_MOBILE = 10 # pcm actually (quck hack for mobile phones)

# -- implementation --

class Audio(Stream):
    def __init__(self, src = None):
        self.__format = AUDIO_UNKNOWN
        self.__language = 'xx'
        self.__frequency = 0
        self.__kbits = 0
        self.__number_channels = 0
        self.__length = 0
        self.__filename = None
        self.__id = -1
        self.__title_id = -1
        self.__container_id = -1
        self.__descr = None
        self.__source = None

        if src:
            self.__format = src.__format
            self.__language = deepcopy(src.__language)
            self.__frequency = src.__frequency
            self.__kbits = src.__kbits
            self.__number_channels = src.__number_channels
            self.__length = src.__length
            self.__filename = deepcopy(src.__filename)
            self.__id = src.__id
            self.__title_id = src.__title_id
            self.__container_id = src.__container_id
            self.__descr = deepcopy(src.__descr)
        return

    # -- methods inherited from Stream --

    def stream_type(self):
        return TYPE_AUDIO

    # -- get/set --

    def get_format(self):
        return self.__format

    def get_language(self):
        return self.__language

    def get_frequency(self):
        return self.__frequency

    def get_kbits(self):
        return self.__kbits

    def get_number_channels(self):
        return self.__number_channels

    def get_length(self):
        return self.__length

    def get_filename(self):
        return self.__filename

    def get_id(self):
        return self.__id

    def get_title_id(self):
        return self.__title_id

    def get_container_id(self):
        return self.__container_id

    def get_descr(self):
        return self.__descr

    def get_source(self):
        return self.__source

    def set_format(self, format):
        self.__format = format
        return

    def set_language(self, language):
        self.__language = language
        return

    def set_frequency(self, frequency):
        self.__frequency = int(frequency)
        return

    def set_kbits(self, kbits):
        self.__kbits = kbits
        return

    def set_number_channels(self, number_channels):
        self.__number_channels = number_channels
        return

    def set_length(self, length):
        self.__length = length
        return

    def set_filename(self, filename):
        self.__filename = filename
        return

    def set_id(self, id):
        self.__id = id
        return

    def set_title_id(self, title_id):
        self.__title_id = title_id
        return

    def set_container_id(self, container_id):
        self.__container_id = container_id
        return

    def set_descr(self, descr):
        self.__descr = descr
        return

    def set_source(self, source):
        if isinstance(source, Audio):
            self.__source = source
        else:
            assert(False)
        return

    def reset_source(self):
        self.__source = None
        return

    # -- public interface --

    def copy(self):
        return Audio(self)

    def get_formatted_length(self):
        return sec2str(self.__length)

    def format2str(self):
        result = 'dat'

        if self.__format == AUDIO_PCM:
            result = 'pcm'
        elif self.__format == AUDIO_AC3:
            result = 'ac3'
        elif self.__format == AUDIO_OGG:
            result = 'ogg'
        elif self.__format == AUDIO_MP2:
            result = 'mp2'
        elif self.__format == AUDIO_MP3:
            result = 'mp3'
        elif self.__format == AUDIO_FLAC:
            result = 'flac'

        return result

    def __repr__(self):
        result = '    Audio(fname: %s, id: %d, cid: %d, tid: %d)\n' % (self.__filename, self.__id, self.__container_id, self.__title_id)
        result += '      format: %d, lang: %s, freq: %d, kbits: %d, channels: %d, len: %d, descr: %s\n' % (self.__format, self.__language, self.__frequency, self.__kbits, self.__number_channels, self.__length, self.__descr)
        result += '     SOURCE: %s\n' % self.__source
        return result

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
