# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


WRITE_CDR = 1
WRITE_CDRW = 2
READ_DVD = 4
WRITE_DVDR = 8
WRITE_DVDRAM = 16

class DeviceInfo:
    def __init__(self):
        self.__device_file = None
        self.__speed = -1
        self.__capabilities = 0
        self.__manufacturer = None
        self.__model = None
        self.__version = None
        return

    def set_device_file(self, device_file):
        self.__device_file = device_file
        return

    def set_speed(self, speed):
        self.__speed = speed
        return

    def set_capability(self, capability):
        self.__capabilities |= capability
        return

    def set_capabilities(self, capabilities):
        self.__capabilities = capabilities
        return

    def set_manufacturer(self, manufacturer):
        self.__manufacturer = manufacturer
        return

    def set_model(self, model):
        self.__model = model
        return

    def set_version(self, version):
        self.__version = version
        return

    def get_device_file(self):
        return self.__device_file

    def get_full_device_file(self):
        return ('/dev/%s' % self.__device_file)

    def get_speed(self):
        return self.__speed

    def get_speed_string(self):
        return '%d' % self.__speed

    def get_manufacturer(self):
        return self.__manufacturer

    def get_capabilities(self):
        return self.__capabilities

    def get_capabilities_string(self):
        return '%d' % self.__capabilities

    def get_model(self):
        return self.__model

    def get_version(self):
        return self.__version

    def is_dvd_reader(self):
        return (self.__capabilities & READ_DVD)
 
    def __repr__(self):
        result = 'dev_file: %s\n' % self.__device_file
        result += 'speed: %d\n' % self.__speed
        result += 'caps: %d\n' % self.__capabilities
        result += 'manuf: %s\n' % self.__manufacturer
        result += 'model: %s\n' % self.__model
        result += 'version: %s\n' % self.__version
        return result

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
