# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# all sizes should be specified in bytes
# all times - in seconds

from winkirip.container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_FILE
from math import ceil, floor

class Estimation:
    def __init__(self, target_size = 0):
        self.__occupied = 0
        self.__seconds = 0
        self.__penalty_ratio = 0.0
        self.__target = target_size
        self.__ctype = CONTAINER_UNKNOWN
        return

    # -- public interface --

    def estimate_vbitrate(self):
        if self.__seconds <= 0 or self.__penalty_ratio <= 0.0:
            return -1

        penalty = self.__target * self.__penalty_ratio
        result = ((self.__target - self.__occupied - penalty) / self.__seconds / 125)

        if self.__ctype == CONTAINER_DVD and result > 9800:
            result = 9800
        elif self.__ctype == CONTAINER_FILE and result > 16000:
            result = 16000

        return result

    def estimate_number_targets(self, chunk_size, pref_bitrate, min_bitrate):
        penalty = chunk_size * self.__penalty_ratio

        if (chunk_size - penalty) <= 0:
            return 1

        chunks = (125 * pref_bitrate * self.__seconds + self.__occupied) / (chunk_size - penalty)

        # test the result
        target = chunk_size * int(chunks)
        penalty = target * self.__penalty_ratio

        if self.__seconds == 0:
            return max(int(chunks), 1)

        vbitrate = (target - self.__occupied - penalty) / self.__seconds / 125
        if vbitrate > min_bitrate:
            chunks = floor(chunks)
        else:
            chunks = ceil(chunks)

        return max(int(chunks), 1)

    def add_occupied(self, occupied):
        self.__occupied += occupied
        return

    def add_time(self, seconds):
        self.__seconds += seconds
        return

    def set_occupied(self, occupied):
        self.__occupied = occupied
        return

    def set_time(self, seconds):
        self.__seconds = seconds
        return

    def set_target(self, target):
        self.__target = target
        return

    def set_penalty_ratio(self, penalty_ratio):
        self.__penalty_ratio = penalty_ratio
        return

    def set_container_type(self, ctype):
        self.__ctype = ctype
        return

    def correct_bitrate(self, vbitrate, kind):
        max_rate = 16000
        if kind == 'svcd':
            max_rate = 2600
        elif kind == 'vcd':
            max_rate = 1152
        elif kind == 'dvd':
            max_rate = 9800

        return min(max_rate, vbitrate)

    def reset(self):
        self.__occupied = 0
        self.__seconds = 0
        self.__penalty_ratio = 0.0
        self.__target = 0
        return

    # -- helpers --

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
