# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.factory_base import BaseFactory
from winkirip.worker_audio import WorkerPCM, WorkerCopy
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_audio import AUDIO_PCM, AUDIO_MOBILE
from winkirip.alert import AlertError
from winkirip.lib import debug, error

class FactoryAudio(BaseFactory):
    def __init__(self, project):
        BaseFactory.__init__(self, project)
        return

    def create_worker(self, context, source_cid):
        cfg = self.get_project().get_config()
        dest_cont_type = self.get_project().get_dest_type()
        alist = self.get_audio_list(source_cid)

        worker = None
        if dest_cont_type == CONTAINER_DVD:
            self.__check_dvd_audio_presence(alist)
                
            aformat = cfg.get_audio_format_dvd()

            if self.get_force_pcm(alist, aformat, CONTAINER_DVD):
                worker = WorkerPCM(context, source_cid)

            else:
                worker = WorkerCopy(context, source_cid)

        elif dest_cont_type == CONTAINER_FILE:
            aformat = cfg.get_audio_format_file()

            # this is a nice short cut but...
            # just copying the stream does not work wit a lot of crappy encoded files
            #if self.get_force_pcm(alist, aformat, CONTAINER_FILE):
            #    worker = WorkerPCM(context, source_cid)
            #else:
            #    worker = WorkerCopy(context, source_cid)
            worker = WorkerPCM(context, source_cid)

        return worker

    # -- helpers --

    def __check_dvd_audio_presence(self, alist):
        if len(alist) < 1:
            primary = _('No audio selected')
            secondary = _('You decided to create a DVD without audio. However the program dvdauthor used to create the DVD does not like this constelation. The rip will most probably fail.')
            dlg_error = AlertError()
            dlg_error.show(primary, secondary, modal = False)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
