# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.factory_base import BaseFactory
from winkirip.worker_audio_compress import WorkerOGG, WorkerMP3, WorkerAAC
from winkirip.worker_audio_compress import WorkerAC3, WorkerMP2
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_audio import AUDIO_AC3, AUDIO_OGG, AUDIO_MP2
from winkirip.container_audio import AUDIO_MP3, AUDIO_AAC
from winkirip.lib import debug, error

class FactoryAudioCompress(BaseFactory):
    def __init__(self, project):
        BaseFactory.__init__(self, project)
        return

    def create_worker(self, context, source_cid):
        cfg = self.get_project().get_config()
        dest_cont_type = self.get_project().get_dest_type()
        alist = self.get_audio_list(source_cid)

        worker = None
        if dest_cont_type == CONTAINER_DVD:
            aformat = cfg.get_audio_format_dvd()
            
            # if the audio has been just copied no compression is needed
            if not self.get_force_pcm(alist, aformat, CONTAINER_DVD):
                return None

            if aformat == AUDIO_AC3:
                worker = WorkerAC3(context, source_cid)

            elif aformat == AUDIO_MP2:
                worker = WorkerMP2(context, source_cid)

            else:
                debug('FactoryAudioCompress.create_worker()', 'unexpected dvd format requested: %d' % aformat, error)

        elif dest_cont_type == CONTAINER_FILE:
            aformat = cfg.get_audio_format_file()

            if aformat == AUDIO_AC3:
                worker = WorkerAC3(context, source_cid)

            elif aformat == AUDIO_MP2:
                worker = WorkerMP2(context, source_cid)

            elif aformat == AUDIO_OGG:
                worker = WorkerOGG(context, source_cid)

            elif aformat == AUDIO_MP3:
                worker = WorkerMP3(context, source_cid)

            elif aformat == AUDIO_AAC:
                worker = WorkerAAC(context, source_cid)

            else:
                debug('FactoryAudioCompress.create_worker()', 'unexpected file format requested: %d' % aformat, error)

        return worker

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
