# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.container import CONTAINER_DVD
from winkirip.container_audio import AUDIO_PCM, AUDIO_MOBILE
from winkirip.lib import debug, error

class BaseFactory:
    def __init__(self, project):
        self.__project = project
        return

    # -- common functions for the derived classes --

    def get_project(self):
        return self.__project

    def get_force_pcm(self, alist, aformat, ctype):
        cfg = self.__project.get_config()
        dest_titles = self.__project.get_dest()
        source_titles = self.__project.get_source()

        for audio in alist:
            # dest format is pcm
            if aformat == AUDIO_PCM or aformat == AUDIO_MOBILE:
                return True

            # format conversion forces intermediate pcm file
            if aformat != audio.get_format():
                return True

            # DVD chapter ripping: without going over pcm only the audio stream
            # for first chapter containes valid ac3 data
            if ctype == CONTAINER_DVD and cfg.get_chapter_rip():
                return True

            # crude approach: if the source title should be splitted
            src = audio.get_source()
            source_title = self.__get_title(source_titles, src.get_container_id(), src.get_title_id())
            dest_title = self.__get_title(dest_titles, audio.get_container_id(), audio.get_title_id())
            source_chapters = source_title.get_chapters()
            dest_chapters = dest_title.get_chapters()
            if len(source_chapters) != len(dest_chapters):
                return True

        return False

    def get_audio_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.__project.get_dest()
        for container in dest:
            for title in container:
                audio_list = title.get_audio()
                for audio in audio_list:
                    source_audio = audio.get_source()
                    source_id = source_audio.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(audio)

        return result

    def get_subtitle_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.__project.get_dest()
        for container in dest:
            for title in container:
                sutbtile_list = title.get_subtitles()
                for subtitle in sutbtile_list:
                    source_subtitle = subtitle.get_source()
                    source_id = source_subtitle.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(subtitle)

        return result

    # -- helpers --

    def __get_title(self, containers, cid, tid):
        clist = filter(lambda container: container.get_id() == cid, containers)
        if len(clist) < 1:
            debug('FactoryAudio.__get_dest_title()', 'cannot find container with id %d' % cid, error)
            return None

        tlist = filter(lambda title: title.get_id() == tid, clist[0])
        if len(tlist) < 1:
            debug('FactoryAudio.__get_dest_title()', 'cannot find title with id %d in cont %d' % (tid, cid), error)
            return None

        return tlist[0]


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
