# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.factory_base import BaseFactory
from winkirip.worker_multiplex import WorkerDVDAudioVideo, WorkerDVDSubtitles
from winkirip.worker_multiplex import WorkerAVI, WorkerMKV, WorkerMP4, WorkerMobile
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE, FILE_AVI
from winkirip.container import FILE_MATROSKA, FILE_MP4, FILE_MOBILE
from winkirip.lib import debug, error

class FactoryMultiplex(BaseFactory):
    def __init__(self, project):
        BaseFactory.__init__(self, project)
        return

    def create_workers(self, context):
        cfg = self.get_project().get_config()
        dest_cont_type = self.get_project().get_dest_type()
        dest_containers = self.get_project().get_dest()
        result = []

        for dest_cont in dest_containers:
            dest_cid = dest_cont.get_id()

            if dest_cont_type == CONTAINER_DVD:
                worker = WorkerDVDAudioVideo(context, dest_cid)
                result.append(worker)

                has_subtitles = False
                for dest_title in dest_cont:
                    slist = dest_title.get_subtitles()
                    if len(slist) > 0:
                        has_subtitles = True
                        break

                if has_subtitles:
                    worker = WorkerDVDSubtitles(context, dest_cid)
                    result.append(worker)

            elif dest_cont_type == CONTAINER_FILE:
                file_encoding = cfg.get_file_encoding()

                if file_encoding == FILE_AVI:
                    worker = WorkerAVI(context, dest_cid)
                    result.append(worker)

                elif file_encoding == FILE_MATROSKA:
                    worker = WorkerMKV(context, dest_cid)
                    result.append(worker)

                elif file_encoding == FILE_MP4:
                    worker = WorkerMP4(context, dest_cid)
                    result.append(worker)

                elif file_encoding == FILE_MOBILE:
                    worker = WorkerMobile(context, dest_cid)
                    result.append(worker)

                else:
                    debug('FactoryMultiplex.create_workers()', 'unknown dest file encoding', error)

            else:
                debug('FactoryMultiplex.create_workers()', 'unknown dest container type', error)

        return result


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
