# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker_video import WorkerFFmpeg4, WorkerXvid, WorkerH264
from winkirip.worker_video import WorkerMpeg2DVD, WorkerMobile, WorkerTcextract, WorkerTcrequant
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_video import VIDEO_MPEG4, VIDEO_XVID, VIDEO_H264
from winkirip.container_video import VIDEO_MPEG1, VIDEO_MPEG2, VIDEO_MOBILE, VIDEO_DIVX
from winkirip.lib import to_int

class FactoryVideo:
    def __init__(self, project):
        self.__project = project
        self.__shrink = {'requantize' : -1 , '1 pass' : 1, '2 pass' : 2}
        self.__shrink_method = -1
        return

    def create_worker(self, context, source_cid):
        cfg = self.__project.get_config()
        dest_cont_type = self.__project.get_dest_type()

        vlist = self.__get_video_list(source_cid)
        for video in vlist:
            cur_method = self.__get_shrink_method(video, dest_cont_type, cfg)
            if cur_method > self.__shrink_method:
                self.__shrink_method = cur_method

        if self.__shrink_method < 0:
            return WorkerTcextract(context, source_cid)

        worker = None
        if dest_cont_type == CONTAINER_DVD:
            vformat = cfg.get_video_format_dvd()

            if vformat == VIDEO_MPEG2:
                worker = WorkerMpeg2DVD(context, source_cid, self.__shrink_method)

            # MPEG-1 still not implemented
            else:
                pass

        elif dest_cont_type == CONTAINER_FILE:
            vformat = cfg.get_video_format_file()
                             
            # MPEG-1 still not implemented
            if vformat == VIDEO_MPEG1:
                pass

            elif vformat == VIDEO_MPEG2:
                worker = WorkerMpeg2DVD(context, source_cid, self.__shrink_method)

            elif vformat == VIDEO_MPEG4 or vformat == VIDEO_DIVX:
                worker = WorkerFFmpeg4(context, source_cid, self.__shrink_method)

            elif vformat == VIDEO_XVID:
                worker = WorkerXvid(context, source_cid, self.__shrink_method)

            elif vformat == VIDEO_H264:
                worker = WorkerH264(context, source_cid, self.__shrink_method)

            elif vformat == VIDEO_MOBILE:
                worker = WorkerMobile(context, source_cid, self.__shrink_method)

        return worker

    def create_shrinker(self, context, source_cid):
        if self.__shrink_method < 0:
            return WorkerTcrequant(context, source_cid)

        return self.create_worker(context, source_cid)

    def get_shrink_method(self):
        return self.__shrink_method

    # -- helpers --

    def __get_video_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.__project.get_dest()
        for container in dest:
            for title in container:
                video_list = title.get_video()
                for video in video_list:
                    source_video = video.get_source()
                    source_id = source_video.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(video)

        return result

    def __get_shrink_method(self, video, dest_cont_type, cfg):
        #source_video_type = video.get_source().get_type()

        shrink_method = '1 pass'

        if dest_cont_type == CONTAINER_DVD:
            shrink_method = cfg.get_video_shrink_method_dvd()

            # we can shrink mpeg1/2 streams only
            if video.get_type() != VIDEO_MPEG1 and video.get_type() != VIDEO_MPEG2 and shrink_method == 'requantize':
                shrink_method = '1 pass'

        elif dest_cont_type == CONTAINER_FILE:
            shrink_method = cfg.get_video_shrink_method_file()

        else:
            # todo
            assert(False)
            pass

        return self.__shrink[shrink_method]

    # todo: remove this function
    def __get_shrink_method_dvd(self, video, source_video_type, cfg):
        shrink_method = cfg.get_video_shrink_method_dvd()

        if shrink_method == 'requantize':
            resol = cfg.get_video_resolution_dvd()
            w, h = resol.split('x')
            tw = to_int(w)
            th = to_int(h)
            sw = video.get_width()
            sh = video.get_height()

            saspect = video.get_aspect()
            taspect = cfg.get_video_aspect_dvd()
            if taspect == 'original':
                taspect = saspect

            # we cannot requantize if resolution or aspect change should be
            # made
            if taspect != saspect or tw != sw or th != sh:
                shrink_method = '1 pass'

            # we can requantize only mpeg2 streams
            if source_video_type != VIDEO_MPEG2:
                shrink_method = '1 pass'

        return shrink_method

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
