# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import events_pending, main_iteration
from gtk.gdk import pixbuf_new_from_file
from winkirip.lib import build_path, winki_path, lower_strip, debug, error

class Lang:
    def __init__(self):
        self.__iso_639_1 = None
        self.__iso_639_2 = []
        self.__name = None
        self.__flag = None
        return

    # -- get/set  --

    def get_iso_639_1(self):
        return self.__iso_639_1

    def get_iso_639_2(self, all = False):
        if all:
            return self.__iso_639_2

        return self.__iso_639_2[0]

    def get_name(self):
        return self.__name

    def get_flag(self):
        return self.__flag

    def set_iso_639_1(self, iso_639_1):
        self.__iso_639_1 = iso_639_1
        return

    def set_iso_639_2(self, iso_639_2):
        self.__iso_639_2 = iso_639_2
        return

    def set_name(self, name):
        self.__name = name
        return

    def set_flag(self, flag):
        self.__flag = flag
        return

    def __repr__(self):
        return 'Name: %s | iso639-1: %s | iso639-2: %s' % (self.__name, self.__iso_639_1, self.__iso_639_2)

class ISOCodes:
    def __init__(self):
        self.__langs = []
        self.__empty = None
        self.__img_path = build_path([winki_path, 'glade', 'flags'])
        self.__data_file = build_path([winki_path, 'iso_flags.csv'])
        return

    # -- public interface --

    def load(self):
        lines = self.__load()
        if not lines:
            return False

        for line in lines:
            while events_pending():
                main_iteration(False)

            items = line.split('\t')
            if len(items) != 4:
                print items
                debug('ISOCodes.load()', 'unexpected file format', error)
                continue

            lang = Lang()
            lang.set_iso_639_2(items[0].split('/'))
            lang.set_iso_639_1(items[1])
            lang.set_name(items[2])
            flag_name = build_path([self.__img_path, items[3]])
            pixbuf = pixbuf_new_from_file(flag_name)
            lang.set_flag(pixbuf)
            self.__langs.append(lang)

        self.__empty = Lang()
        self.__empty.set_iso_639_2(['und'])
        self.__empty.set_iso_639_1('xx')
        self.__empty.set_name('Unknown')
        flag_name = build_path([self.__img_path, 'empty.png'])
        pixbuf = pixbuf_new_from_file(flag_name)
        self.__empty.set_flag(pixbuf)
        return True

    def find_iso_639_1(self, text):
        text = lower_strip(text)
        for lang in self.__langs:
            cur_lang = lower_strip(lang.get_iso_639_1())
            if cur_lang == text:
                return lang
        return self.__empty

    def find_iso_639_2(self, text):
        text = lower_strip(text)
        get_all = True
        for lang in self.__langs:
            cur_langs = lang.get_iso_639_2(get_all)
            for cur_lang in cur_langs:
                if lower_strip(cur_lang) == text:
                    return lang
        return self.__empty

    def find_name(self, text):
        text = lower_strip(text)
        for lang in self.__langs:
            cur_name = lower_strip(lang.get_name())
            if cur_name == text:
                return lang
        return self.__empty

    # -- helpers --

    def __load(self):
        raw = None
        try:
            f = open(self.__data_file, 'r')
            raw = f.read()
            f.close()
        except:
            debug('ISOCodes.__load()', 'cannot open %s' % self.__data_file, error)
            return None

        lines = raw.split('\n')
        lines.pop()

        return lines

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
