# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk.gdk import _2BUTTON_PRESS, keyval_name
from winkirip.lib import debug, error
from winkirip.page_base import BasePage
from winkirip.page_base_language import BasePageLanguage
from winkirip.page_base_two_trees import COLUMN_LANG_OBJECT
from winkirip.page_base_two_trees import COLUMN_LANG_PIXBUF, COLUMN_LANG_TITLE
from winkirip.container_audio import Audio

class PageAudio(BasePage, BasePageLanguage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BasePageLanguage.__init__(self)
        return
        
    # -- methods inherited from BasePage --

    def get_icon(self):
        return 'page_audio.png'

    def get_title(self):
        return _('Audio Selection')

    def get_signal_list(self):
        return {'on_treeview_button_press_event': self.on_treeview_button_pressed,
                'on_treeview_key_press_event': self.on_treeview_key_pressed}

    def get_widget_list(self):
        return []

    def on_prepare(self):
        self.set_stoplight('go')

        if not self.get_needs_init():
            return True

        self.set_treeviews(self.get_widget('treeview_source'), self.get_widget('treeview_dest'))

        self.fill_dest_container(self.get_project())
        self.__init_treeview('treeview_source', _('Available audio languages'))

        self.reset_dest_container(self.get_project())
        self.__init_treeview('treeview_dest', _('Audio languages in use'))

        self.get_widget('treeview_source').grab_focus()

        self.set_needs_init(False)
        return True

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        return True

    def can_exit(self):
        return True

    def update_project(self):
        self.disable_buttons()

        treeview = self.get_widget('treeview_dest')
        model = treeview.get_model()

        if not self.__check_selected_streams(model):
            self.enable_buttons()
            return False

        if not self.sumbit_selected_streams(model, self.get_project().get_dest()):
            self.enable_buttons()
            return False

        self.get_druid().invalidate_after('audio')
        self.enable_buttons()
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- methods inherited from BaseTwoTreesPage --

    def get_title_streams(self, title):
        return title.get_audio()

    def add_title_stream(self, title, stream):
        title.add_audio(stream)
        return

    def reset_title_streams(self, title):
        title.reset_audio()
        return

    # -- signal handlers --

    def on_treeview_key_pressed(self, treeview, event):
        if keyval_name(event.keyval) != 'space' and keyval_name(event.keyval) != 'Return':
            return False

        # space or return pressed
        self.move_selected(treeview)
        self.get_druid().invalidate_after('audio')
        return True

    def on_treeview_button_pressed(self, treeview, event):
        result = False

        # double click with the left mouse button
        if event.button == 1 and event.type == _2BUTTON_PRESS:
            self.move_selected(treeview)
            self.get_druid().invalidate_after('audio')
            result = True

        # right click
        if event.button == 3:
            # defined only for the source tree
            # I suspect that the user might be confused if this operation is allowed
            # in the dest treeview: he might think that this represents some sort of conversion
            if treeview.get_name() == 'treeview_dest':
                return False

            result = False
            selection = treeview.get_selection()
            (model, pathlist) = selection.get_selected_rows()

            alist = []
            for path in pathlist:
                it = model.get_iter(path)
                obj = model.get_value(it, COLUMN_LANG_OBJECT)
                if not obj:
                    continue

                # append all selected audio streams
                if isinstance(obj, Audio):
                    alist.append(obj)

                # a title:
                # if only one title is selected: use all audio streams from it
                else:
                    if len(pathlist) == 1:
                        aiter = model.iter_children(it)
                        while aiter:
                            alist.append(model.get_value(aiter, COLUMN_LANG_OBJECT))
                            aiter = model.iter_next(aiter)
                        break

            vlist = []
            slist = []
            if len(alist) > 0:
                self.show_stream_properties(vlist, alist, slist, self.__update_tree_data)

        return result

    def on_drag_data_received(self, treeview, context, x, y, selection, info, timestamp):
        self.drag_data_receive(treeview, x, y, selection)
        self.get_druid().invalidate_after('audio')
        return True

    def on_drag_data_get(self, treeview, context, selection, info, timestamp):
        return self.drag_data_get(treeview, selection)

    # -- public interface --

    # -- helpers --

    def __init_treeview(self, treeview_name, treeview_title):
        treeview = self.get_widget(treeview_name)
        model = treeview.get_model()
        if not model:
            self.create_treeview(treeview, treeview_title)
            self.setup_dnd(treeview, self.on_drag_data_get, self.on_drag_data_received)
        else:
            model.clear()

        title_list = []
        for container in self.get_project().get_dest():
            for title in container:
                title_list.append(title)

        iso_codes = self.get_project().get_iso_codes()
        self.fill_model(title_list, treeview, iso_codes, _('Audio Stream %d'))
        return

    def __update_tree_data(self):
        treeview = self.get_widget('treeview_source')
        model = treeview.get_model()
        iso_codes = self.get_project().get_iso_codes()

        for row in model:
            title_iter = row.iter
            aiter = model.iter_children(title_iter)
            while aiter:
                self.__update_tree_from_object(aiter, model, iso_codes)
                aiter = model.iter_next(aiter)
        return

    def __update_tree_from_object(self, aiter, model, iso_codes):
        audio = model.get_value(aiter, COLUMN_LANG_OBJECT)
        lang = iso_codes.find_iso_639_1(audio.get_language())

        model.set_value(aiter, COLUMN_LANG_PIXBUF, lang.get_flag())
        model.set_value(aiter, COLUMN_LANG_TITLE, lang.get_name())
        return

    def __check_selected_streams(self, model):
        for row in model:
            if not model.iter_children(row.iter):
                title = model.get_value(row.iter, COLUMN_LANG_OBJECT)
                primary = _('No audio for "%s" selected') % title.get_name()
                secondary = _('You won\'t be able to hear sound in this title if you choose to continue')
                affirmative = _('Continue without audio')
                result = self.show_alert(primary, secondary, affirmative)
                if not result:
                    return False

        return True


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
