# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
import gettext
from winkirip.lib import build_path, winki_path, debug, error

class BaseFileSelectorPage:
    def __init__(self):
        self.__dlg = None
        self.__callback_func = None
        self.__data = None
        self.__filters = {}
        self.__select_multiple = False

        xml = self.__setup()
        if not xml:
            return

        self.__dlg = xml.get_widget('fileselection')
        self.__setup_filters()
        return

    # -- signal handlers --

    def no_quit(self, button, event = None):
        return True

    def on_ok(self, widget, event = None):
        path = ''
        if self.__select_multiple:
            path = self.__dlg.get_filenames()
        else:
            path = self.__dlg.get_filename()

        if not self.__callback_func:
            assert(False)
            return True

        self.__hide()

        self.__callback_func(path, self.__data)
        return True

    def on_cancel(self, widget, event = None):
        self.__hide()
        return True


    # -- public interface

    def show_file_selection(self, file_name, title, filter_list, action, callback_func, data, select_multiple = False):
        if title and len(title) > 0:
            self.__dlg.set_title(title)

        if file_name and len(file_name) > 0:
            self.__dlg.set_filename(file_name)

        self.__set_filter(filter_list)
        self.__dlg.set_action(action)

        self.__select_multiple = select_multiple
        self.__dlg.set_select_multiple(select_multiple)

        self.__callback_func = callback_func
        self.__data = data

        self.__dlg.show()
        self.__dlg.set_modal(True)
        return True

    # -- helpers --

    def __setup(self):
        appname = 'winki'
        i18ndir = build_path([winki_path, 'i18n'])
        gettext.bindtextdomain(appname, i18ndir)
        gettext.textdomain(appname)
        gettext.install(appname, i18ndir)

        xml = None
        try:
            gtk.glade.bindtextdomain(appname, i18ndir)
            gtk.glade.textdomain(appname)
            xml_path = build_path([winki_path, 'glade', 'file_selector.glade'])
            xml = gtk.glade.XML(xml_path, 'fileselection', gettext.textdomain())
        except:
            debug('BaseFileSelectorPage.__setup()', 'cannot load glade UI file', error)
            return None

        signals = {'on_ok_button_fileselection_clicked' : self.on_ok,
                   'on_cancel_button_fileselection_clicked' : self.on_cancel}
        xml.signal_autoconnect(signals)

        dlg = xml.get_widget('fileselection')
        dlg.connect('delete_event', self.no_quit)

        return xml

    def __hide(self):
        if not self.__dlg:
            return False

        self.__dlg.hide()
        return True

    def __setup_filters(self):
        self.__filters[_('All files')] = ['*']
        self.__filters[_('Multimedia files')] = ['*.mpeg', '*.avi', '*.mkv', '*.mpg', '*.ogg', '*.ogm', '*.wmv', '*.vob', '*.m2v', '*.mp4', '*.264', '*.h264', '*.x264', '*.3gp', '*.mov', '*.MPEG', '*.AVI', '*.MKV', '*.MPG', '*.OGG', '*.OGM', '*.WMV', '*.VOB', '*.M2V', '*.MP4', '*.H264', '*.X264', '*.3GP', '*.MOV']
        self.__filters[_('Image files')] = ['*.iso', '*.udf', '*.dat', '*.ISO', '*.UDF', '*.DAT']
        self.__filters[_('Subtitle files')] = ['*.srt', '*.sub', '*.idx', '*.SRT', '*.SUB', '*.IDX']
        self.__filters[_('Folders')] = ['']
        return  

    def __create_filter(self, filter_name):
        ffilter = gtk.FileFilter()
        ffilter.set_name(filter_name)

        try:
            for pattern in self.__filters[filter_name]:
                ffilter.add_pattern(pattern)
        except KeyError:
            print 'filter "%s" not available' % filter_name
            return None

        return ffilter

    def __set_filter(self, filter_list):
        available_filter = self.__dlg.list_filters()
        for available in available_filter:
            self.__dlg.remove_filter(available)

        first_filter = None
        for desired in filter_list:
            ffilter = self.__create_filter(desired)
            if not ffilter:
                continue

            if not first_filter:
                first_filter = ffilter

            self.__dlg.add_filter(ffilter)

        if first_filter:
            self.__dlg.set_filter(first_filter)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
