# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.page_base_two_trees import BaseTwoTreesPage, COLUMN_LANG_PIXBUF
from winkirip.page_base_two_trees import COLUMN_LANG_TITLE, COLUMN_LANG_OBJECT
from winkirip.container_title import Title
from winkirip.lib import treepath_string_to_tuple, debug, error

class BasePageLanguage(BaseTwoTreesPage):

    def __init__(self):
        BaseTwoTreesPage.__init__(self)
        return
        
    # -- methods inherited from page_base_two_trees.BaseTwoTreesPage --

    def is_tree(self):
        return False

    def is_language(self):
        return True

    def validate_move(self, source_tv_name, source_path, dest_tv_name, dest_path, source_items):
        source_treeview = self.get_dest_treeview()
        if self.get_source_treeview().get_name() == source_tv_name:
            source_treeview = self.get_source_treeview()
        source_model = source_treeview.get_model()

        source_iter = source_model.get_iter(source_path)
        source_object = source_model.get_value(source_iter, COLUMN_LANG_OBJECT)

        # titles could not be rearanged or moved
        if isinstance(source_object, Title):
            return False

        # dropping into the other treeview is always permitted
        if source_tv_name != dest_tv_name:
            return True

        # rearrange is possible only inside a title
        return self.__same_title(source_path, dest_path)

    def get_dest_iter(self, source_iter, source_model, dest_model, dest_pos, dest_path):
        parent_iter = source_model.iter_parent(source_iter)
        if not parent_iter:
            return None, None

        source_title = source_model.get_value(parent_iter, COLUMN_LANG_OBJECT)

        for dest_row in dest_model:
            dest_title = dest_row[COLUMN_LANG_OBJECT]

            # destination title found
            if dest_title.get_id() == source_title.get_id():
                if not dest_pos:
                    return dest_pos, dest_row.iter
                else:
                    return dest_pos, dest_model.get_iter(dest_path)

        debug('BasePageLanguage.__get_dest_iter_language()', 'no dest iter found', error)
        return None, None

    def get_values(self, model, iter, copy_object):
        text = model.get_value(iter, COLUMN_LANG_TITLE)
        obj = model.get_value(iter, COLUMN_LANG_OBJECT)
        pixbuf = model.get_value(iter, COLUMN_LANG_PIXBUF)
        return [pixbuf, text, obj]
        
    # -- helpers --

    def __same_title(self, source_path, dest_path):
        source_tuple = source_path
        if not isinstance(source_tuple, tuple):
            source_tuple = treepath_string_to_tuple(source_path)

        dest_tuple = dest_path
        if not isinstance(dest_tuple, tuple):
            dest_tuple = treepath_string_to_tuple(dest_path)

        if len(source_tuple) != len(dest_tuple):
                return False

        # both paths point to the same title if all but the last elements equal
        for i in range (len(source_tuple) - 1):
            if source_tuple[i] != dest_tuple[i]:
                return False

        return True

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
