# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.page_base_two_trees import BaseTwoTreesPage, TITLE_PATH_LEN
from winkirip.page_base_two_trees import STREAM_PATH_LEN, COLUMN_TREE_LANG
from winkirip.page_base_two_trees import COLUMN_TREE_TYPE, COLUMN_TREE_TITLE
from winkirip.page_base_two_trees import COLUMN_TREE_OBJECT
from winkirip.container import Container
from winkirip.container_title import Title
from winkirip.container_video import Video
from winkirip.container_audio import Audio
from winkirip.container_subtitle import Subtitle
from winkirip.lib import treepath_string_to_tuple
from copy import deepcopy
from gtk import TREE_VIEW_DROP_BEFORE, TREE_VIEW_DROP_INTO_OR_BEFORE
from gtk import TREE_VIEW_DROP_INTO_OR_AFTER

class BasePageTree(BaseTwoTreesPage):

    def __init__(self):
        BaseTwoTreesPage.__init__(self)
        return
        
    # -- methods inherited from page_base_two_trees.BaseTwoTreesPage --

    def is_tree(self):
        return True

    def is_language(self):
        return False

    def validate_move(self, source_tv_name, source_path, dest_tv_name, dest_path, source_items):
        if not dest_path or not len(dest_path):
            return False

        source_treeview = self.get_dest_treeview()
        if self.get_source_treeview().get_name() == source_tv_name:
            source_treeview = self.get_source_treeview()
        source_model = source_treeview.get_model()

        source_iter = source_model.get_iter(source_path)
        source_object = source_model.get_value(source_iter, COLUMN_TREE_OBJECT)

        # containers could not be rearanged or moved
        if isinstance(source_object, Container):
            return False

        # dropping from dest into source treeview is not permitted
        if source_tv_name != dest_tv_name and source_tv_name == 'treeview_dest':
            return False

        # rearanging objects is the source treeview is not permitted
        if source_tv_name == dest_tv_name and source_tv_name == 'treeview_source':
            return False

        # rearanging titles is meaningfull only in the dest_treeview 
        if isinstance(source_object, Title) and source_tv_name != 'treeview_dest':
            return False

        # all the dragged objects have to have the same path length (should be from the same tree level)
        # with the exception that it is possible to drop streams over a title
        path_len = len(dest_path)
        for sitem in source_items:
            if not sitem or len(sitem) < 1:
                continue

            tv_name, srcpath = sitem.split('\t')
            spath = treepath_string_to_tuple(srcpath)
            if len(spath) != path_len and len(spath) != STREAM_PATH_LEN and path_len != TITLE_PATH_LEN:
                return False

        return True

    def get_dest_iter(self, source_iter, source_model, dest_model, dest_pos, dest_path):
        dest_iter = dest_model.get_iter(dest_path)
        dest_object = dest_model.get_value(dest_iter, COLUMN_TREE_OBJECT)
        if not dest_object:
            return None, None

        # don't allow streams to be dragged into a container
        if isinstance(dest_object, Container):
            return None, None

        # if the streams are dragged into a title - append them at the end
        if isinstance(dest_object, Title):
            return dest_pos, dest_iter

        if isinstance(dest_object, Audio) or isinstance(dest_object, Video) or isinstance(dest_object, Subtitle):
            if dest_pos == TREE_VIEW_DROP_BEFORE or TREE_VIEW_DROP_INTO_OR_BEFORE:
                dest_pos = TREE_VIEW_DROP_INTO_OR_BEFORE
            else:
                dest_pos = TREE_VIEW_DROP_INTO_OR_AFTER
            return dest_pos, dest_iter

        return None, None

    def get_values(self, model, iter, copy_object):
        lang = model.get_value(iter, COLUMN_TREE_LANG)
        ttype = model.get_value(iter, COLUMN_TREE_TYPE)
        text = model.get_value(iter, COLUMN_TREE_TITLE)
        o = model.get_value(iter, COLUMN_TREE_OBJECT)
        if not copy_object:
            obj = o
        else:
            obj = deepcopy(o)
            assert(not obj.get_source())
            obj.set_source(o)

        return [lang, ttype, text, obj]


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
