# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import TreeStore, CellRendererPixbuf, CellRendererText, TreeViewColumn
from gtk.gdk import pixbuf_new_from_file, Pixbuf
from gobject import TYPE_STRING, TYPE_PYOBJECT
import gettext
from winkirip.lib import build_path, winki_path
from winkirip.page_base import BasePage
from winkirip.container_title import Title


COLUMN_PIXBUF = 0
COLUMN_TITLE = 1
COLUMN_OBJECT = 2

class PageDescr(BasePage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_descr.png'

    def get_title(self):
        return _('Title and Chapter Names')

    def get_signal_list(self):
        return {}

    def get_widget_list(self):
        return []

    def on_prepare(self):
        self.set_stoplight('go')

        if not self.get_needs_init():
            return

        title_list = self.get_project().get_dest()[0]
        self.__init_treeview(title_list)

        self.set_needs_init(False)
        return True

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        return True

    def can_exit(self):
        return True

    def update_project(self):
        self.disable_buttons()

        title_list = self.get_project().get_dest()[0]
        for title in title_list:
            counter = 1
            for chapter in title.get_chapters():
                chapter_name = chapter.get_name()
                if not chapter_name or not len(chapter_name):
                    chapter_name = (_('Chapter %d') % counter)
                chapter.set_name(chapter_name)
                counter += 1

        self.enable_buttons()
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- signal handlers --

    def on_edit(self, renderer, path, text, model):
        it = model.get_iter(path)
        obj = model.get_value(it, COLUMN_OBJECT)

        if isinstance(obj, Title):
            title_name = ('<span weight="bold">%s</span>' % text)
            model.set(it, COLUMN_TITLE, title_name)
        else:
            chapter_name = ('<span weight="normal">%s</span>' % text)
            model.set(it, COLUMN_TITLE, chapter_name)

        obj.set_name(text)

        self.get_druid().invalidate_after('description')
        return True

    # -- public interface --

    # -- helpers --

    def __init_treeview(self, title_list):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()
        if not model:
            self.__create_treeview()
        else:
            model.clear()

        self.__fill_model(title_list)
        return

    def __fill_model(self, title_list):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()

        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        chapter_fname = build_path([winki_path, 'glade', 'media_chapter.png'])
        chapter_pixbuf = pixbuf_new_from_file(chapter_fname)

        for title in title_list:
            title_name = ('<span weight="bold">%s</span>' % title.get_name())
            iterator = model.append(None, [title_pixbuf, title_name, title])
            counter = 1
            for chapter in title.get_chapters():
                chapter_name = chapter.get_name()
                if not chapter_name or not len(chapter_name):
                    chapter_name = (_('Chapter %d') % counter)
                chapter_name = ('<span weight="normal">%s</span>' % chapter_name)
                model.append(iterator, [chapter_pixbuf, chapter_name, chapter])
                counter += 1

        treeview.expand_all()
        return

    def __create_treeview(self):
        model = TreeStore(Pixbuf, TYPE_STRING, TYPE_PYOBJECT)

        treeview = self.get_widget('treeview_titles')
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_PIXBUF)
        treeview.append_column(column)

        renderer = CellRendererText()
        renderer.set_property('editable', True)
        renderer.connect('edited', self.on_edit, model)
        column = TreeViewColumn(_('Title'), renderer, markup = COLUMN_TITLE)
        treeview.append_column(column)
        return

    def __enable_ui(self, enable):
        treeview = self.get_widget('treeview_titles')
        treeview.set_sensitive(enable)
        treeview.grab_focus()
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
