# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.page_base import BasePage
from winkirip.container import Container, CONTAINER_FILE, CONTAINER_STREAMS
from winkirip.container import CONTAINER_DVD, CONTAINER_VCD, CONTAINER_SVCD
from winkirip.container import CONTAINER_UNKNOWN
from winkirip.lib import debug, error, textcombo_setup, textcombo_get_value

COMBO_COLUMN_TEXT = 0
COMBO_COLUMN_FTYPE = 1

class PageEncoding(BasePage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        self.__has_container_merger = True
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_encoding.png'

    def get_title(self):
        return _('Encoding')

    def get_signal_list(self):
        return {'on_radiobutton_multiple_pass_toggled' : self.on_encoding_toggled,
                'on_radiobutton_copy_stream_toggled' : self.on_encoding_toggled,
                'on_radiobutton_vcd_toggled' : self.on_encoding_toggled,
                'on_radiobutton_dvd_toggled' : self.on_encoding_toggled,
                'on_radiobutton_svcd_toggled' : self.on_encoding_toggled
                }

    def get_widget_list(self):
        widgets = ['label_multi_pass', 'label_stream_copy', 'label_create_vcd',
        'label_create_svcd', 'radiobutton_copy_stream',
        'radiobutton_multiple_pass', 'radiobutton_vcd', 'radiobutton_svcd']
        return widgets

    def on_prepare(self):
        self.set_stoplight('go')

        if not self.get_needs_init():
            return True

        radio_multi = self.get_widget('radiobutton_multiple_pass')
        radio_copy = self.get_widget('radiobutton_copy_stream')
        radio_vcd = self.get_widget('radiobutton_vcd')
        radio_svcd = self.get_widget('radiobutton_svcd')
        radio_dvd = self.get_widget('radiobutton_dvd')

        cfg = self.get_project().get_config()
        enc = cfg.get_encoding()

        if enc == 'multi pass':
            radio_multi.set_active(True)
            radio_multi.grab_focus()
            self.on_encoding_toggled(radio_multi)

        elif enc == 'copy':
            radio_copy.set_active(True)
            radio_copy.grab_focus()
            self.on_encoding_toggled(radio_copy)

        elif enc == 'vcd':
            radio_vcd.set_active(True)
            radio_vcd.grab_focus()
            self.on_encoding_toggled(radio_vcd)

        elif enc == 'svcd':
            radio_svcd.set_active(True)
            radio_svcd.grab_focus()
            self.on_encoding_toggled(radio_svcd)

        elif enc == 'dvd':
            radio_dvd.set_active(True)
            radio_dvd.grab_focus()
            self.on_encoding_toggled(radio_dvd)

        else:
            # new encoding?
            assert(False)

        self.set_needs_init(False)

        return self.__check_tools()

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        return True

    def can_exit(self):
        return True

    def update_project(self):
        self.disable_buttons()

        self.get_project().reset_dest()
        self.get_druid().invalidate_after('encoding')

        radio_multi = self.get_widget('radiobutton_multiple_pass')
        radio_copy = self.get_widget('radiobutton_copy_stream')
        radio_vcd = self.get_widget('radiobutton_vcd')
        radio_svcd = self.get_widget('radiobutton_svcd')
        radio_dvd = self.get_widget('radiobutton_dvd')

        cfg = self.get_project().get_config()

        # todo: this assumes one source container only
        # todo: container destruction not obvious
        enc = ''
        media = Container()
        media.set_id(0)
        media.set_device(self.get_project().get_source()[0].get_device())

        if radio_multi.get_active() and radio_multi.get_property('sensitive'):
            if not self.__has_container_merger:
                self.alert_missing_tool(_('mkvmerge or avimerge'))
                self.enable_buttons()
                return False

            media.set_type(CONTAINER_FILE)
            enc = 'multi pass'

        elif radio_copy.get_active() and radio_copy.get_property('sensitive'):
            media.set_type(CONTAINER_STREAMS)
            enc = 'copy'

        elif radio_vcd.get_active() and radio_vcd.get_property('sensitive'):
            media.set_type(CONTAINER_VCD)
            enc = 'vcd'

        elif radio_svcd.get_active() and radio_svcd.get_property('sensitive'):
            media.set_type(CONTAINER_SVCD)
            enc = 'svcd'

        elif radio_dvd.get_active() and radio_dvd.get_property('sensitive'):
            media.set_type(CONTAINER_DVD)
            enc = 'dvd'

        if media.get_type() == CONTAINER_UNKNOWN:
            self.enable_buttons()
            return False

        self.get_project().add_dest(media)
        self.enable_buttons()

        cfg.set_encoding(enc)
        cfg.write()
        return True

    # -- signal handlers --

    def on_encoding_toggled(self, widget, event = None):
        self.get_druid().invalidate_after('encoding')
        self.get_project().reset_dest()
        return True

    # -- helpers --

    def __check_tools(self):
        tchecker = self.get_project().get_tool_checker()
        has_paldump = tchecker.has_tool('paldump')
        has_dvdauthor = tchecker.has_tool('dvdauthor')
        has_ffmpeg = tchecker.has_tool('ffmpeg')
        has_mp2enc = tchecker.has_tool('mp2enc')
        has_spuunmux = tchecker.has_tool('spuunmux')
        has_spumux = tchecker.has_tool('spumux')
        has_tcextract = tchecker.has_tool('tcextract')
        has_tcrequant = tchecker.has_tool('tcrequant')
        has_mplex = tchecker.has_tool('mplex')
        has_oggenc = tchecker.has_tool('oggenc')
        has_lame = tchecker.has_tool('lame')
        has_mkvmerge = tchecker.has_tool('mkvmerge')

        radio_dvd = self.get_widget('radiobutton_dvd')
        label_dvd = self.get_widget('label_dvd')
        radio_multi = self.get_widget('radiobutton_multiple_pass')
        label_multi = self.get_widget('label_multi_pass')
        #radio_vcd = self.get_widget('radiobutton_vcd')
        #radio_svcd = self.get_widget('radiobutton_svcd')
        #radio_copy = self.get_widget('radiobutton_copy_stream')

        if not has_paldump:
            radio_dvd.set_sensitive(False)
            label_dvd.set_sensitive(False)
            #self.alert_missing_tool('paldump')
            return False

        if not has_dvdauthor:
            radio_dvd.set_sensitive(False)
            label_dvd.set_sensitive(False)
            #self.alert_missing_tool('dvdauthor')
            return False

        if not has_ffmpeg and not has_mp2enc:
            radio_dvd.set_sensitive(False)
            label_dvd.set_sensitive(False)
            #self.alert_missing_tool('ffmpeg or mp2enc')
            return False

        if not has_spuunmux:
            radio_dvd.set_sensitive(False)
            label_dvd.set_sensitive(False)
            #self.alert_missing_tool('spuunmux')
            return False

        if not has_spumux:
            radio_dvd.set_sensitive(False)
            label_dvd.set_sensitive(False)
            #self.alert_missing_tool('spumux')
            return False

        if not has_tcextract:
            radio_dvd.set_sensitive(False)
            label_dvd.set_sensitive(False)
            #self.alert_missing_tool('tcextract')
            return False

        if not has_tcrequant:
            radio_dvd.set_sensitive(False)
            label_dvd.set_sensitive(False)
            #self.alert_missing_tool('tcrequant')
            return False

        if not has_mplex:
            radio_dvd.set_sensitive(False)
            label_dvd.set_sensitive(False)
            #radio_multi.set_sensitive(False)
            #self.alert_missing_tool('mplex')
            return False

        if not has_oggenc or not has_ffmpeg or not has_lame or not has_mp2enc:
            radio_multi.set_sensitive(False)
            label_multi.set_sensitive(False)
            #self.alert_missing_tool('oggenc or ffmpeg or lame or mp2enc')
            return False

        if not has_mkvmerge:
            radio_multi.set_sensitive(False)
            label_multi.set_sensitive(False)
            #self.alert_missing_tool('mkvmerge')
            return False

        return True

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
