# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import FILE_CHOOSER_ACTION_CREATE_FOLDER, TreeStore, CellRendererText, TreeViewColumn
from gobject import TYPE_STRING
import gettext
from winkirip.lib import normalize_path, build_path, fix_name, fix_fname
from winkirip.page_base import BasePage
from winkirip.page_base_fileselect import BaseFileSelectorPage
from winkirip.select_imdb import SelectIMDB
from winkirip.container import CONTAINER_FILE


class PageInfo(BasePage, BaseFileSelectorPage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BaseFileSelectorPage.__init__(self)
        self.__generated = True
        self.__out_folder = None
        self.__imdb_dialog = None
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_info.png'

    def get_title(self):
        return _('Movie Information')

    def get_signal_list(self):
        return {'on_entry_name_changed' : self.on_name_changed,
                'on_entry_output_file_changed' : self.on_output_dir_changed,
                'on_button_output_file_clicked' : self.on_fileselection_clicked,
                'on_button_imdb_clicked' : self.on_imdb_search_clicked,
                'on_checkbutton_manual_edit_toggled' : self.on_manual_edit_toggled,
                'on_button_title_clicked' : self.on_title_clicked,
                'on_button_director_clicked' : self.on_director_clicked,
                'on_button_artist_clicked' : self.on_artist_clicked,
                'on_button_actor_clicked' : self.on_actor_clicked,
                'on_button_composer_clicked' : self.on_composer_clicked,
                'on_button_lyrics_clicked' : self.on_lyrics_clicked,
                'on_button_written_by_clicked' : self.on_written_clicked,
                'on_button_content_type_clicked' : self.on_content_clicked,
                'on_button_genre_clicked' : self.on_genre_clicked,
                'on_button_summary_clicked' : self.on_summary_clicked,
                'on_button_date_released_clicked' : self.on_release_clicked,
                'on_button_delete_clicked' : self.on_delete_clicked}

    def get_widget_list(self):
        # todo: check for the buttons in the button box
        return ['label_name', 'label_output_file', 'entry_name',
        'button_imdb', 'entry_output_file', 'button_output_file',
        'treeview_info', 'vbuttonbox_tags', 'checkbutton_manual_edit']

    def on_prepare(self):
        self.__toggle_stoplight()

        if not self.get_needs_init():
            return

        treeview = self.get_widget('treeview_info')
        model = treeview.get_model()
        if model:
            model.clear()
        else:
            self.__create_model()

        self.__generated = True
        self.__out_folder = None

        sources = self.get_project().get_source()
        if len(sources) < 1:
            print len(sources)
            assert(False)

        movie_name = sources[0].get_name()

        entry_name = self.get_widget('entry_name')
        entry_name.set_text(movie_name)

        self.__name_changed()
        self.__update_ui(False)
        self.__toggle_stoplight()

        self.set_needs_init(False)

        entry_name.grab_focus()
        return True

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        return True

    def can_exit(self):
        return (not self.__imdb_dialog)

    def update_project(self):
        self.disable_buttons()

        if not self.__toggle_stoplight():
            self.enable_buttons()
            return False

        entry_output_dir = self.get_widget('entry_output_file')
        output_dir = entry_output_dir.get_text()

        entry_name = self.get_widget('entry_name')
        self.get_project().set_dest_dir(output_dir)
        dest = self.get_project().get_dest()
        tags = self.__get_tags()
        for container in dest:
            container.set_name(entry_name.get_text())
            container.set_tags(tags)

        self.get_druid().invalidate_after('info')
        self.enable_buttons()
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- signal handlers --

    def on_name_changed(self, widget, event = None):
        self.__name_changed()
        self.__toggle_stoplight()
        self.__set_app_title()

        self.get_druid().invalidate_after('info')
        return True

    def on_output_dir_changed(self, widget, event = None):
        self.__toggle_stoplight()

        entry_output_file = widget

        if entry_output_file.get_property('has-focus'):
            self.__generated = False

        return True

    def on_fileselection_clicked(self, widget, event = None):
        if not self.__out_folder:
            config = self.get_project().get_config()
            self.__out_folder = normalize_path(config.get_output_dir())

        title = _('Select output folder')
        filter_list = [_('Folders')]
        action = FILE_CHOOSER_ACTION_CREATE_FOLDER
        callback = self.__output_dir_selected
        return self.show_file_selection(self.__out_folder, title, filter_list, action, callback, None)

    def on_imdb_search_clicked(self, widget, event = None):
        if self.__imdb_dialog:
            # print warning/error
            return True

        entry_name = self.get_widget('entry_name')
        name = entry_name.get_text()

        tchecker = self.get_project().get_tool_checker()
        self.__imdb_dialog = SelectIMDB(tchecker)
        self.__imdb_dialog.run(name.strip())

        if self.__imdb_dialog.get_state() == 'select':
            model = self.__create_model()
            self.__fill_model(self.__imdb_dialog.get_data(), model)
            entry_name.set_text(self.__imdb_dialog.get_data()['title'][0])

        self.__imdb_dialog = None

        checkbutton = self.get_widget('checkbutton_manual_edit')
        self.__update_ui(checkbutton.get_active())
        return True

    def on_manual_edit_toggled(self, widget, event = None):
        checkbutton = self.get_widget('checkbutton_manual_edit')

        self.__update_ui(checkbutton.get_active())
        return True

    def on_edit_field(self, renderer, path, text, model):
        # skip top level (title, director, name, ...)
        items = path.split(':')
        if len(items) < 2:
            return

        it = model.get_iter(path)
        model.set(it, 0, text)

        self.get_druid().invalidate_after('info')
        return

    def on_title_clicked(self, widget, event = None):
        self.__add_model_item('title', _('Title'))
        return True

    def on_director_clicked(self, widget, event = None):
        self.__add_model_item('director', _('Director'))
        return True

    def on_artist_clicked(self, widget, event = None):
        self.__add_model_item('artist', _('Artist'))
        return True

    def on_actor_clicked(self, widget, event = None):
        self.__add_model_item('actor', _('Actor'))
        return True

    def on_composer_clicked(self, widget, event = None):
        self.__add_model_item('composer', _('Composer'))
        return True

    def on_lyrics_clicked(self, widget, event = None):
        self.__add_model_item('lyrics', _('Lyrics'))
        return True

    def on_written_clicked(self, widget, event = None):
        self.__add_model_item('written by', _('Author'))
        return True

    def on_content_clicked(self, widget, event = None):
        self.__add_model_item('content', _('Content type'))
        return True

    def on_genre_clicked(self, widget, event = None):
        self.__add_model_item('genre', _('Genre'))
        return True

    def on_summary_clicked(self, widget, event = None):
        self.__add_model_item('summary', _('Summary'))
        return True

    def on_release_clicked(self, widget, event = None):
        self.__add_model_item('release date', _('Release date'))
        return True

    def on_delete_clicked(self, widget, event = None):
        treeview = self.get_widget('treeview_info')
        selection = treeview.get_selection()
        model, iterator, = selection.get_selected()
        if not model or not iterator:
            primary = _('No tags selected')
            secondary = _('Select the tag which have to be removed from the list')
            self.show_error(primary, secondary)
            return True

        path = model.get_path(iterator)
        model.remove(iterator)

        selection.select_path(path)

        if not selection.path_is_selected(path):
            if len(path) > 1 and path[1] > 0:
                new_path = path[0], path[1] - 1
                selection.select_path(new_path)
                if not selection.path_is_selected(new_path):
                    model.remove(model.get_iter(path[0]))
            else:
                model.remove(model.get_iter(path[0]))

                row = path[0] - 1
                if row >= 0:
                    selection.select_path((row,))

        return True


    # -- public interface --

    # -- helpers --

    def __update_ui(self, manual_edit):
        button_imdb = self.get_widget('button_imdb')
        button_box = self.get_widget('vbuttonbox_tags')
        button_delete = self.get_widget('button_delete')

        button_imdb.set_sensitive(not manual_edit)

        for button in button_box.get_children():
            button.set_sensitive(manual_edit)

        file_dest = False
        containers = self.get_project().get_dest()
        for container in containers:
            if container.get_type() == CONTAINER_FILE: 
                file_dest = True
                break

        checkbutton = self.get_widget('checkbutton_manual_edit')
        treeview = self.get_widget('treeview_info')

        # only mkv-files can contain tags (dvds, vcds and streams cannot)
        if file_dest:
            checkbutton.set_sensitive(True)
            button_imdb.set_sensitive(True)
            treeview.set_sensitive(True)
        else:
            checkbutton.set_sensitive(False)
            button_imdb.set_sensitive(False)
            treeview.set_sensitive(False)

        # missing tools
        tchecker = self.get_project().get_tool_checker()
        if not tchecker.has_tool('title') or not tchecker.has_tool('lindex'):
            button_imdb.set_sensitive(False)

        selection = treeview.get_selection()
        model, iterator, = selection.get_selected()
        if not model or len(model) < 1 or not iterator:
            button_delete.set_sensitive(False)
        else:
            button_delete.set_sensitive(True)

        return

    def __set_app_title(self):
        entry_name = self.get_widget('entry_name')
        app_title = entry_name.get_text()

        self.get_druid().set_window_title(app_title)
        return

    def __name_changed(self):
        entry_name = self.get_widget('entry_name')
        entry_output_dir = self.get_widget('entry_output_file')

        movie_name = entry_name.get_text()
        if not self.__generated:
            return

        source = self.get_project().get_source()
        if len(source) < 1:
            print len(source)
            assert(False)
        source[0].set_name(movie_name)

        config = self.get_project().get_config()
        out_dir = config.get_output_dir()
        out_dir = build_path([out_dir, movie_name])

        if not out_dir or not len(out_dir):
            entry_output_dir.set_text('')
            return
            
        entry_output_dir.set_text(out_dir)
        return

    def __output_dir_selected(self, fname, data = None):
        if not fname:
            return

        self.__out_folder = fname

        entry_output_dir = self.get_widget('entry_output_file')
        entry_output_dir.set_text(fname)
        return

    def __toggle_stoplight(self):
        entry_output_dir = self.get_widget('entry_output_file')

        output_dir = entry_output_dir.get_text()
        config = self.get_project().get_config()
        default_output_dir = config.get_output_dir()

        result = False
        if normalize_path(output_dir) == normalize_path(default_output_dir):
            self.set_stoplight('stop')
        else:
            self.set_stoplight('go')
            result = True

        return result

    def __create_model(self):
        treeview = self.get_widget('treeview_info')
        model = treeview.get_model()
        if model:
            model.clear()
            return model

        model = TreeStore(TYPE_STRING)
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererText()
        renderer.set_property('editable', True)
        renderer.connect('edited', self.on_edit_field, model)
        column = TreeViewColumn(_('Movie Tags'), renderer, text = 0)
        treeview.append_column(column)
        return model

    def __fill_model(self, data, model):
        for key in data:
            iterator = model.append(None, [key])
            items = data[key]
            for item in items:
                model.append(iterator, [item])

        if len(data) > 0:
            treeview = self.get_widget('treeview_info')
            selection = treeview.get_selection()
            selection.select_path(0)
        return

    def __add_model_item(self, key, value):
        treeview = self.get_widget('treeview_info')
        model = treeview.get_model()
        selection = treeview.get_selection()

        for row in model:
            if key == row[0]:
                item_iter = model.append(row.iter, [value])
                treeview.expand_row(model.get_path(row.iter), True)
                selection.select_path(model.get_path(item_iter))
                return

        iterator = model.append(None, [key])
        item_iter = model.append(iterator, [value])
        treeview.expand_row(model.get_path(iterator), True)
        selection.select_path(model.get_path(item_iter))

        treeview.grab_focus()
        return

    def __get_tags(self):
        result = {}
        treeview = self.get_widget('treeview_info')
        model = treeview.get_model()
        if not model:
            return result

        for row in model:
            key = self.__translate_tag(row[0])
            value = []
            iterator = model.iter_children(row.iter)
            while iterator:
                value.append(model.get_value(iterator, 0))
                iterator = model.iter_next(iterator)
            result[key] = value

        return result

    def __translate_tag(self, tag):
        if tag == 'producers':
            result = 'PRODUCER'

        elif tag == 'directors':
            result = 'DIRECTOR'

        elif tag == 'writers':
            result = 'WRITTEN_BY'

        elif tag == 'composers':
            result = 'COMPOSER'

        elif tag == 'lyrics':
            result = 'LYRICIST'

        elif tag == 'cinematographers':
            result = 'DIRECTOR_OF_PHOTOGRAPHY'

        elif tag == 'editors':
            result = 'EDITED_BY'

        elif tag == 'cast':
            result = 'THANKS_TO'

        elif tag == 'countries':
            result = 'COUNTRY'

        elif tag == 'comment':
            result = 'COMMENT'

        # non-standard matroska tag
        elif tag == 'languages':
            result = 'LANGUAGE'

        else:
            result = tag.upper()

        return result.replace(' ', '_')

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
