# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.page_base import BasePage
from winkirip.builder_factory import BuilderFactory
from winkirip.lib import set_button_data, winki_path, build_path, debug, error
from gtk import STOCK_EXECUTE, STOCK_YES, STOCK_NO, STOCK_CONNECT, STOCK_DISCONNECT, STOCK_CANCEL
from gtk import STOCK_CDROM, STOCK_MEDIA_PLAY, ICON_SIZE_BUTTON, TreeStore, TreeViewColumn, Entry
from gtk import CellRendererText, CellRendererPixbuf, events_pending, main_iteration
from gtk.gdk import Pixbuf, pixbuf_new_from_file
from gobject import TYPE_STRING, TYPE_PYOBJECT
from time import sleep
from threading import Thread

STATE_UNKNOWN = -1
STATE_RUNNING = 0
STATE_CANCELED = 1
STATE_FAILED = 2

COLUMN_PIXBUF = 0
COLUMN_TEXT = 1
COLUMN_COMMAND = 2

class PageRip(BasePage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        self.__state = STATE_UNKNOWN
        self.__pulse_thread = None
        self.__last_progress_value = 0.
        self.__command = None
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_rip.png'

    def get_title(self):
        return _('Rip')

    def get_signal_list(self):
        return {'on_button_rip_clicked' : self.on_rip_clicked,
        'on_checkbutton_disable_update_toggled' : self.on_toggle_update_ui,
        'on_checkbutton_test_rip_toggled' : self.on_toggle_test_rip}

    def get_widget_list(self):
        return []

    def on_prepare(self):
        self.set_stoplight('empty')

        self.get_druid().enable_buttons(False, ['button_settings'])
        self.get_druid().show_buttons(False, ['button_next'])

        if not self.get_needs_init():
            return True

        self.__init_treeview()

        checkbutton_update = self.get_widget('checkbutton_disable_update')
        cfg = self.get_project().get_config()
        if not cfg.get_update_rip_ui():
            checkbutton_update.set_active(False)
        else:
            checkbutton_update.set_active(True)
        self.on_toggle_update_ui(checkbutton_update)

        checkbutton_test_rip = self.get_widget('checkbutton_test_rip')
        button_rip = self.get_widget('button_rip')
        pixbuf = button_rip.render_icon(STOCK_CDROM, ICON_SIZE_BUTTON)
        if not cfg.get_test_rip():
            checkbutton_test_rip.set_active(False)
            set_button_data(button_rip, _('_Rip'), pixbuf)
        else:
            checkbutton_test_rip.set_active(True)
            set_button_data(button_rip, _('_Test rip'), pixbuf)
        self.__enable_test_rip_button(True)
        return True

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        self.get_druid().enable_buttons(True, ['button_settings'])
        self.get_druid().show_buttons(True, ['button_next'])
        self.get_app().set_tray_icon('winki')
        return True

    def can_exit(self):
        return True

    def update_project(self):
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- signal handlers --

    def on_rip_clicked(self, widget, event = None):
        if self.__state == STATE_RUNNING:
            self.__abort(STATE_CANCELED)
        else:
            self.__start()

        return True

    def on_toggle_test_rip(self, checkbutton, event = None):
        cfg = self.get_project().get_config()

        button_rip = self.get_widget('button_rip')
        pixbuf = button_rip.render_icon(STOCK_CDROM, ICON_SIZE_BUTTON)

        if checkbutton.get_active():
            cfg.set_test_rip(True)
            set_button_data(button_rip, _('_Test rip'), pixbuf)
        else:
            cfg.set_test_rip(False)
            set_button_data(button_rip, _('_Rip'), pixbuf)

        cfg.write()
        return True

    def on_toggle_update_ui(self, checkbutton, event = None):
        progressbar = self.get_widget('progressbar_current')
        label_estimated_label = self.get_widget('label_estimated_time_label')
        label_elapsed_label = self.get_widget('label_elapsed_time_label')
        label_estimated = self.get_widget('label_estimated_time')
        label_elapsed = self.get_widget('label_elapsed_time')
        cfg = self.get_project().get_config()

        if checkbutton.get_active():
            pixbuf = checkbutton.render_icon(STOCK_CONNECT, ICON_SIZE_BUTTON)
            set_button_data(checkbutton, _('_User interface updated'), pixbuf)

            progressbar.set_sensitive(True)
            self.__start_progress()
            label_estimated.set_sensitive(True)
            if self.__command and not self.__command.get_can_estimate():
                label_estimated.set_text(_('No estimation possible'))

            label_elapsed.set_sensitive(True)
            label_estimated_label.set_sensitive(True)
            label_elapsed_label.set_sensitive(True)

            if self.__command:
                self.__command.set_update_ui(True)

            cfg.set_update_rip_ui(True)
        else:
            pixbuf = checkbutton.render_icon(STOCK_DISCONNECT, ICON_SIZE_BUTTON)
            set_button_data(checkbutton, _('_User interface not updated'), pixbuf)

            self.__stop_progress()
            progressbar.set_text('')
            progressbar.set_sensitive(False)
            label_estimated.set_text('')
            label_estimated.set_sensitive(False)
            label_elapsed.set_sensitive(False)
            label_estimated_label.set_sensitive(False)
            label_elapsed_label.set_sensitive(False)

            if self.__command:
                self.__command.set_update_ui(False)

            cfg.set_update_rip_ui(False)

        cfg.write()
        return True

    def on_start_edit_title(self, renderer, editable, path, model):
        if not path or len(path) < 2:
            return True

        if not isinstance(editable, Entry):
            debug('PageRip.on_start_edit_title()', 'Please send a bug report to velko@winki-the-ripper.de', error)
            debug('PageRip.on_start_edit_title()', 'The GNOME documentation warns about this case:', error)
            debug('PageRip.on_start_edit_title()', 'http://developer.gnome.org/doc/API/2.0/gtk/GtkCellRenderer.html#GtkCellRenderer-editing-started', error)
            return True

        obj = model[path][COLUMN_COMMAND]
        text = obj.get_command()
        if not text:
            return True

        editable.set_text(text)
        return True

    def on_edit_title(self, renderer, path, text, model):
        if not path or len(path) < 2:
            return True

        obj = model[path][COLUMN_COMMAND]
        obj.set_command(text)

        it = model.get_iter(path)
        model.set(it, COLUMN_TEXT, obj.get_ui_name())
        return True

    # -- public interface --

    # -- helpers --

    def __start(self):
        if self.__state == STATE_RUNNING:
            return

        self.get_app().set_tray_icon('winki')
        self.disable_buttons()
        self.__enable_test_rip_button(False)
        self.__clean_status_icons()

        success = True
        self.__state = STATE_RUNNING

        treeview = self.get_widget('treeview_status')
        model = treeview.get_model()

        for row in model:
            self.__ui_start_command(model, row.iter)

            worker = model.get_value(row.iter, COLUMN_COMMAND)
            if not self.__execute_group(worker):
                self.__ui_end_command(model, row.iter)
                success = False
                break

            self.__ui_end_command(model, row.iter)

        if success:
            self.get_app().set_tray_icon('winki_yes')
            self.__abort(STATE_UNKNOWN)
            self.__command = None
            self.__conditional_quit()

        self.__enable_test_rip_button(True)
        self.enable_buttons(['button_quit', 'button_back'])
        return (success and self.__state == STATE_UNKNOWN)

    def __execute_group(self, worker):
        success = True
        first = True
        can_progress = False

        for ui_command in worker.get_commands():
            self.__last_progress_value = 0.
            self.__command = ui_command

            if first:
                self.__start_progress()
                first = False
                can_progress = self.__command.get_can_progress()
            else:
                if can_progress != self.__command.get_can_progress():
                    can_progress = self.__command.get_can_progress()
                    self.__start_progress()

            self.__command.set_log_file('winki_execute.log')
            success = self.__run()

            if not success or self.__state == STATE_CANCELED or self.__state == STATE_FAILED:
                if self.__state != STATE_CANCELED:
                    self.__state = STATE_FAILED

                error_text = ''
                if self.__command:
                    error_text = self.__command.get_error()

                self.__command = None
                self.__abort(self.__state)

                if self.__state != STATE_CANCELED:
                    self.get_app().set_tray_icon('winki_no')
                    self.show_error(_('Execution failed'), error_text)
                    #self.__conditional_quit()
                    success = False

                break

            self.__command = None

        self.__stop_progress()
        return success

    def __abort(self, state):
        self.__state = state

        if state == STATE_CANCELED and self.__command:
            self.__command.cancel()

        button_rip = self.get_widget('button_rip')
        pixbuf = button_rip.render_icon(STOCK_CDROM, ICON_SIZE_BUTTON)
        set_button_data(button_rip, _('_Rip'), pixbuf)
        self.__enable_test_rip_button(True)
        self.enable_buttons(['button_quit', 'button_back'])
        return

    def __run(self):
        button_rip = self.get_widget('button_rip')
        pixbuf = button_rip.render_icon(STOCK_CANCEL, ICON_SIZE_BUTTON)
        set_button_data(button_rip, _('Abort _ripping'), pixbuf)

        if self.__command.get_can_estimate():
            self.__command.set_estimation_callback(self.__on_estimation)
        else:
            label_estimation = self.get_widget('label_estimated_time')
            label_estimation.set_text(_('No estimation possible'))

        checkbutton_update = self.get_widget('checkbutton_disable_update')
        self.__command.set_update_ui(checkbutton_update.get_active())

        self.__continue_progress()
        return self.__command.execute()

    def __start_progress(self):
        if not self.__command:
            return

        checkbutton_update = self.get_widget('checkbutton_disable_update')
        if not checkbutton_update.get_active():
            return

        if self.__pulse_thread:
            del self.__pulse_thread
            self.__pulse_thread = None

        if self.__command.get_can_progress():
            self.__command.set_progress_callback(self.__on_progress)

            progressbar = self.get_widget('progressbar_current')
            progressbar.set_fraction(0.)
            progressbar.set_text('%d%%' % self.__last_progress_value)
        else:
            self.__pulse_thread = Thread(group = None, target = self.__pulse)
            self.__pulse_thread.start()
        return

    def __continue_progress(self):
        if not self.__command:
            return

        checkbutton_update = self.get_widget('checkbutton_disable_update')
        if not checkbutton_update.get_active():
            return

        if self.__command.get_can_progress():
            self.__command.set_progress_callback(self.__on_progress)
        return

    def __stop_progress(self):
        if self.__command:
            if self.__command.get_can_progress():
                self.__last_progress_value = 0.
            else:
                if self.__pulse_thread:
                    del self.__pulse_thread
                    self.__pulse_thread = None

        progressbar = self.get_widget('progressbar_current')
        progressbar.set_fraction(0.)
        progressbar.set_text('')
        return

    def __on_progress(self, fract):
        checkbutton_update = self.get_widget('checkbutton_disable_update')
        if not checkbutton_update.get_active():
            return

        if fract > self.__last_progress_value and fract > 0. and fract < 1.:
            progressbar = self.get_widget('progressbar_current')
            progressbar.set_fraction(fract)
            progressbar.set_text('%d%%' % int(round(fract * 100)))
            self.__last_progress_value = fract
        return

    def __pulse(self):
        progressbar = self.get_widget('progressbar_current')
        progressbar.set_pulse_step(0.004)

        while self.__pulse_thread and self.__pulse_thread.isAlive():
            if self.__state != STATE_RUNNING:
                break

            while events_pending():
                main_iteration(False)
            sleep(0.09)
            progressbar.pulse()

        progressbar.set_fraction(0.0)
        if self.__pulse_thread:
            del self.__pulse_thread
            self.__pulse_thread = None
        return

    def __on_estimation(self, text):
        checkbutton_update = self.get_widget('checkbutton_disable_update')
        if not checkbutton_update.get_active():
            return

        if len(text) > 0:
            label_estimation = self.get_widget('label_estimated_time')
            label_estimation.set_text(text)
        return

    def __clean_status_icons(self):
        empty_fname = build_path([winki_path, 'glade', 'empty.png'])
        pixbuf = pixbuf_new_from_file(empty_fname)

        widget = self.get_widget('treeview_status')
        model = widget.get_model()
        for row in model:
            model.set_value(row.iter, COLUMN_PIXBUF, pixbuf)
        return

    def __create_treeview(self):
        model = TreeStore(Pixbuf, TYPE_STRING, TYPE_PYOBJECT)

        treeview = self.get_widget('treeview_status')
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn(_('Status'), renderer, pixbuf = COLUMN_PIXBUF)
        treeview.append_column(column)

        renderer = CellRendererText()

        cfg = self.get_project().get_config()
        mode = cfg.get_user_mode()
        if mode == 'Advanced':
            renderer.set_property('editable', True)
            renderer.connect('edited', self.on_edit_title, model)
            renderer.connect('editing-started', self.on_start_edit_title, model)
        column = TreeViewColumn(_('The plan'), renderer, markup = COLUMN_TEXT)
        treeview.append_column(column)
        return

    def __init_treeview(self):
        treeview = self.get_widget('treeview_status')
        model = treeview.get_model()
        if not model:
            self.__create_treeview()
        else:
            model.clear()

        self.__fill_model()
        return

    def __fill_model(self):
        treeview = self.get_widget('treeview_status')
        model = treeview.get_model()

        empty_fname = build_path([winki_path, 'glade', 'empty.png'])
        pixbuf = pixbuf_new_from_file(empty_fname)

        build_factory = BuilderFactory()
        builder = build_factory.create_builder(self.get_project())
        if not builder or not builder.build_workers():
            return

        workers = builder.get_workers()
        for worker in workers:
            group_name = ('<span weight="bold">%s</span>' % worker.get_ui_name())
            model.append(None, [pixbuf, group_name, worker])

        treeview.expand_all()
        return

    def __ui_start_command(self, model, iterator):
        widget = self.get_widget('treeview_status')
        pixbuf = widget.render_icon(STOCK_MEDIA_PLAY, ICON_SIZE_BUTTON)
        model.set_value(iterator, COLUMN_PIXBUF, pixbuf)

        selection = widget.get_selection()
        path = model.get_path(iterator)
        selection.select_path(path)
        return

    def __ui_end_command(self, model, iterator):
        widget = self.get_widget('treeview_status')

        pixbuf = None
        if self.__state == STATE_UNKNOWN:
            pixbuf = widget.render_icon(STOCK_YES, ICON_SIZE_BUTTON)
        elif self.__state == STATE_RUNNING:
            pixbuf = widget.render_icon(STOCK_YES, ICON_SIZE_BUTTON)
        else:
            pixbuf = widget.render_icon(STOCK_NO, ICON_SIZE_BUTTON)

        model.set_value(iterator, COLUMN_PIXBUF, pixbuf)
        return

    def __enable_test_rip_button(self, enable):
        checkbutton_test_rip = self.get_widget('checkbutton_test_rip')
        #cfg = self.get_project().get_config()

        #if enable:
        #    # enable the test rip button only if every source has more than one chapter
        #    enabled_titles = self.get_project().get_enabled_titles()
        #    for title in enabled_titles:
        #        if len(title.get_chapters()) < 2:
        #            enable = False
        #            break

        checkbutton_test_rip.set_sensitive(enable)
        return

    def __conditional_quit(self):
        cfg = self.get_project().get_config()
        if cfg.get_exit_winki():
            raise SystemExit
        return


    # vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
