# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import CellRendererPixbuf, CellRendererText, TreeViewColumn
from gtk import TreeStore, ListStore
from gtk.gdk import pixbuf_new_from_file, Pixbuf
from gobject import TYPE_STRING, TYPE_PYOBJECT
from winkirip.lib import winki_path, build_path, to_int, debug, error
from winkirip.page_base import BasePage
from winkirip.split_chapters import ChapterSplitter
from winkirip.container import Container, CONTAINER_UNKNOWN, CONTAINER_DVD
from winkirip.container import CONTAINER_VCD, CONTAINER_SVCD
from winkirip.container_title import Title
from winkirip.container_audio import Audio, AUDIO_OGG
from winkirip.container_video import Video, VIDEO_MPEG4
from winkirip.container_subtitle import Subtitle
from winkirip.estimation import Estimation
from copy import deepcopy

COLUMN_PIXBUF = 0
COLUMN_TITLE = 1
COLUMN_OBJECT = 2

class MediaHelper:
    def __init__(self, container, title, chapter):
        self.__container = container
        self.__title = title
        self.__chapter = chapter
        return

    def get_container(self):
        return self.__container

    def get_title(self):
        return self.__title

    def get_chapter(self):
        return self.__chapter



class PageSize(BasePage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        self.__media = []
        self.__chunk_list = []
        self.__orig_container = None
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_size.png'

    def get_title(self):
        return _('Target')

    def get_signal_list(self):
        return {'on_spinbutton_number_value_changed' : self.on_spin_number_targets_changed,
                'on_checkbutton_bitrate_toggled' : self.on_toggle_bitrate,
                'on_spinbutton_bitrate_value_changed' : self.on_spin_bitrate_value_changed,
                'on_comboboxentry_size_changed' : self.on_combo_size_changed}

    def get_widget_list(self):
        widgets = []
        return widgets

    def on_prepare(self):
        self.set_stoplight('go')

        #print "page_size: PREPARE -->"
        #for container in self.get_project().get_dest():
        #    print container
        #print "page_size: PREPARE <--"

        self.__setup_checkbutton_bitrate_sensitivity()

        if not self.get_needs_init():
            return True

        if len(self.get_project().get_dest()) != 1:
            return False

        self.__orig_container = deepcopy(self.get_project().get_dest()[0])

        self.__fill_media()

        self.__setup_combo_size()

        desired_size = self.__get_media_size()
        nr_target_discs = self.__estimate_number_targets(desired_size)
        
        spinbutton = self.get_widget('spinbutton_number')
        spinbutton.set_value(nr_target_discs)

        self.set_needs_init(False)
        return True

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        # All previous pages (up to this one inclusive) expect to have only one dest container
        # which gets eventually splitted by this class into multiple ones.
        # So we restore the one and only container we have received initially in order not to confuse
        # the previous pages (and ourselves later)
        self.get_project().reset_dest()
        self.get_project().add_dest(self.__orig_container)
        return True

    def can_exit(self):
        return True

    def show_help(self):
        return BasePage.show_help(self)
    
    def update_project(self):
        self.disable_buttons()

        self.__create_dest()

        checkbox_bitrate = self.get_widget('checkbutton_bitrate')
        manual_bitrate = checkbox_bitrate.get_active()

        if not manual_bitrate:
            selected_size = self.__get_media_size()
            dest_containers = self.get_project().get_dest()
            for container in dest_containers:
                container.set_size(selected_size)

        else:
            spin_bitrate = self.get_widget('spinbutton_bitrate')
            vbitrate = to_int(spin_bitrate.get_value())
            dest_containers = self.get_project().get_dest()
            for container in dest_containers:
                for title in container:
                    for video in title.get_video():
                        video.set_bitrate(vbitrate)

        self.enable_buttons()
        self.get_druid().invalidate_after('size')
        #print "page_size: READY -->"
        #for container in self.get_project().get_dest():
        #    print container
        #print "page_size: READY <--"
        return True

    def get_settings_callback(self):
        return self.__setup_checkbutton_bitrate_sensitivity

    # -- signal handlers --

    def on_spin_number_targets_changed(self, spin_button, event = None):
        nr_chunks = int(spin_button.get_value())

        splitter = ChapterSplitter(self.__media)
        self.__chunk_list = splitter.split(nr_chunks)

        self.__init_treeview()

        checkbox_bitrate = self.get_widget('checkbutton_bitrate')
        manual_bitrate = checkbox_bitrate.get_active()

        self.__update_label_bitrate(not manual_bitrate)
        return True

    def on_combo_size_changed(self, widget, event = None):
        self.__update_label_bitrate(True)
        return True

    def on_toggle_bitrate(self, checkbox_bitrate):
        spin_bitrate = self.get_widget('spinbutton_bitrate')
        combo_size = self.get_widget('comboboxentry_size')
        label_estimation = self.get_widget('label_estimation_label')
        label_size = self.get_widget('label_size')
        label_mb = self.get_widget('label_mb')
        label_kbs = self.get_widget('label_kbs')

        activated = checkbox_bitrate.get_active()

        spin_bitrate.set_sensitive(activated)
        label_kbs.set_sensitive(activated)
        combo_size.set_sensitive(not activated)
        label_size.set_sensitive(not activated)
        label_mb.set_sensitive(not activated)

        if activated:
            #label_estimation.set_text(_('Estimated size'))
            label_estimation.set_text('')
        else:
            label_estimation.set_text(_('Estimated video bitrate'))

        self.__update_label_bitrate(not activated)
        return True

    def on_spin_bitrate_value_changed(self, spin_button, event = None):
        checkbox_bitrate = self.get_widget('checkbutton_bitrate')
        manual_bitrate = checkbox_bitrate.get_active()

        if manual_bitrate:
            value = spin_button.get_text()
            if value and len(value) > 0:
                spin_button.set_value(float(value))

        self.__update_label_bitrate(not manual_bitrate)
        return True

    # -- helpers --

    def __init_treeview(self):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()
        if not model:
            self.__create_treeview()
        else:
            model.clear()

        self.__fill_model()
        return

    def __create_treeview(self):
        model = TreeStore(Pixbuf, TYPE_STRING, TYPE_PYOBJECT)
        treeview = self.get_widget('treeview_titles')
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_PIXBUF)
        treeview.append_column(column)

        renderer = CellRendererText()
        column = TreeViewColumn(_('Target structure preview'), renderer, markup = COLUMN_TITLE)
        treeview.append_column(column)
        return

    def __fill_model(self):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()

        disc_fname = build_path([winki_path, 'glade', 'media_media.png'])
        disc_pixbuf = pixbuf_new_from_file(disc_fname)

        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        chapter_fname = build_path([winki_path, 'glade', 'media_chapter.png'])
        chapter_pixbuf = pixbuf_new_from_file(chapter_fname)

        counter = 0
        container_id = -1
        title_id = -1
        disc_iter = None
        title_iter = None
        chunk_index = 0
        for item in self.__media:
            container = item.get_container()
            title = item.get_title()
            chapter = item.get_chapter()

            if counter == self.__chunk_list[chunk_index]:
                disc_name = '<span weight="bold" size="larger">Disc %d</span>' % (chunk_index + 1)
                disc_iter = model.append(None, [disc_pixbuf, disc_name, container])
                if chunk_index < (len(self.__chunk_list) - 1):
                    chunk_index += 1

                title_id = title.get_id()
                container_id = title.get_container_id()
                title_name = ('<span weight="bold">%s</span>' % title.get_name())
                assert(disc_iter)
                title_iter = model.append(disc_iter, [title_pixbuf, title_name, title])

            if title.get_id() != title_id or title.get_container_id() != container_id:
                title_id = title.get_id()
                container_id = title.get_container_id()
                title_name = ('<span weight="bold">%s</span>' % title.get_name())
                assert(disc_iter)
                title_iter = model.append(disc_iter, [title_pixbuf, title_name, title])

            chapter_name = chapter.get_name()
            if not chapter_name:
                chapter_name = (_('Chapter %d') % chapter.get_id())
            chapter_name = ('<span weight="normal">%s</span>' % chapter_name)
            assert(title_iter)
            model.append(title_iter, [chapter_pixbuf, chapter_name, chapter])
            counter += 1

        treeview.expand_all()
        return

    def __fill_media(self):
        self.__media = []

        for container in self.get_project().get_dest():
            for title in container:
                for chapter in title.get_chapters():
                    item = MediaHelper(container, title, chapter)
                    self.__media.append(item)

        self.__chunk_list = [0]
        return

    def __create_dest(self):
        # todo: not obvious dest destruction
        self.get_project().reset_dest()

        counter = 0
        title_id = -1
        chunk_index = 0
        dest_cont = None
        dest_title = None
        dest_title_id = 0
        for item in self.__media:
            container = item.get_container()
            title = item.get_title()
            chapter = item.get_chapter()

            if counter == self.__chunk_list[chunk_index]:
                dest_cont = self.__create_container(container)
                dest_title = self.__create_title(dest_cont, title, 1)
                dest_title_id = 2
                title_id = title.get_id()
                if chunk_index < (len(self.__chunk_list) - 1):
                    chunk_index += 1

            if title.get_id() != title_id:
                title_id = title.get_id()
                assert(dest_cont)
                dest_title = self.__create_title(dest_cont, title, dest_title_id)
                dest_title_id += 1

            assert(dest_cont and dest_title)
            dest_title.add_chapter(chapter)
            counter += 1
        return

    def __create_container(self, source_container):
        container = Container()
        container.set_type(source_container.get_type())
        container.set_name(source_container.get_name())
        container.set_size(source_container.get_size())
        container.set_tags(source_container.get_tags())
        container.set_device(source_container.get_device())
        container.set_id(source_container.get_id())

        self.get_project().add_dest(container)
        return container

    def __create_title(self, container, source_title, title_id):
        title = Title()
        container.add_title(title)
        title.set_id(title_id)
        title.set_name(source_title.get_name())
        title.set_crop(source_title.get_crop())
        if source_title.get_palette():
            title.set_palette(source_title.get_palette())
        title.set_palette_file(source_title.get_palette_file())
        for video in source_title.get_video():
            title.add_video(deepcopy(video))
        for audio in source_title.get_audio():
            title.add_audio(deepcopy(audio))
        for subtitle in source_title.get_subtitles():
            title.add_subtitle(deepcopy(subtitle))
        return title

    def __setup_combo_size(self):
        combo = self.get_widget('comboboxentry_size')
        model = combo.get_model()
        if not model:
            model = ListStore(str)
            combo.set_model(model)
            combo.set_text_column(0)
        else:
            model.clear()

        cfg = self.get_project().get_config()

        ctype = CONTAINER_UNKNOWN
        default_size = 700

        if len(self.__media) > 0:
            container = self.__media[0].get_container()
            ctype = container.get_type()
            default_size = cfg.get_size(ctype)

        sizes = self.__get_standard_sizes()

        pos = 0
        active_elem = -1
        for item  in sizes:
            key, value = item
            if value == default_size:
                active_elem = pos
            combo.append_text(key)
            pos += 1

        if active_elem < 0:
            combo.append_text('%d' % default_size)
            active_elem = len(sizes)
        combo.set_active(active_elem)
        return

    def __update_label_bitrate(self, estimate_bitrate):
        label_mb = self.get_widget('label_mb')
        label_bitrate = self.get_widget('label_bitrate')
        label_recommend = self.get_widget('label_recommend')
        label_recommend_label = self.get_widget('label_recommend_label')
        text_bitrate = ''
        text_recommend = ''

        # show estimation about the target size
        if not estimate_bitrate:
            spin_button_bitrate = self.get_widget('spinbutton_bitrate')
            desired_bitrate = spin_button_bitrate.get_value()
            text_bitrate, text_recommend = self.__estimate_target_size(desired_bitrate)

        # show estimation about the target bitrate
        else:
            desired_size = self.__get_media_size()

            show_label_mb = True
            sizes = self.__get_standard_sizes()
            for item in sizes:
                key, value = item
                if value == desired_size:
                    show_label_mb = False
                    break
            label_mb.set_property('visible', show_label_mb)

            text_bitrate, text_recommend = self.__estimate_video_bitrate(desired_size * 1024 * 1024)

        label_bitrate.set_markup(text_bitrate)
        label_recommend.set_markup(text_recommend)

        if len(text_recommend) > 0:
            label_recommend_label.set_markup(_('Recommendation'))
        else:
            label_recommend_label.set_markup('')
        return

    def __get_standard_sizes(self):
        return ((_('CD 650MB'), 650), (_('CD 700MB'), 700), (_('CD 800MB'), 800),
        (_('DVD 4.37GB'), 4370), (_('1/2 DVD 4.37GB'), 2185),
        (_('1/3 DVD 4.37GB'), 1456.7), (_('1/4 DVD 4.37GB'), 1092.5),
        (_('1/5 DVD 4.37GB'), 874), (_('1/6 DVD 4.37GB'), 728.4),
        (_('1/7 DVD 4.37GB'), 624.3), (_('1/8 DVD 4.37GB'), 546.3))

    def __get_media_size(self):
        sizes = self.__get_standard_sizes()
        
        combo = self.get_widget('comboboxentry_size')
        entry = combo.get_child()

        target = entry.get_text()
        for item in sizes:
            key, value = item
            if target == key:
                return value

        return to_int(entry.get_text())

    def __estimate_chapter_size(self, chapter, config):
        occupied = 0.0
        alist = []
        slist = []

        aformat = AUDIO_OGG
        if len(self.__media) > 0 and self.__media[0].get_container().get_type() == CONTAINER_DVD:
            aformat = config.get_audio_format_dvd()
        else:
            aformat = config.get_audio_format_file()

        len_in_seconds = chapter.get_length()

        disc_id = chapter.get_container_id()
        title_id = chapter.get_title_id()
        dest = self.get_project().get_dest()
        for container in dest:
            if container.get_id() == disc_id:
                for title in container:
                    if title.get_id() == title_id:
                        alist = title.get_audio()
                        slist = title.get_subtitles()
                        break

        for audio in alist:
            audio_penalty = config.get_audio_bytes_per_second(aformat)
            occupied += (audio_penalty * len_in_seconds)

        for subtitle in slist:
            sformat = subtitle.get_format()
            subtitle_penalty = config.get_subtitle_bytes_per_second(sformat)
            occupied += (subtitle_penalty * len_in_seconds)

        return occupied

    def __estimate_number_targets(self, desired_size):
        cfg = self.get_project().get_config()
        penalty_ratio = cfg.get_container_penalty()

        length = 0.0
        size = 0.0
        for item in self.__media:
            chapter = item.get_chapter()
            length += chapter.get_length()
            size += self.__estimate_chapter_size(chapter, cfg)

        estim = Estimation()
        estim.set_penalty_ratio(penalty_ratio)
        estim.add_occupied(size)
        estim.add_time(length)

        vformat = VIDEO_MPEG4
        if len(self.__media) > 0 and self.__media[0].get_container().get_type() == CONTAINER_DVD:
            vformat = cfg.get_video_format_dvd()
        else:
            vformat = cfg.get_video_format_file()

        pref_bitrate = cfg.get_preferred_bitrate(vformat)
        min_bitrate = cfg.get_minimum_bitrate(vformat)
        chunk_size = desired_size * 1024 * 1024
        return estim.estimate_number_targets(chunk_size, pref_bitrate, min_bitrate)

    def __estimate_video_bitrate(self, desired_size):
        result = []
        cfg = self.get_project().get_config()
        penalty_ratio = cfg.get_container_penalty()
        estim = Estimation(desired_size)
        estim.set_penalty_ratio(penalty_ratio)

        disc_len = 0.0
        disc_size = 0.0
        chunk_index = 0
        for i in range(len(self.__media)):
            container = self.__media[i].get_container()
            chapter = self.__media[i].get_chapter()
            estim.set_container_type(container.get_type())

            if i == self.__chunk_list[chunk_index]:
                if chunk_index < (len(self.__chunk_list) - 1):
                    chunk_index += 1

                if i > 0:
                    estim.set_time(disc_len)
                    estim.set_occupied(disc_size)
                    result.append(estim.estimate_vbitrate())
                    disc_len = 0.0

            disc_len += chapter.get_length()
            disc_size += self.__estimate_chapter_size(chapter, cfg)

        estim.set_time(disc_len)
        estim.set_occupied(disc_size)
        result.append(estim.estimate_vbitrate())

        return self.__format_estimation(result, False), ''
        #return self.__format_estimation(result, False), self.__format_recommendation(result)

    def __estimate_target_size(self, desired_bitrate):
        return '', ''

    def __format_estimation(self, estim_list, show_size):
        result = ''

        if not estim_list or not len(estim_list):
            return '<span weight="normal">cannot split into chunks</span>'

        count = 1
        for number in estim_list:
            color_text = self.__format_color(number, show_size)
            result += '<span weight="bold">Disc %d: %s</span>\n' % (count, color_text)
            count += 1

        return result

    def __format_recommendation(self, estim_list):
        result = ''

        if not estim_list or not len(estim_list):
            return ''

        if len(self.__media) < 1:
            return ''

        ctype = self.__media[0].get_container().get_type()
        if ctype == CONTAINER_DVD or ctype == CONTAINER_VCD or ctype == CONTAINER_SVCD:
            return ''

        average = 0
        for number in estim_list:
            average += number
        average /= len(estim_list)

        if average > 2000:
            result = _('For best quality by this video bitrate use MPEG-2 encoding.')
        else:
            result = _('For best quality by this video bitrate use either XviD or DivX encoding.')

        return result

    def __format_color(self, number, show_size):
        color = 'black'
        templ = '<span weight="bold" foreground="%s">%d</span>'

        if not show_size:
            if number < 800:
                color = 'red'
            elif number < 850:
                color = 'yellow'
            elif number < 1200:
                color = 'blue'
            else:
                color = 'darkgreen'
        else:
            number = (number / 1024.0)
            templ = '<span weight="bold" foreground="%s">%d MB</span>'

            if number > 1000:
                number /= 1024.0
                templ = '<span weight="bold" foreground="%s">%d GB</span>'

        return templ % (color, number)

    def __setup_checkbutton_bitrate_sensitivity(self):
        checkbox_bitrate = self.get_widget('checkbutton_bitrate')
        cfg = self.get_project().get_config()

        dest_type = self.get_project().get_dest_type()
        shrink_method = cfg.get_video_shrink_method_dvd()

        if shrink_method == 'requantize' and dest_type == CONTAINER_DVD:
            checkbox_bitrate.set_sensitive(False)
        else:
            checkbox_bitrate.set_sensitive(True)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
