# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk.gdk import _2BUTTON_PRESS, keyval_name
from winkirip.page_base import BasePage
from winkirip.page_base_language import BasePageLanguage
from winkirip.page_base_two_trees import COLUMN_LANG_OBJECT
from winkirip.page_base_two_trees import COLUMN_LANG_PIXBUF, COLUMN_LANG_TITLE
from winkirip.container_subtitle import Subtitle


class PageSubtitles(BasePage, BasePageLanguage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BasePageLanguage.__init__(self)
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_subtitle.png'

    def get_title(self):
        return _('Subtitles Selection')

    def get_signal_list(self):
        return {'on_checkbutton_mix_subtitles_toggled' : self.on_mix_subtitles,
                'on_treeview_button_press_event': self.on_treeview_button_pressed,
                'on_treeview_key_press_event': self.on_treeview_key_pressed}

    def get_widget_list(self):
        return []

    def on_prepare(self):
        self.__toggle_stoplight()

        if not self.get_needs_init():
            return True

        self.set_treeviews(self.get_widget('treeview_source'), self.get_widget('treeview_dest'))

        self.fill_dest_container(self.get_project())
        self.__init_treeview('treeview_source', _('Available subtitles'))

        self.reset_dest_container(self.get_project())
        self.__init_treeview('treeview_dest', _('Subtitles in use'))

        self.get_widget('treeview_source').grab_focus()

        self.set_needs_init(False)
        return True

    def on_next(self):
        return self.update_project()

    def on_prev(self):
        return True

    def can_exit(self):
        return True

    def update_project(self):
        self.disable_buttons()

        treeview = self.get_widget('treeview_dest')
        model = treeview.get_model()

        checkbutton = self.get_widget('checkbutton_mix_subtitles')
        if checkbutton.get_active():
            stream_status = self.__check_selected_streams(model)
            if stream_status == 0:
                pass

            elif stream_status == 1:
                primary = _('No subtitles selected')
                secondary = _('You have choosen to mix the subtitles into the movie but havent selected any. If you choose to go ahead there will be no subtitles in the movie.')
                affirmative = _('Continue without subtitles')
                result = self.show_alert(primary, secondary, affirmative)
                if not result:
                    self.enable_buttons()
                    return False

            elif stream_status == 2:
                primary = _('Too many subtitles selected')
                secondary = _('You have choosen to mix the subtitles into the movie. At most one subtitle per title may be mixed into the video.')
                self.show_error(primary, secondary)
                self.enable_buttons()
                return False

            else:
                assert(False)

        if not self.sumbit_selected_streams(model, self.get_project().get_dest()):
            return False

        self.get_druid().invalidate_after('subtitles')

        self.enable_buttons()
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- methods inherited from BaseTwoTreesPage --

    def get_title_streams(self, title):
        return title.get_subtitles()

    def add_title_stream(self, title, stream):
        title.add_subtitle(stream)
        return

    def reset_title_streams(self, title):
        title.reset_subtitles()
        return

    # -- signal handlers --

    def on_treeview_key_pressed(self, treeview, event):
        if keyval_name(event.keyval) != 'space' and keyval_name(event.keyval) != 'Return':
            return False

        # space or return pressed
        self.move_selected(treeview)
        self.get_druid().invalidate_after('subtitles')
        return True

    def on_treeview_button_pressed(self, treeview, event):
        result = False

        # double click with the left mouse button
        if event.button == 1 and event.type == _2BUTTON_PRESS:
            self.move_selected(treeview)
            self.get_druid().invalidate_after('subtitles')
            result = True

        # right click
        if event.button == 3:
            # defined only for the source tree
            # I suspect that the user might be confused if this operation is allowed
            # in the dest treeview: he might think that this represents some sort of conversion
            if treeview.get_name() == 'treeview_dest':
                return False

            result = False
            selection = treeview.get_selection()
            (model, pathlist) = selection.get_selected_rows()

            slist = []
            for path in pathlist:
                it = model.get_iter(path)
                obj = model.get_value(it, COLUMN_LANG_OBJECT)
                if not obj:
                    continue

                # append all selected subtitle streams
                if isinstance(obj, Subtitle):
                    slist.append(obj)

                # a title:
                # if only one title is selected: use all subtitle streams from it
                else:
                    if len(pathlist) == 1:
                        siter = model.iter_children(it)
                        while siter:
                            slist.append(model.get_value(siter, COLUMN_LANG_OBJECT))
                            siter = model.iter_next(siter)
                        break

            vlist = []
            alist = []
            if len(slist) > 0:
                self.show_stream_properties(vlist, alist, slist, self.__update_tree_data)

        return result

    def on_mix_subtitles(self, widget, event = None):
        self.__toggle_stoplight()
        return

    def on_drag_data_received(self, treeview, context, x, y, selection, info, timestamp):
        self.drag_data_receive(treeview, x, y, selection)

        self.__toggle_stoplight()
        self.get_druid().invalidate_after('subtitles')
        return True

    def on_drag_data_get(self, treeview, context, selection, info, timestamp):
        return self.drag_data_get(treeview, selection)

    # -- public interface --

    # -- helpers --

    def __init_treeview(self, treeview_name, treeview_title):
        treeview = self.get_widget(treeview_name)
        model = treeview.get_model()
        if not model:
            self.create_treeview(treeview, treeview_title)
            self.setup_dnd(treeview, self.on_drag_data_get, self.on_drag_data_received)
        else:
            model.clear()

        title_list = []
        for container in self.get_project().get_dest():
            for title in container:
                title_list.append(title)

        iso_codes = self.get_project().get_iso_codes()
        self.fill_model(title_list, treeview, iso_codes, _('Subtitle %d'))
        return

    def __update_tree_data(self):
        treeview = self.get_widget('treeview_source')
        model = treeview.get_model()
        iso_codes = self.get_project().get_iso_codes()

        for row in model:
            title_iter = row.iter
            siter = model.iter_children(title_iter)
            while siter:
                self.__update_tree_from_object(siter, model, iso_codes)
                siter = model.iter_next(siter)
        return

    def __update_tree_from_object(self, siter, model, iso_codes):
        subtitle = model.get_value(siter, COLUMN_LANG_OBJECT)
        lang = iso_codes.find_iso_639_1(subtitle.get_language())

        model.set_value(siter, COLUMN_LANG_PIXBUF , lang.get_flag())
        model.set_value(siter, COLUMN_LANG_TITLE , lang.get_name())
        return

    def __toggle_stoplight(self):
        checkbutton = self.get_widget('checkbutton_mix_subtitles')
        if checkbutton.get_active():
            treeview = self.get_widget('treeview_dest')
            model = treeview.get_model()

            if self.__check_selected_streams(model) == 0:
                self.set_stoplight('go')
            else:
                self.set_stoplight('stop')
        else:
            self.set_stoplight('go')
        return

    def __check_selected_streams(self, model):
        checkbutton = self.get_widget('checkbutton_mix_subtitles')

        if checkbutton.get_active():
            for row in model:
                iterator = model.iter_children(row.iter)
                if not iterator:
                    return 1

                # no more than one subtitle per title is allowed in this mode
                if model.iter_next(iterator):
                    return 2

        return 0

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
