# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import TreeStore, CellRendererPixbuf, CellRendererToggle
from gtk import TreeViewColumn, CellRendererText
from gtk.gdk import pixbuf_new_from_file, Pixbuf
from gobject import TYPE_BOOLEAN, TYPE_STRING, TYPE_PYOBJECT
from winkirip.lib import debug, error, info, build_path, winki_path
from winkirip.lib import treepath_string_to_tuple
from winkirip.page_base import BasePage
from winkirip.page_base_player import BasePlayerPage
from winkirip.container import Container, CONTAINER_UNKNOWN
from winkirip.container_title import Title
from winkirip.constructor_player import ConstructorPlayer


COLUMN_PIXBUF = 0
COLUMN_ENABLED = 1
COLUMN_TEXT = 2
COLUMN_TITLE = 3
COLUMN_CONTAINER = 4
COLUMN_TOGGLE_VISIBLE = 5

class PageTitle(BasePage, BasePlayerPage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BasePlayerPage.__init__(self)

        dock_target = self.get_widget('player_dock')
        BasePlayerPage.dock(self, dock_target)
        BasePlayerPage.set_action_text(self, _('Pre_view'), _('S_top'))
        return

    # -- methods inherited from page_base.BasePlayerPage --

    def on_button_click(self, treeview, event):
        # ignore everything but the right mouse button
        if event.button != 3:
            return False

        selection = treeview.get_selection()
        (model, pathlist) = selection.get_selected_rows()
        if len(pathlist) > 1:
            print 'too many items: still not implemented :-)'
            return False

        vlist = []
        alist = []
        slist = []
        for path in pathlist:
            it = model.get_iter(path)
            title = model.get_value(it, COLUMN_TITLE)
            if not title:
                return False

            vlist.extend(title.get_video())
            alist.extend(title.get_audio())
            slist.extend(title.get_subtitles())

        if len(vlist) > 0 or len(alist) > 0 or len(slist) > 0:
            self.show_stream_properties(vlist, alist, slist)
        return False

    def build_player_command(self):
        treeview = self.get_widget('treeview_titles')
        selection = treeview.get_selection()
        model, iterator, = selection.get_selected()
        if not iterator:
            primary = _('No title selected')
            secondary = _('Select a title from the list in order to preview it\'s content')
            self.show_error(primary, secondary)
            return None

        # container and not title is selected: grab the longest title in the container
        path = model.get_path(iterator)
        if len(path) < 2:
            iterator = self.__get_longest_title_iter(iterator, model)

        if not iterator:
            return None

        title = model.get_value(iterator, COLUMN_TITLE)
        title_length = title.get_length()
        BasePlayerPage.set_range(self, title_length)

        container = model.get_value(iterator, COLUMN_CONTAINER)
        device = container.get_device()

        tool_checker = self.get_project().get_tool_checker()

        constr = ConstructorPlayer(tool_checker,
                                   self.get_project().get_config())
        constr.set_source(device, title.get_id())
        constr.set_source_type(container.get_type())

        success, shell_cmd = constr.construct()
        if not success:
            debug('PageTitle.build_player_command()', 'command construction failed', error)
            return ''
        return shell_cmd

    def get_player_callback(self):
        return self.__ignore_player_output

    def get_player_target_widget(self):
        return self.get_widget('drawingarea_player')

    def set_player_stoplight(self, light):
        self.set_stoplight(light)
        return

    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_title.png'

    def get_title(self):
        return _('Title Selection')

    def get_signal_list(self):
        return {'on_treeview_titles_button_press_event' : self.on_button_click}

    def get_widget_list(self):
        widgets = []

        return widgets

    def on_prepare(self):
        self.__set_stop_light()

        if not self.get_needs_init():
            return

        self.__init_treeview()
        self.__setup_longest_titles()

        self.__set_stop_light()
        self.set_needs_init(False)
        return True

    def on_next(self):
        if not self.__has_enabled_title():
            primary = _('No title selected')
            secondary = _('Select at least one title for ripping')
            self.show_error(primary, secondary)
            return False

        # stop the player if it's running
        if BasePlayerPage.has_player(self):
            self.on_action(None)

        return self.update_project()

    def on_prev(self):
        return (not BasePlayerPage.has_player(self))

    def can_exit(self):
        return (not BasePlayerPage.has_player(self))

    def update_project(self):
        self.disable_buttons()

        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()
        if not model:
            self.set_stoplight('stop')
            debug('PageTitle.update_project()', 'oops: no model', error)
            self.enable_buttons()
            return False

        # todo: container destruction not obvious
        media_type = CONTAINER_UNKNOWN
        if len(self.get_project().get_dest()) > 0:
            media_type = self.get_project().get_dest()[0].get_type()
            media_name = self.get_project().get_dest()[0].get_name()
            media_tags = self.get_project().get_dest()[0].get_tags()
            media_device = self.get_project().get_dest()[0].get_device()
        elif len(self.get_project().get_source()) > 0:
            media_type = self.get_project().get_source()[0].get_type()
            media_name = self.get_project().get_source()[0].get_name()
            media_tags = self.get_project().get_source()[0].get_tags()
            media_device = self.get_project().get_source()[0].get_device()

        self.get_project().reset_enabled_titles()
        self.get_project().reset_dest()
        media = Container()
        media.set_id(0)
        media.set_type(media_type)
        media.set_name(media_name)
        media.set_tags(media_tags)
        media.set_device(media_device)
        for row in model:
            if row[COLUMN_ENABLED]:
                iter_title = model.iter_children(row.iter)
                while iter_title:
                    if model.get_value(iter_title, COLUMN_ENABLED):
                        src_title = model.get_value(iter_title, COLUMN_TITLE)
                        self.get_project().add_enabled_title(src_title)
                        dest_title = Title(src_title)
                        media.add_title(dest_title)

                    iter_title = model.iter_next(iter_title)

        self.get_project().add_dest(media)
        self.get_druid().invalidate_after('title')

        self.enable_buttons()
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- signal handlers --

    def on_toggle_enabled(self, cell, path, model):
        it = model.get_iter(path)

        enabled = model.get_value(it, COLUMN_ENABLED)
        enabled = not enabled
        model.set(it, COLUMN_ENABLED, enabled)

        self.__set_stop_light()

        self.get_druid().invalidate_after('title')
        return True

    def on_edit_title(self, renderer, path, text, model):
        it = model.get_iter(path)
        model.set(it, COLUMN_TEXT, text)

        if model.get_value(it, COLUMN_TITLE):
            model.get_value(it, COLUMN_TITLE).set_name(text)

        elif model.get_value(it, COLUMN_CONTAINER):
            model.get_value(it, COLUMN_CONTAINER).set_name(text)

        self.get_druid().invalidate_after('title')
        return True

    # -- helpers --

    def __init_treeview(self):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()
        if not model:
            self.__create_treeview()
        else:
            model.clear()

        self.__fill_model()
        return

    def __get_longest_title_iter(self, iter_cont, model):
        longest_time = 0
        longest_title_iter = None

        iter_title = model.iter_children(iter_cont)
        while iter_title:
            title = model.get_value(iter_title, COLUMN_TITLE)
            if title.get_length() > longest_time:
                longest_title_iter = iter_title
                longest_time = title.get_length()
            iter_title = model.iter_next(iter_title)

        return longest_title_iter

    def __setup_longest_titles(self):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()

        first = True
        for cont_row in model:
            longest_title_iter = self.__get_longest_title_iter(cont_row.iter, model)

            if(longest_title_iter):
                model.set_value(cont_row.iter, COLUMN_ENABLED, True)
                model.set_value(longest_title_iter, COLUMN_ENABLED, True)

            if first:
                treeview.expand_row(model.get_path(cont_row.iter), True)

                selection = treeview.get_selection()
                selection.select_path(model.get_path(longest_title_iter))

                treeview.grab_focus()
                first = False
        return

    def __fill_model(self):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()

        cont_fname = build_path([winki_path, 'glade', 'media_media.png'])
        cont_pixbuf = pixbuf_new_from_file(cont_fname)

        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        for cont in self.get_project().get_source():
            iterator = model.append(None)
            model.set(iterator,
                      COLUMN_PIXBUF, cont_pixbuf,
                      COLUMN_ENABLED, True,
                      COLUMN_TEXT, cont.get_name(),
                      COLUMN_TITLE, None,
                      COLUMN_CONTAINER, cont,
                      COLUMN_TOGGLE_VISIBLE, False)
            for title in cont:
                it = model.append(iterator)
                model.set(it,
                          COLUMN_PIXBUF, title_pixbuf,
                          COLUMN_ENABLED, False,
                          COLUMN_TEXT, title.get_name(),
                          COLUMN_TITLE, title,
                          COLUMN_CONTAINER, cont,
                          COLUMN_TOGGLE_VISIBLE, True)
        return

    def __create_treeview(self):
        model = TreeStore(Pixbuf, TYPE_BOOLEAN, TYPE_STRING, TYPE_PYOBJECT, TYPE_PYOBJECT, TYPE_BOOLEAN)

        treeview = self.get_widget('treeview_titles')
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_PIXBUF)
        treeview.append_column(column)

        renderer = CellRendererToggle()
        renderer.connect('toggled', self.on_toggle_enabled, model)
        column = TreeViewColumn(_('Rip'), renderer, active = COLUMN_ENABLED)
        column.set_attributes(renderer, visible = COLUMN_TOGGLE_VISIBLE, active = True)
        treeview.append_column(column)

        renderer = CellRendererText()
        renderer.set_property('editable', True)
        renderer.connect('edited', self.on_edit_title, model)
        column = TreeViewColumn(_('Title'), renderer, text = COLUMN_TEXT)
        treeview.append_column(column)

        return

    def __enable_ui(self, enable):
        treeview = self.get_widget('treeview_titles')
        area = self.get_widget('drawingarea_mplayer')
        scroll = self.get_widget('scrolledwindow1')

        treeview.set_sensitive(enable)
        scroll.set_sensitive(enable)
        area.set_sensitive(not enable)
        # todo: was this function removed? is it necessary?
        #BasePlayerPage.enable_ui(self, not enable)

        treeview.grab_focus()
        return

    def __has_enabled_title(self):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()
        if not model:
            self.set_stoplight('stop')
            # this is actually an error but since the function is called in
            # on_prepare() (the first time there is no model) and the terminal
            # output makes me sick I lowered its priority to info...
            debug('PageTitle.__has_enabled_title()', 'oops: no model', info)
            return False

        enabled = False
        for cont in model:
            if enabled:
                break

            iter_title = model.iter_children(cont.iter)
            while iter_title:
                if model.get_value(iter_title, COLUMN_ENABLED):
                    enabled = True
                    break
                iter_title = model.iter_next(iter_title)

        return enabled

    def __set_stop_light(self):
        if self.__has_enabled_title():
            self.set_stoplight('go')
        else:
            self.set_stoplight('stop')
        return

    def __ignore_player_output(self, text):
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
