# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import CellRendererPixbuf, CellRendererText, TreeViewColumn
from gtk import TreeStore, SELECTION_MULTIPLE, TreeRowReference
from gtk import TREE_VIEW_DROP_BEFORE, TREE_VIEW_DROP_INTO_OR_BEFORE
from gtk.gdk import Pixbuf, pixbuf_new_from_file, ACTION_DEFAULT
from gtk.gdk import ACTION_MOVE, ACTION_COPY, BUTTON1_MASK
from gobject import TYPE_STRING, TYPE_PYOBJECT
from winkirip.page_base import BasePage
from winkirip.page_base_tree import BasePageTree
from winkirip.page_base_two_trees import COLUMN_TREE_LANG
from winkirip.page_base_two_trees import COLUMN_TREE_TYPE, COLUMN_TREE_TITLE
from winkirip.page_base_two_trees import COLUMN_TREE_OBJECT
from winkirip.container import Container
from winkirip.container_title import Title
from winkirip.container_stream import TYPE_VIDEO, TYPE_AUDIO, TYPE_SUBTITLE 
from winkirip.lib import build_path, winki_path, debug, error

class PageTree(BasePage, BasePageTree):

    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BasePageTree.__init__(self)
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_tree.png'

    def get_title(self):
        return _('Build target')


    def get_signal_list(self):
        return {'on_button_remove_clicked' : self.on_remove,
                'on_button_title_clicked' : self.on_new_title}

    def get_widget_list(self):
        widgets = []
        return widgets

    def on_prepare(self):
        self.set_stoplight('go')

        if not self.get_needs_init():
            return True

        self.set_treeviews(self.get_widget('treeview_source'),
                           self.get_widget('treeview_dest'))

        self.__init_treeviews()

        self.set_needs_init(False)
        return True

    def on_next(self):
        if not self.__check_stream_order():
            return False

        return self.update_project()

    def on_prev(self):
        return True

    def can_exit(self):
        return True

    def update_project(self):
        self.disable_buttons()
        self.__sumbit_selected_streams()
        self.get_druid().invalidate_after('tree')
        self.enable_buttons()
        return True

    # -- signal handlers --

    def on_remove(self, widget):
        treeview = self.get_widget('treeview_dest')
        selection = treeview.get_selection()
        model, selected = selection.get_selected_rows()

        # remove the elements with the longest paths (the leafs) first
        slist = sorted(selected, self.__cmp_tuple_len)
        slist.reverse()

        prev_element = (0)
        iters = [model.get_iter(path) for path in slist]
        for it in iters:
            # don't remove the one and only container element
            if len(model.get_path(it)) == 1 and model.get_path(it)[0] == 0:
                continue

            prev_element = self.__get_prev_tree_element(model.get_path(it))
            model.remove(it)

        selection.select_path(prev_element)

        self.__update_title_indices(treeview)
        return True

    def on_new_title(self, widget):
        treeview = self.get_widget('treeview_dest')
        selection = treeview.get_selection()
        model, selected = selection.get_selected_rows()

        new_title = Title()

        slist = sorted(selected, self.__cmp_tuple_len)
        if not slist or len(slist) < 1:
            debug('PageTree.on_new_title()', 'oops: no model!?!', error)
            return False

        # append to the top level
        elif len(slist[0]) == 1:
            self.__insert_tile_as_last(model, new_title)

        # append after this element
        elif len(slist[0]) == 2:
            self.__insert_tile_after(model.get_iter(slist[0]), model, new_title)

        # append after the parent of this element
        else:
            self.__insert_tile_after_parent(model.get_iter(slist[0]), model, new_title)

        self.__update_title_indices(treeview)
        return True

    def on_drag_data_get(self, treeview, context, selection, info, timestamp):
        return self.drag_data_get(treeview, selection)

    def on_drag_data_received(self, treeview, context, x, y, selection, info, timestamp):
        self.drag_data_receive(treeview, x, y, selection)
        self.get_druid().invalidate_after('tree')
        return True

    # -- helpers --

    def __init_treeviews(self):
        treeview_source = self.get_widget('treeview_source')
        model = treeview_source.get_model()
        if not model:
            self.__create_treeview(treeview_source, _('Available streams'))
            self.setup_dnd(treeview_source, self.on_drag_data_get, self.on_drag_data_received)
        else:
            model.clear()
        self.__fill_model(treeview_source, self.get_project().get_source())

        treeview_dest = self.get_widget('treeview_dest')
        model = treeview_dest.get_model()
        if not model:
            self.__create_treeview(treeview_dest, _('Result'))
            self.setup_dnd(treeview_dest, self.on_drag_data_get, self.on_drag_data_received)
        else:
            model.clear()

        self.__fill_model(treeview_dest, self.get_project().get_dest())
        return

    def __create_treeview(self, treeview, header_text):
        model = TreeStore(Pixbuf, Pixbuf, TYPE_STRING, TYPE_PYOBJECT)
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_TREE_LANG)
        treeview.append_column(column)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_TREE_TYPE)
        treeview.append_column(column)

        renderer = CellRendererText()
        column = TreeViewColumn(header_text, renderer, markup = COLUMN_TREE_TITLE)
        treeview.append_column(column)

        selection = treeview.get_selection()
        selection.set_mode(SELECTION_MULTIPLE)

        treeview.set_reorderable(True)
        return

    def __fill_model(self, treeview, containers):
        model = treeview.get_model()

        empty_fname = build_path([winki_path, 'glade', 'empty.png'])
        cont_fname = build_path([winki_path, 'glade', 'media_media.png'])
        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        video_fname = build_path([winki_path, 'glade', 'media_video.png'])
        audio_fname = build_path([winki_path, 'glade', 'media_audio.png'])
        subtitle_fname = build_path([winki_path, 'glade', 'media_subtitle.png'])
        empty_pixbuf = pixbuf_new_from_file(empty_fname)
        cont_pixbuf = pixbuf_new_from_file(cont_fname)
        title_pixbuf = pixbuf_new_from_file(title_fname)
        video_pixbuf = pixbuf_new_from_file(video_fname)
        audio_pixbuf = pixbuf_new_from_file(audio_fname)
        subtitle_pixbuf = pixbuf_new_from_file(subtitle_fname)

        iso_codes = self.get_project().get_iso_codes()

        for cont in containers:
            cont_iterator = model.append(None, [cont_pixbuf, empty_pixbuf, '<span weight="bold">%s</span>' % cont.get_name(), cont])
            for title in cont:
                title_name = ('<span weight="bold">%s</span>' % title.get_name())
                title_iterator = model.append(cont_iterator, [title_pixbuf, None, title_name, title])

                self.__add_streams(title.get_video(), 'video', video_pixbuf, iso_codes, model, title_iterator)
                self.__add_streams(title.get_audio(), 'audio', audio_pixbuf, iso_codes, model, title_iterator)
                self.__add_streams(title.get_subtitles(), 'subtitle', subtitle_pixbuf, iso_codes, model, title_iterator)

        selection = treeview.get_selection()
        selection.select_path((0))

        if treeview.get_name() == 'treeview_dest':
            treeview.expand_row((0), True)
        return

    def __add_streams(self, stream_list, stream_type, pixbuf, iso_codes, model, iterator):
        counter = 1

        for stream in stream_list:
            lang = None
            stream_name = ''
            if stream_type != 'video':
                lang = iso_codes.find_iso_639_1(stream.get_language())
                stream_name = lang.get_name()

            if not lang and stream_type != 'video':
                debug('PageTree.__add_stareams()', 'unknown language %s' % stream.get_language())
                continue

            if not stream_name or not len(stream_name):
                stream_name = (_('%(stream_index)d %(stream_type)s stream') % {'stream_index':counter, 'stream_type':stream_type})
            else:
                stream_name = (_('%(stream_name)s %(stream_type)s stream') % {'stream_name':stream_name, 'stream_type':stream_type})

            stream_name = ('<span weight="normal">%s</span>' % stream_name)

            if stream_type != 'video':
                model.append(iterator, [lang.get_flag(), pixbuf, stream_name, stream])
            else:
                model.append(iterator, [None, pixbuf, stream_name, stream])
            counter += 1
        return

    def __get_prev_tree_element(self, path):
        result = ()
        path_len = len(path)

        # prev child element at the same level (sister element)
        if path[path_len - 1] > 0:
            for i in range(path_len - 1):
                result += path[i],
            last_elem = path[path_len - 1] - 1
            result += last_elem,

        # the parent element
        elif path_len > 1:
            for i in range(path_len - 1):
                result += path[i],

        # the root element
        else:
            result = (0)

        return result

    def __cmp_tuple_len(self, tuple1, tuple2):
        if len(tuple1) > len(tuple2):
            return 1

        elif len(tuple1) < len(tuple2):
            return -1

        return 0

    def __update_title_indices(self, treeview):
        """This function will be called after titles are added
            or deleted from the container. While adding or deliting
            items the title id's will change and we fix the id's and
            the dependent names here
        """
        model = treeview.get_model()
        iter_cont = model.get_iter((0))
        cont = model.get_value(iter_cont, COLUMN_TREE_OBJECT)

        iter_title = model.iter_children(iter_cont)
        title_id = 1
        while iter_title:
            title = model.get_value(iter_title, COLUMN_TREE_OBJECT)

            title.set_id(title_id)
            title.set_container_id(cont.get_id())
            title_name = ('<span weight="bold">%s</span>' % title.get_name())
            model.set_value(iter_title, COLUMN_TREE_TITLE, title_name)

            title_id += 1
            iter_title = model.iter_next(iter_title)
        return

    def __append_title(self, iter_title, model, title):
        empty_fname = build_path([winki_path, 'glade', 'empty.png'])
        empty_pixbuf = pixbuf_new_from_file(empty_fname)
        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        title_name = ('<span weight="bold">%s</span>' % title.get_name())
        parent_iter = model.iter_parent(iter_title)
        model.insert_after(parent_iter, iter_title, [title_pixbuf, empty_pixbuf, title_name, title])
        return

    def __insert_tile_as_last(self, model, title):
        iter_cont = model.get_iter((0))
        iter_title = model.iter_children(iter_cont)
        while iter_title:
            if not model.iter_next(iter_title):
                self.__append_title(iter_title, model, title)
                return
            iter_title = model.iter_next(iter_title)

        # the container has no titles at all: we have to create the first child
        empty_fname = build_path([winki_path, 'glade', 'empty.png'])
        empty_pixbuf = pixbuf_new_from_file(empty_fname)
        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        title_name = ('<span weight="bold">%s</span>' % title.get_name())
        model.append(iter_cont, [title_pixbuf, empty_pixbuf, title_name, title])
        return

    def __insert_tile_after(self, cur_iter, model, title):
        self.__append_title(cur_iter, model, title)
        return

    def __insert_tile_after_parent(self, cur_iter, model, title):
        parent_iter = model.iter_parent(cur_iter)
        self.__append_title(parent_iter, model, title)
        return

    def __check_stream_order(self):
        treeview = self.get_widget('treeview_dest')
        model = treeview.get_model()
        iter_cont = model.iter_children(None)
        if not iter_cont:
            debug('PageTree.__check_stream_order()', 'oops: exactly one container expected - no one found', error)
            return False

        # winki would have nothing to do in this case :-)
        iter_title = model.iter_children(iter_cont)
        if not iter_title:
            return False

        # collect the count and type of the streams from each title
        stream_list =[]
        while iter_title:
            iter_stream = model.iter_children(iter_title)
            lst = []
            while iter_stream:
                lst.append(model.get_value(iter_stream, COLUMN_TREE_OBJECT))
                iter_stream = model.iter_next(iter_stream)

            stream_list.append(lst)
            iter_title = model.iter_next(iter_title)

        # winki would have nothing to do in this case :-)
        if len(stream_list) < 1 or len(stream_list[0]) < 1:
            return False

        # test if the stream count differs between titles
        stream_count = map(lambda lst: len(lst), stream_list)
        if len(filter(lambda el: el != stream_count[0], stream_count)) > 0:
            primary = _('Different stream count')
            secondary = _('The number of streams in the constructed titles differ.')
            self.show_error(primary, secondary)
            return False

        # test if the stream types differ between titles
        for stream_nr in range(len(stream_list[0])):
            stream_type = stream_list[0][stream_nr].stream_type()
            for title_nr in range(len(stream_list)):
                if stream_list[title_nr][stream_nr].stream_type() != stream_type:
                    primary = _('Different stream types')
                    secondary = _('The stream types in the constructed titles are not consistent. It is not possible to interconnect streams with different types.')
                    self.show_error(primary, secondary)
                    return False

        return True

    def __sumbit_selected_streams(self):
        treeview = self.get_widget('treeview_dest')
        model = treeview.get_model()
        iter_cont = model.iter_children(None)
        iter_title = model.iter_children(iter_cont)
        while iter_title:
            dest_title = model.get_value(iter_title, COLUMN_TREE_OBJECT)
            if not isinstance(dest_title, Title):
                debug('PageTree.__submit_selected_streams()', 'expecting title got someting else', error)
                continue

            vstreams = []
            astreams = []
            sstreams = []
            iter_stream = model.iter_children(iter_title)
            while iter_stream:
                stream = model.get_value(iter_stream, COLUMN_TREE_OBJECT)
                if stream.stream_type() == TYPE_VIDEO:
                    vstreams.append(stream)
                elif stream.stream_type() == TYPE_AUDIO:
                    astreams.append(stream)
                elif stream.stream_type() == TYPE_SUBTITLE:
                    sstreams.append(stream)
                else:
                    debug('PageTree.__submit_selected_streams()', 'unexpected stream type', error)
                    continue
                iter_stream = model.iter_next(iter_stream)

            dest_title.reset_video()
            vid = 0
            for video in vstreams:
                video.set_id(vid)
                dest_title.add_video(video)
                vid += 1

            dest_title.reset_audio()
            aid = 0
            for audio in astreams:
                audio.set_id(aid)
                dest_title.add_audio(audio)
                aid += 1

            dest_title.reset_subtitles()
            sid = 0
            for subtitle in sstreams:
                subtitle.set_id(sid)
                dest_title.add_subtitle(subtitle)
                sid += 1

            iter_title = model.iter_next(iter_title)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
