# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.device_info import DeviceInfo, WRITE_CDR, WRITE_CDRW, READ_DVD
from winkirip.device_info import WRITE_DVDR, WRITE_DVDRAM
import re

class InfoItem:
    def __init__(self, lines):
        self.__dic = {}
        self.__regex_line = re.compile(r'(^.+) = (.+)\((.+)\)$')

        first = True
        for line in lines:
            if first:
                first = False
                continue
            self.__parse(line.strip())
        return

    # -- public interface --

    def has_key(self, k):
        try:
            discard = self.__dic[k]
        except KeyError:
            return False
        return True

    def get_value(self, k):
        value = ''
        try:
            value = self.__dic[k]
        except KeyError:
            pass
        return value

    # -- helpers --

    def __parse(self, line):
        if not line or not len(line):
            return

        m = self.__regex_line.match(line)
        if not m or len(m.groups()) != 3:
            assert(False)
            return

        skey, svalue, stype = m.groups()

        if stype == 'string list':
            # todo: the list items have to be unpacked (needed?)
            self.__dic[skey] = svalue
        elif stype == 'int' or stype == 'uint64':
            # todo: the int values are expressed in hex as well as in dec (needed?)
            self.__dic[skey] = svalue
        elif stype == 'string':
                self.__dic[skey] = svalue.strip().replace('\'', '')
        elif stype == 'bool':
            if svalue.strip() == 'true':
                self.__dic[skey] = True
            else:
                self.__dic[skey] = False
        else:
            self.__dic[skey] = svalue
        return

class HALDeviceParser:
    def __init__(self, device_list, lines):
        self.__lines = lines
        self.__info_items = []
        self.__device_list = device_list
        return

    # -- public interface --

    def run(self):
        self.__build_info_items()

        for item in self.__info_items:
            if item.has_key('storage.drive_type') and item.get_value('storage.drive_type') == 'cdrom':
                drive = DeviceInfo()
                drive.set_device_file(item.get_value('block.device').replace('/dev/', ''))
                drive.set_speed(0)
                #drive.set_manufacturer(item.get_value('info.product'))
                drive.set_manufacturer('')
                if item.get_value('storage.cdrom.cdr'):
                    drive.set_capability(WRITE_CDR)
                if item.get_value('storage.cdrom.cdrw'):
                    drive.set_capability(WRITE_CDRW)
                if item.get_value('storage.cdrom.dvd'):
                    drive.set_capability(READ_DVD)
                if item.get_value('storage.cdrom.dvdr'):
                    drive.set_capability(WRITE_DVDR)
                if item.get_value('storage.cdrom.dvdram'):
                    drive.set_capability(WRITE_DVDRAM)
                drive.set_model(item.get_value('storage.model'))
                drive.set_version(item.get_value('storage.firmware_version'))

                self.__device_list.append(drive)
        return

    def get_device_list(self):
        return self.__device_list

    # -- helpers --

    def __build_info_items(self):
        start_line = 0
        end_line = -1
        cur_line = -1
        regexp = re.compile('^\d+: ')

        for line in self.__lines:
            cur_line += 1

            if regexp.match(line.strip()):
                end_line = cur_line
                if end_line <= start_line:
                    continue

                info_item = InfoItem(self.__lines[start_line : end_line])
                self.__info_items.append(info_item)
                start_line = cur_line
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
