# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import events_pending, main_iteration


class IMDBParser:
    def __init__(self, lines):
        self.__lines = lines
        self.__result = {}

        self.__func = self.__dummy
        
        # todo: make them visible in the output? icons? something else?
        # (TV) = made for TV (TV-movies, MOWs, pilots, TV specials)
        # (V) = made for video or direct-to-video release
        # (VG) = video game
        # (mini) = television miniseries
        self.__noise_title = ['(TV)', '(V)', '(VG)', '(mini)']

        # (roman letter) = database key helper (if the name is already there); 30 should be ok for now
        # (r) = ??? (found in composers.list)
        self.__noise_name = ['(I)', '(II)', '(III)', '(IV)', '(V)',
                        '(VI)', '(VII)', '(VIII)', '(IX)', '(X)',
                        '(XI)', '(XII)', '(XIII)', '(XIV)',
                        '(XV)', '(XVI)', '(XVII)', '(XVIII)',
                        '(XIX)', '(XX)', '(XXI)', '(XXII)', '(XXIII)',
                        '(XXIV)', '(XXV)', '(XXVI)', '(XXVII)',
                        '(XXVIII)', '(XXIX)', '(XXX)', '(r)']

        self.__states = {"Unknown" : self.__dummy,
                         "Genre" : self.__get_genre,
                         "Genres" : self.__get_genre,
                         "Keyword" : self.__get_keyword,
                         "Keywords" : self.__get_keyword,
                         "Produced by" : self.__get_producer,
                         "Producer" : self.__get_producer,
                         "Producers" : self.__get_producer,
                         "Directing" : self.__get_director,
                         "Director" : self.__get_director,
                         "Directors" : self.__get_director,
                         "Writing" : self.__get_writer,
                         "Writer" : self.__get_writer,
                         "Writers" : self.__get_writer,
                         "Score" : self.__get_composer,
                         "Composer" : self.__get_composer,
                         "Composers" : self.__get_composer,
                         "Soundtrack" : self.__dummy,
                         "Soundtracks" : self.__dummy,
                         "Cinematography" : self.__get_cinematographer,
                         "Cinematographer" : self.__get_cinematographer,
                         "Cinematographers" : self.__get_cinematographer,
                         "Editing" : self.__get_editor,
                         "Editor" : self.__get_editor,
                         "Editors" : self.__get_editor,
                         "Cast" : self.__get_cast,
                         "Casting" : self.__get_cast,
                         "Complete Cast" : self.__get_cast,
                         "Verified Complete Cast" : self.__get_cast,
                         "Remainder of Cast" : self.__get_cast,
                         "Actor" : self.__get_cast,
                         "Acting" : self.__get_cast,
                         "Actors" : self.__get_cast,
                         "Actress" : self.__get_cast,
                         "Country" : self.__get_country,
                         "Country of Production" : self.__get_country,
                         "Language" : self.__get_language,
                         "Languages" : self.__get_language,
                         "Summary" : self.__dummy,
                         "Summary by" : self.__dummy,
                         "Production Design" : self.__dummy,
                         "Production Designer" : self.__dummy,
                         "Production Designers" : self.__dummy,
                         "Costume Design" : self.__dummy,
                         "Costume Designer" : self.__dummy,
                         "Costume Designers" : self.__dummy,
                         "Art Direction" : self.__dummy,
                         "Art Director" : self.__dummy,
                         "Art Directors" : self.__dummy,
                         "Set Decoration" : self.__dummy,
                         "Set Decorator" : self.__dummy,
                         "Set Decorators" : self.__dummy,
                         "Stunts" : self.__dummy,
                         "Stunt Person" : self.__dummy,
                         "Stunt People" : self.__dummy,
                         "Assistant Directing" : self.__dummy,
                         "Assistant Director" : self.__dummy,
                         "Assistant Directors" : self.__dummy,
                         "Production Managing" : self.__dummy,
                         "Production Manager" : self.__dummy,
                         "Production Managers" : self.__dummy,
                         "Production Company" : self.__dummy,
                         "Sound" : self.__dummy,
                         "Member of Sound Department" : self.__dummy,
                         "Sound Department" : self.__dummy,
                         "Member of Special Effects Department" : self.__dummy,
                         "Special Effects Department" : self.__dummy,
                         "Special Effects" : self.__dummy,
                         "Member of Make-Up Department" : self.__dummy,
                         "Make-Up Department" : self.__dummy,
                         "Make-Up" : self.__dummy,
                         "Member of Art Department" : self.__dummy,
                         "Art Department" : self.__dummy,
                         "Member of Visual Effects Department" : self.__dummy,
                         "Visual Effects Department" : self.__dummy,
                         "Visual Effects" : self.__dummy,
                         "Miscellaneous" : self.__dummy,
                         "Casting Director" : self.__dummy,
                         "Casting by" : self.__dummy,
                         "Casting Directors" : self.__dummy,
                         "Trivia" : self.__dummy,
                         "Crazy Credits" : self.__dummy,
                         "Goofs" : self.__dummy,
                         "Quotes" : self.__dummy,
                         "Alternate Versions" : self.__dummy,
                         "Tag Line" : self.__dummy,
                         "Tag Lines" : self.__dummy,
                         "Distributor" : self.__dummy,
                         "Certificates" : self.__dummy,
                         "Running Time" : self.__dummy,
                         "Running Times" : self.__dummy,
                         "Color Information" : self.__dummy,
                         "Filmed In" : self.__dummy,
                         "Sound Mix" : self.__dummy,
                         "Technical" : self.__dummy,
                         "Technical Info" : self.__dummy,
                         "Release Date" : self.__dummy,
                         "Release Dates" : self.__dummy,
                         "Locations" : self.__dummy,
                         "Special Effects Company" : self.__dummy,
                         "Miscellaneous Company" : self.__dummy,
                         "Literature" : self.__dummy,
                         "Business Info" : self.__dummy,
                         "Movie Links" : self.__dummy,
                         "URL" : self.__dummy}
        return

    # -- public interface --

    def run(self):
        self.__result = {}

        for line in self.__lines:
            while events_pending():
                main_iteration(False)

            if not line or not len(line):
                continue

            keyword = self.__find_keyword(line)
            if self.__switch_state(keyword):
                continue

            stripped = line.strip()
            decoded = stripped.decode('iso-8859-1')
            self.__func(decoded.encode('utf-8'))
        return

    def get_data(self):
        return self.__result

    # -- helpers --

    def __find_keyword(self, line):
        if not line or not len(line):
            return None

        colon_pos = line.find(':')
        if colon_pos > -1 and line[0] != ' ' and line[0] != '\t':
            return line[ : colon_pos].strip()
            
        return None

    def __switch_state(self, keyword):
        if not keyword:
            return False

        for key in self.__states:
            if key == keyword:
                self.__func = self.__states[key]
                return True

        return False

    def __dummy(self, line):
        return

    def __get_genre(self, line):
        genres = []
        try:
            genres = self.__result['genre']
        except:
            pass

        items = line.split()
        for item in items:
            genres.append(item)

        self.__result['genre'] = genres
        return

    def __get_keyword(self, line):
        keywords = []
        try:
            keywords = self.__result['keywords']
        except:
            pass

        items = line.split()
        for item in items:
            keywords.append(item)

        self.__result['keywords'] = keywords
        return

    def __get_producer(self, line):
        producers = []
        try:
            producers = self.__result['producers']
        except:
            pass

        producers.append(self.__denoise_name(line))

        self.__result['producers'] = producers
        return

    def __get_director(self, line):
        directors = []
        try:
            directors = self.__result['directors']
        except:
            pass

        directors.append(self.__denoise_name(line))

        self.__result['directors'] = directors
        return

    def __get_writer(self, line):
        writers = []
        try:
            writers = self.__result['writers']
        except:
            pass

        writers.append(self.__denoise_name(line))

        self.__result['writers'] = writers
        return

    def __get_composer(self, line):
        composers = []
        try:
            composers = self.__result['composers']
        except:
            pass

        composers.append(self.__denoise_name(line))

        self.__result['composers'] = composers
        return

    def __get_cinematographer(self, line):
        cines = []
        try:
            cines = self.__result['cinematographers']
        except:
            pass

        cines.append(self.__denoise_name(line))

        self.__result['cinematographers'] = cines
        return

    def __get_editor(self, line):
        editors = []
        try:
            editors = self.__result['editors']
        except:
            pass

        editors.append(self.__denoise_name(line))

        self.__result['editors'] = editors
        return

    def __get_cast(self, line):
        cast = []
        try:
            cast = self.__result['cast']
        except:
            pass

        cast.append(self.__split_cast(line))

        self.__result['cast'] = cast
        return

    def __get_country(self, line):
        countries = []
        try:
            countries = self.__result['countries']
        except:
            pass

        items = line.split()
        for item in items:
            countries.append(item)

        self.__result['countries'] = countries
        return

    def __get_language(self, line):
        languages = []
        try:
            languages = self.__result['languages']
        except:
            pass

        items = line.split()
        for item in items:
            languages.append(item)

        self.__result['languages'] = languages
        return

    def __split_cast(self, line):
        result = ''

        start_pos = line.find('..')
        bracket_pos = line.find('(')
        #end_pos = line.rfind('..')

        if bracket_pos > -1 and bracket_pos < start_pos:
            start_pos = bracket_pos

        if start_pos > -1:
            result = line[ : start_pos].strip()
        else:
            result = line

        #if end_pos > -1 and (end_pos + 2) < len(line):
        #   result[1] = line[end_pos + 2: ].strip()

        return self.__denoise_name(result)

    def __denoise_name(self, name):
        for noise in self.__noise_name:
            name = name.replace(noise, '')
        return name

    def __denoise_title(self, title):
        for noise in self.__noise_title:
            title = title.replace(noise, '')
        return title

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
