# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import events_pending, main_iteration
from winkirip.lib import to_int
from winkirip.device_info import DeviceInfo, WRITE_CDR, WRITE_CDRW
from winkirip.device_info import READ_DVD, WRITE_DVDR, WRITE_DVDRAM

class ProcParser:
    def __init__(self, lines):
        self.__lines = lines
        self.__device_list = []
        self.__dispatch = { 'drive name' : self.__set_name,
                            'drive speed' : self.__set_speed,
                            'Can write CD-R' : self.__write_cdr,
                            'Can write CD-RW' : self.__write_cdrw,
                            'Can read DVD' : self.__read_dvd,
                            'Can write DVD-R' : self.__write_dvdr,
                            'Can write DVD-RAM' : self.__write_dvdram}
        return

    # -- public interface --

    def run(self):
        self.__device_list = []

        for line in self.__lines:
            for keyword in self.__dispatch:
                if line.find(keyword) > -1:
                    self.__dispatch[keyword](line)
        return

    def get_device_list(self):
        return self.__device_list

    # -- helpers --

    def __get_values(self, line):
        keyword, text = line.split(':')
        result = text.split()
        return result

    def __set_name(self, line):
        devices = self.__get_values(line)

        for device in devices:
            drive = DeviceInfo()
            drive.set_device_file(device)
            self.__device_list.append(drive)
        return

    def __set_speed(self, line):
        speed = self.__get_values(line)

        assert(len(speed) == len(self.__device_list))

        for i in range(len(self.__device_list)):
            self.__device_list[i].set_speed(to_int(speed[i]))
        return

    def __write_cdr(self, line):
        capa = self.__get_values(line)

        assert(len(capa) == len(self.__device_list))

        for i in range(len(self.__device_list)):
            if capa[i] == '1':
                self.__device_list[i].set_capability(WRITE_CDR)
        return

    def __write_cdrw(self, line):
        capa = self.__get_values(line)

        assert(len(capa) == len(self.__device_list))

        for i in range(len(self.__device_list)):
            if capa[i] == '1':
                self.__device_list[i].set_capability(WRITE_CDRW)
        return

    def __read_dvd(self, line):
        capa = self.__get_values(line)

        assert(len(capa) == len(self.__device_list))

        for i in range(len(self.__device_list)):
            if capa[i] == '1':
                self.__device_list[i].set_capability(READ_DVD)
        return

    def __write_dvdr(self, line):
        capa = self.__get_values(line)

        assert(len(capa) == len(self.__device_list))

        for i in range(len(self.__device_list)):
            if capa[i] == '1':
                self.__device_list[i].set_capability(WRITE_DVDR)
        return

    def __write_dvdram(self, line):
        capa = self.__get_values(line)

        assert(len(capa) == len(self.__device_list))

        for i in range(len(self.__device_list)):
            if capa[i] == '1':
                self.__device_list[i].set_capability(WRITE_DVDRAM)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
