# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.config import Config
from winkirip.lang import ISOCodes
from winkirip.tool_checker import ToolChecker
from winkirip.container import CONTAINER_UNKNOWN

class Project:
    def __init__(self):
        self.__source = [] # list of source containers
        self.__dest = [] # list of dest containers
        self.__enabled_titles = [] # list of source titles to be considered for input (not all source titles are such)

        self.__config = Config()
        self.__config.load()

        self.__iso_codes = ISOCodes()
        self.__iso_codes.load()

        self.__tool_checker = ToolChecker(self.__config)
        self.__tool_checker.load()

        self.__dest_dir = None
        return

    # -- get/set functions --

    def get_source(self):
        return self.__source

    def get_source_type(self):
        if not len(self.__source):
            return CONTAINER_UNKNOWN

        return self.__source[0].get_type()

    def get_dest(self):
        return self.__dest

    def get_dest_type(self):
        if not len(self.__dest):
            return CONTAINER_UNKNOWN

        return self.__dest[0].get_type()

    def get_enabled_titles(self):
        return self.__enabled_titles

    def get_config(self):
        return self.__config

    def get_iso_codes(self):
        return self.__iso_codes

    def get_tool_checker(self):
        return self.__tool_checker

    # -- public interface --

    def add_source(self, container):
        self.__source.append(container)
        return

    def add_dest(self, container):
        cont_id = len(self.__dest)
        container.set_id(cont_id)
        self.__dest.append(container)
        return

    def add_enabled_title(self, title):
        self.__enabled_titles.append(title)
        return

    def reset_source(self):
        self.__source = []
        return

    def reset_dest(self):
        self.__dest = []
        return

    def reset_enabled_titles(self):
        self.__enabled_titles = []
        return

    def set_dest_dir(self, dest_dir):
        self.__dest_dir = dest_dir
        return

    def get_dest_dir(self):
        return self.__dest_dir

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
