# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.scan import Scan
from winkirip.command import Command


class QueryIMDB(Scan):

    def __init__(self, container, tool_checker):
        Scan.__init__(self, container, tool_checker)
        self.__title_name = ''
        self.__lines = []
        return

    # -- methods inherited from scan.Scan --

    def run(self):
        if not self.__query_title() or self.is_canceled():
            return False

        # prevent parsing the imdb copyright note as film info
        for i in range(3):
            self.__lines.pop()
        return True

    def reset(self):
        Scan.reset(self)
        self.__lines = []
        return

    # -- public interface --

    def set_title(self, title_name):
        self.__title_name = title_name
        return

    def get_result(self):
        return self.__lines

    # -- helpers --

    def __query_title(self):
        if not self.__title_name or not len(self.__title_name):
            return False

        self.reset()

        name_escaped = self.__title_name.replace('"', '\\"')

        cmd_text = 'title -t "%s" -full' % name_escaped
        cmd = Command(cmd_text, self.__accumulate_lines)
        cmd.run()
        if cmd and cmd.get_status() != 0:
            self.reset()
            return False

        # todo: delete the last 2 or 3 lines
        return True

    def __accumulate_lines(self, line):
        items = line.split('\n')
        for item in items:
            decoded = item.decode('iso-8859-15')
            self.__lines.append(decoded.encode('utf-8'))
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
