# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.lib import debug, error
from winkirip.tool_checker import TOOL_EXTERNAL


class Scan:
    def __init__(self, container, tool_checker):
        self.__canceled = False
        self.__container = container
        self.__tool_checker = tool_checker
        self.__progress_callback = None
        return

    # -- get/set --

    def get_container(self):
        return self.__container

    def set_progress(self, prog):
        if self.__progress_callback:
            self.__progress_callback(prog)
        return

    def set_progress_callback(self, prog):
        self.__progress_callback = prog
        return

    def has_tool(self, toolname, tool_type = TOOL_EXTERNAL):
        return self.__tool_checker.has_tool(toolname, tool_type)

    def get_tool(self, toolname, tool_type = TOOL_EXTERNAL):
        return self.__tool_checker.get_tool(toolname, tool_type)

    def get_tool_location(self, toolname, lang_neutral = True):
        return self.__tool_checker.get_tool_location(toolname, lang_neutral)

    # -- public interface --

    def run(self):
        debug('ScanMedia.run(): not overwritten', error)
        assert(False)
        return False

    def cancel(self):
        self.__canceled = True
        return

    def is_canceled(self):
        return self.__canceled

    def reset(self):
        self.__canceled = False
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
