# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.scan import Scan
from winkirip.command import Command


class ScanIMDB(Scan):

    def __init__(self, container, tool_checker):
        Scan.__init__(self, container, tool_checker)
        self.__command = None
        self.__success = True
        self.__title_name = ''
        self.__lines = []
        return

    # -- methods inherited from scan.Scan --

    def run(self):
        self.__success = (self.__query_titles() and not self.is_canceled())
        return self.__success

    def reset(self):
        Scan.reset(self)
        self.__command = None
        self.__lines = []
        return

    # -- public interface --

    def set_title(self, title_name):
        self.__title_name = title_name
        return

    def get_result(self):
        return self.__lines

    def get_success(self):
        return self.__success

    # -- helpers --

    def __query_titles(self):
        if not self.__title_name or not len(self.__title_name):
            return False

        self.reset()

        if not self.has_tool('lindex'):
            debug('ScanIMDB.__query_titles()', 'cannot find lindex executable', error)
            return False

        prog = self.get_tool_location('lindex')

        cmd_text = '%s -title -s "%s"' % (prog, self.__title_name)
        self.__command = Command(cmd_text, self.__accumulate_lines)
        self.__command.set_bufsize(20)
        self.__command.run()
        if self.__command and self.__command.get_status() != 0:
            self.reset()
            return False

        return True

    def __accumulate_lines(self, line):
        if self.is_canceled():  
            if self.__command:
                self.__command.stop()
                self.__command = None
            return

        items = line.split( '\n')
        for item in items:
            if not item or not len(item):
                continue

            decoded = item.decode('iso-8859-1')
            self.__lines.append(decoded.encode('utf-8'))
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
