# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import Window, WINDOW_TOPLEVEL, VBox, HBox, Image, Label, ComboBox
from gtk import Table, ComboBoxEntry, SpinButton, Entry, Button, Alignment
from gtk import Adjustment, HSeparator, STOCK_CLOSE
from winkirip.lib import build_path, winki_path

SMALL_SPACE = 2
BIG_SPACE = 6

# I hate to build interfaces that way (especially now that glade has been
# really improved and does not crash anymore) but there is no other easy way to
# deal with multiple repeating sections. So here we go...

class PropFrame:
    def __init__(self, parent):
        self.__wnd = None
        self.__target = None
        self.__parent = parent
        return

    def create_window(self):
        self.__wnd = Window(WINDOW_TOPLEVEL)
        self.__wnd.connect("delete_event", self.on_no_close)
        self.__wnd.set_border_width(BIG_SPACE)
        self.__wnd.set_title(_('Source Properties'))

        fname = build_path([winki_path, 'glade', 'winki.png'])
        self.__wnd.set_icon_from_file(fname)

        self.__target = VBox(False, BIG_SPACE)
        self.__target.set_homogeneous(False)
        return

    def add_section(self, sect):
        self.__target.pack_start(sect.get_ui(), expand = False)
        return

    def add_bottom_section(self):
        self.__target.pack_start(self.__close_section(), expand = False)
        self.__wnd.add(self.__target)
        return

    def show(self):
        self.__target.show()
        self.__wnd.show()
        return

    def hide(self):
        self.__parent.hide()
        self.__wnd.hide()
        return

    def get_window(self):
        return self.__wnd

    # --- signal handlers ---

    def on_no_close(self, widget, event):
        return True

    def on_close(self, button):
        self.hide()
        return True

    # --- helpers ---

    def __close_section(self):
        result = Alignment()
        result.set(1, 1, 0, 0)
        result.set_padding(BIG_SPACE, SMALL_SPACE, 0, SMALL_SPACE)

        button = Button(stock = STOCK_CLOSE)
        button.connect("clicked", self.on_close)
        result.add(button)

        button.show()
        result.show()
        return result


class UISection:
    def __init__(self):
        return

    # --- public interface ---

    def get_header_pack(self, icon, text):
        result = HBox(False, BIG_SPACE)
        result.set_homogeneous(False)

        fname = build_path([winki_path, 'glade', icon])
        image = Image()
        image.set_from_file(fname)
        result.pack_start(image, fill = False, expand = False)
        image.show()

        label = Label()
        label.set_alignment(xalign = 0, yalign = 0)
        label.set_property('use-markup', True)
        label.set_property('label', '<b>%s</b>' % text)
        result.pack_start(label, expand = False)
        label.show()

        result.show()
        return result

    def get_combo_pack(self, text):
        label = Label(text)
        label.set_alignment(xalign = 0, yalign = 0.5)
        label.show()

        combo = ComboBox()
        combo.show()

        return (label, combo)

    def get_combo_box_pack(self, text):
        label = Label(text)
        label.set_alignment(xalign = 0, yalign = 0.5)
        label.show()

        combo = ComboBoxEntry()
        combo.show()

        return (label, combo)

    def get_spin_pack(self, text, value, lower, upper, si, pi, ps, unit = None):
        label = Label(text)
        label.set_alignment(xalign = 0, yalign = 0.5)
        label.show()

        adj = Adjustment(value, lower, upper, si, pi, ps)
        spin = SpinButton(adj)
        spin.show()

        if unit:
            label_unit = Label(unit)
            label_unit.set_alignment(xalign = 0, yalign = 0.5)
            label_unit.show()
            return (label, spin, label_unit)

        return (label, spin)

    def get_entry_pack(self, text, show_image = False, unit = None):
        label = Label(text)
        label.set_alignment(xalign = 0, yalign = 0.5)
        label.show()

        entry = Entry()
        entry.show()

        if show_image:
            img = Image()
            img.show()
            return (label, entry, img)

        if unit:
            label_unit = Label(unit)
            label_unit.set_alignment(xalign = 0, yalign = 0.5)
            label_unit.show()
            return (label, entry, label_unit)

        return (label, entry)

    def get_table(self, rows, columns):
        result = Table(rows, columns)
        result.set_row_spacings(5)
        result.set_col_spacings(10)
        result.set_homogeneous(False)

        result.show()
        return result

    def table_insert(self, table, items, row, col):
        if len(items) == 2:
            item1, item2 = items
            table.attach(item1, col, col + 1, row, row + 1)
            table.attach(item2, col + 1, col + 2, row, row + 1)

        else:
            item1, item2, item3 = items
            box = HBox(False, SMALL_SPACE)
            box.pack_start(item2, expand = False)
            box.pack_start(item3, expand = False)
            box.set_homogeneous(False)
            box.show()

            table.attach(item1, col, col + 1, row, row + 1)
            table.attach(box, col + 1, col + 2, row, row + 1)

        return

    def table_insert_img(self, table, items, row, col):
        item1, item2, item3 = items

        box = HBox(False, SMALL_SPACE)
        box.set_homogeneous(False)
        box.pack_start(item3, expand = False)
        box.pack_start(item2, expand = False)
        box.show()

        self.table_insert(table, (item1, box), row, col)
        return

class VideoSection(UISection):
    def __init__(self):
        UISection.__init__(self)
        self.__format = None
        self.__aspect = None
        self.__len = None
        self.__width = None
        self.__height = None
        self.__fps = None
        return

    # --- public interface ---

    def get_ui(self):
        result = VBox(False, BIG_SPACE)
        result.set_homogeneous(False)
        
        # first row: label and text
        header_pack = self.get_header_pack('page_title.png', _('Video'))
        result.pack_start(header_pack, expand = False)

        # second row: a table containing all other controls
        table = self.get_table(3, 4)

        l, self.__format = self.get_combo_pack(_('Format'))
        self.table_insert(table, (l, self.__format), 0, 0)

        l, self.__aspect = self.get_combo_box_pack(_('Aspect'))
        self.table_insert(table, (l, self.__aspect), 1, 0)

        l, self.__len, u = self.get_spin_pack(_('Length'), 1, 1, 30000, 10, 100, 100,  _('sec'))
        self.table_insert(table, (l, self.__len, u), 2, 0)

        l, self.__width, u = self.get_spin_pack(_('Width'), 1, 1, 5000, 10, 100, 100, _('px'))
        self.table_insert(table, (l, self.__width, u), 0, 2)

        l, self.__height, u = self.get_spin_pack(_('Height'), 1, 1, 5000, 10, 100, 100, _('px'))
        self.table_insert(table, (l, self.__height, u), 1, 2)

        l, self.__fps, u = self.get_entry_pack(_('Frame rate'), unit = _('fps'))
        self.table_insert(table, (l, self.__fps, u), 2, 2)

        result.pack_start(table, expand = False)

        # last row: hseparator
        hsep = HSeparator()
        hsep.show()
        result.pack_start(hsep, expand = False)

        result.show()
        return result

    def get_format(self):
        return self.__format

    def get_aspect(self):
        return self.__aspect

    def get_len(self):
        return self.__len

    def get_width(self):
        return self.__width

    def get_height(self):
        return self.__height

    def get_fps(self):
        return self.__fps


class AudioSection(UISection):
    def __init__(self):
        UISection.__init__(self)
        self.__format = None
        self.__lang = None
        self.__lang_img = None
        self.__len = None
        self.__bitrate = None
        self.__nrchan = None
        self.__freq = None
        return

    # --- public interface ---

    def get_ui(self):
        result = VBox(False, BIG_SPACE)
        result.set_homogeneous(False)
        
        # first row: label and text
        header_pack = self.get_header_pack('page_audio.png', _('Audio'))
        result.pack_start(header_pack, expand = False)

        # second row: a table containing all other controls
        table = self.get_table(3, 4)

        l, self.__format = self.get_combo_pack(_('Format'))
        self.table_insert(table, (l, self.__format), 0, 0)

        l, self.__lang, self.__lang_img = self.get_entry_pack(_('Language'), True)
        self.table_insert_img(table, (l, self.__lang, self.__lang_img), 1, 0)

        l, self.__len, u = self.get_spin_pack(_('Length'), 1, 1, 30000, 10, 100, 100, _('sec'))
        self.table_insert(table, (l, self.__len, u), 2, 0)

        l, self.__bitrate, u = self.get_spin_pack(_('Bitrate'), 1, 1, 200, 1, 10, 100, _('kb/s'))
        self.table_insert(table, (l, self.__bitrate, u), 0, 2)

        l, self.__nrchan = self.get_spin_pack(_('Number channels'), 1, 1, 10, 1, 5, 5)
        self.table_insert(table, (l, self.__nrchan), 1, 2)

        l, self.__freq, u = self.get_spin_pack(_('Frequency'), 1, 1, 200000, 100, 1000, 10, _('Hz'))
        self.table_insert(table, (l, self.__freq, u), 2, 2)

        result.pack_start(table, expand = False)

        # last row: hseparator
        hsep = HSeparator()
        hsep.show()
        result.pack_start(hsep, expand = False)

        result.show()
        return result

    def get_format(self):
        return self.__format

    def get_lang(self):
        return self.__lang

    def get_lang_image(self):
        return self.__lang_img

    def get_len(self):
        return self.__len

    def get_bitrate(self):
        return self.__bitrate

    def get_nrchan(self):
        return self.__nrchan

    def get_freq(self):
        return self.__freq

class SubtitleSection(UISection):
    def __init__(self):
        UISection.__init__(self)
        self.__format = None
        self.__lang = None
        self.__lang_img = None
        self.__encoding = None
        self.__label_encoding = None
        return

    # --- public interface ---

    def get_ui(self):
        result = VBox(False, BIG_SPACE)
        result.set_homogeneous(False)
        
        # first row: label and text
        header_pack = self.get_header_pack('page_subtitle.png', _('Subtitle'))
        result.pack_start(header_pack, expand = False)

        # second row: a table containing all other controls
        table = self.get_table(2, 4)

        l, self.__format = self.get_combo_pack(_('Format'))
        self.table_insert(table, (l, self.__format), 0, 0)

        l, self.__lang, self.__lang_img = self.get_entry_pack(_('Language'), True)
        self.table_insert_img(table, (l, self.__lang, self.__lang_img), 1, 0)

        self.__label_encoding, self.__encoding = self.get_combo_box_pack(_('Encoding'))
        self.table_insert(table, (self.__label_encoding, self.__encoding), 0, 2)

        result.pack_start(table, expand = False)

        # last row: hseparator
        hsep = HSeparator()
        hsep.show()
        result.pack_start(hsep, expand = False)

        result.show()
        return result

    def get_format(self):
        return self.__format

    def get_lang(self):
        return self.__lang

    def get_lang_image(self):
        return self.__lang_img

    def get_encoding(self):
        return self.__encoding

    def get_label_encoding(self):
        return self.__label_encoding

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
