# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import events_pending, main_iteration, ListStore, CellRendererPixbuf
from gtk import CellRendererText, TreeViewColumn
from gtk.gdk import Pixbuf, pixbuf_new_from_file
from gtk.glade import XML
from gobject import TYPE_STRING
from time import sleep
from threading import Thread
from winkirip.lib import build_path, winki_path, check_widgets, debug, error, to_int
from winkirip.scan_imdb import ScanIMDB
from winkirip.query_imdb import QueryIMDB
from winkirip.parser_imdb import IMDBParser


COLUMN_PIXBUF = 0
COLUMN_TITLE = 1
COLUMN_YEAR = 2
COLUMN_FULL = 3

class SelectIMDB:
    def __init__(self, tool_checker):
        self.__xml = None
        self.__state = 'unknown'
        self.__command = None
        self.__thread = None
        self.__result = None
        self.__tool_checker = tool_checker

        xml_path = build_path([winki_path, 'glade', 'imdb_select.glade'])
        try:
            self.__xml = XML(xml_path)
        except:
            print 'SelectIMDB: Cannot load "window1" from %s' % (xml_path)
            raise SystemExit

        dic = {'on_button_close_clicked' : self.on_close,
               'on_button_select_clicked' : self.on_ok,
               'on_button_cancel_search_clicked' : self.on_cancel}
        self.__xml.signal_autoconnect(dic)

        check_widgets(self.__xml, ['button_close',
                                                'button_select',
                                                'button_cancel_search',
                                                'progressbar1', 'treeview1'])

        self.__dialog = self.__xml.get_widget('window1')
        self.__dialog.connect('delete_event', self.no_quit)
        return

    # -- signal handlers --

    def no_quit(self, button, event = None):
        "disallow close"
        return True

    def on_ok(self, widget, event = None):
        if self.__command:
            debug('SelectIMDB.on_ok()', 'command object exist', error)
            return True

        self.__enable_ui(False)
        self.__query_imdb()
        self.__parse_imdb_result()
        return True
        

    def on_close(self, widget, event = None):
        if self.__command:
            debug('SelectIMDB.on_close()', 'command object exist', error)

        self.__result = None
        self.__command = None

        self.__state = 'close'
        self.__hide()
        return True

    def on_cancel(self, widget, event = None):
        if self.__thread:
            if self.__command:
                self.__command.cancel()
                self.__state = 'close'

        self.__enable_ui(True)
        return True

    def get_state(self):
        return self.__state

    # -- public interface --

    def run(self, title_name):
        if self.__command:
            debug('SelectIMDB.run()', 'command object exist', error)
            return False

        self.__result = None

        self.__dialog.show()
        self.__enable_ui(False)

        progressbar = self.__xml.get_widget('progressbar1')
        progressbar.set_pulse_step(0.004)

        self.__state = 'unknown'
        self.__command = ScanIMDB(None, self.__tool_checker)
        self.__command.set_title(title_name)
        self.__thread = Thread(group = None, target = self.__command.run)
        self.__thread.start()

        while self.__thread.isAlive():
            if self.__state != 'unknown':
                break
            while events_pending():
                main_iteration(False)
            sleep(0.005)
            progressbar.pulse()

        progressbar.set_fraction(0.0)
        self.__thread = None

        self.__enable_ui(True)
        self.__show_results()
        return True

    def get_data(self):
        return self.__result

    # -- helpers --

    def __show_results(self):
        self.__state = 'unknown'

        self.__create_model()
        self.__fill_model()
        self.__execute()
        return True

    def __hide(self):
        self.__dialog.hide()
        return True

    def __create_model(self):
        treeview = self.__xml.get_widget('treeview1')
        model = treeview.get_model()
        if model:
            model.clear()
            return

        model = ListStore(Pixbuf, TYPE_STRING, TYPE_STRING, TYPE_STRING)
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_PIXBUF)
        treeview.append_column(column)

        renderer = CellRendererText()
        col = TreeViewColumn(_('Name'), renderer, text = COLUMN_TITLE)
        treeview.append_column(col)

        renderer = CellRendererText()
        col = TreeViewColumn(_('Year'), renderer, text = COLUMN_YEAR)
        treeview.append_column(col)
        return

    def __load_pixmaps(self):
        result = {}

        film_fname = build_path([winki_path, 'glade', 'type_film.png'])
        film_pixbuf = pixbuf_new_from_file(film_fname)

        tv_fname = build_path([winki_path, 'glade', 'type_tv.png'])
        tv_pixbuf = pixbuf_new_from_file(tv_fname)
        result['(TV)'] = tv_pixbuf

        video_fname = build_path([winki_path, 'glade', 'type_video.png'])
        video_pixbuf = pixbuf_new_from_file(video_fname)
        result['(V)'] = video_pixbuf

        game_fname = build_path([winki_path, 'glade', 'type_game.png'])
        game_pixbuf = pixbuf_new_from_file(game_fname)
        result['(VG)'] = game_pixbuf

        mini_fname = build_path([winki_path, 'glade', 'type_tv.png'])
        mini_pixbuf = pixbuf_new_from_file(mini_fname)
        result['(mini)'] = mini_pixbuf

        return result, film_pixbuf

    def __parse_title(self, line, pixbuf_dic, default_pixbuf):
        pixbuf = default_pixbuf
        name = line
        year = _('unknown')

        for key in pixbuf_dic:
            if name.find(key) > -1:
                pixbuf = pixbuf_dic[key]
                name = name.replace(key, '')
                break

        pos1 = name.find('(')
        pos2 = name.find(')')
        if pos1 > -1 and pos2 > -1 and pos1 < pos2:
            temp = name[pos1 + 1 : pos2]
            if to_int(temp) > -1:
                year = temp
                name = name.replace('(%s)' % temp, '')

        name = name.strip()
        return [pixbuf, name, year, line]

    def __fill_model(self):
        treeview = self.__xml.get_widget('treeview1')
        model = treeview.get_model()
        model.clear()

        if not self.__command or not self.__command.get_success():
            return

        pixbuf_dic, default_pixbuf = self.__load_pixmaps()

        title_list = self.__command.get_result()
        for title in title_list:
            model.append(self.__parse_title(title, pixbuf_dic, default_pixbuf))

        progressbar = self.__xml.get_widget('progressbar1')
        if not title_list or not len(title_list):
            progressbar.set_text(_('No matches found'))
        else:
            progressbar.set_text(_('%d matches found') % len(title_list))

        self.__command = None
        return

    def __execute(self):
        while self.__state == 'unknown':
            while events_pending():
                main_iteration(False)
            sleep(0.001)
        return

    def __enable_ui(self, enabled):
        button_close = self.__xml.get_widget('button_close')
        button_select = self.__xml.get_widget('button_select')
        button_cancel = self.__xml.get_widget('button_cancel_search')
        progressbar = self.__xml.get_widget('progressbar1')
        treeview = self.__xml.get_widget('treeview1')

        button_close.set_sensitive(enabled)
        button_select.set_sensitive(enabled)
        treeview.set_sensitive(enabled)
        button_cancel.set_sensitive(not enabled)
        progressbar.set_sensitive(not enabled)
        return

    def __get_choice(self, user_readable = False):
        treeview = self.__xml.get_widget('treeview1')
        selection = treeview.get_selection()

        model, iterator, = selection.get_selected()
        if iterator:
            path = model.get_path(iterator)
            if user_readable:
                return model[path[0]][COLUMN_TITLE]
            else:
                return model[path[0]][COLUMN_FULL]

        return ''

    def __query_imdb(self):
        if self.__command:
            debug('SelectIMDB.__query_imdb()', 'command object exist', error)
            return

        progressbar = self.__xml.get_widget('progressbar1')

        self.__state = 'unknown'
        self.__command = QueryIMDB(None, self.__tool_checker)
        self.__command.set_title(self.__get_choice())
        self.__thread = Thread(group = None, target = self.__command.run)
        self.__thread.start()
        
        while self.__thread.isAlive():
            if self.__state != 'unknown':
                break
            while events_pending():
                main_iteration(False)
            sleep(0.005)
            progressbar.pulse()

        progressbar.set_fraction(0.0)
        self.__thread = None
        return

    def __parse_imdb_result(self):
        if not self.__command:
            debug('SelectIMDB.__parse_imdb_result()', 'command object does not exist', error)
            return False

        parser = IMDBParser(self.__command.get_result())
        parser.run()

        self.__result = parser.get_data()
        if not self.__result or not len(self.__result):
            self.__state = 'close'
            self.__command = None
            self.__hide()
            return False

        self.__result['title'] = [self.__get_choice(True)]
        self.__result['comment'] = ['Information courtesy of The Internet Movie Database (http://www.imdb.com). Used with permission.']
        self.__command = None

        self.__state = 'select'
        self.__hide()
        return True

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
