# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import STOCK_YES, STOCK_NO, ICON_SIZE_BUTTON
import gtk
from gtk.glade import XML
from winkirip.lib import build_path, winki_path, check_widgets, debug, error
from winkirip.lib import textcombo_setup, textcombo_get_value, textcombo_set_value
from winkirip.container import CONTAINER_UNKNOWN, FILE_MATROSKA, FILE_AVI
from winkirip.container import FILE_MP4, FILE_MOBILE
from winkirip.container_video import VIDEO_MPEG1, VIDEO_MPEG2, VIDEO_MPEG4
from winkirip.container_video import VIDEO_XVID, VIDEO_H264, VIDEO_MOBILE
from winkirip.container_audio import AUDIO_PCM, AUDIO_AC3, AUDIO_OGG, AUDIO_MOBILE
from winkirip.container_audio import AUDIO_MP2, AUDIO_MP3, AUDIO_FLAC
from winkirip.container_audio import AUDIO_AAC
from winkirip.tool_checker import TOOL_EXTERNAL, TOOL_MENCODER
import os.path

PRESET_FILE_NONE = 0
PRESET_FILE_QUALITY = 1
PRESET_FILE_COMPRESSION = 2
PRESET_FILE_SPEED = 3
PRESET_FILE_IPOD = 4
PRESET_FILE_MOBILE = 5

PRESET_DVD_NONE = 0
PRESET_DVD_QUALITY = 1
PRESET_DVD_SPEED = 2

class SelectSettings:
    def __init__(self, config, tool_checker):
        self.__xml = None
        self.__config = config
        self.__tool_checker = tool_checker
        self.__ignore_update = False
        self.__store_position_func = None
        self.__restore_position_func = None
        self.__callback = None

        xml_path = build_path([winki_path, 'glade', 'settings.glade'])
        try:
            self.__xml = XML(xml_path)
        except:
            print 'SelectSettings: Cannot load "window1" from %s' % (xml_path)
            raise SystemExit

        dic = {'on_button_close_clicked' : self.on_close,
               'on_combobox_video_resolution_dvd_changed' : self.on_video_resolution_dvd_changed,
               'on_combobox_video_format_dvd_changed' : self.on_dvd_changed,
               'on_combobox_video_aspect_dvd_changed' : self.on_vdeo_aspect_dvd_changed,
               'on_combobox_video_encoding_dvd_changed' : self.on_dvd_changed,
               'on_combobox_audio_format_dvd_changed' : self.on_dvd_changed,
               'on_combobox_dvd_presets_changed' : self.on_dvd_presets_changed,
               'on_combobox_file_format_changed' : self.on_file_format_changed,
               'on_combobox_video_format_file_changed' : self.on_file_changed,
               'on_combobox_video_resolution_file_changed' : self.on_file_changed,
               'on_combobox_video_aspect_file_changed' : self.on_file_changed,
               'on_combobox_video_encoding_file_changed' : self.on_file_changed,
               'on_combobox_audio_format_file_changed' : self.on_audio_format_file_changed,
               'on_combobox_audio_quality_file_changed' : self.on_audio_quality_file_changed,
               'on_combobox_file_presets_changed' : self.on_file_presets_changed,
               'on_button_output_folder_clicked' : self.on_output_folder_clicked }
        self.__xml.signal_autoconnect(dic)

        check_widgets(self.__xml, ['button_close'])

        self.__dialog = self.__xml.get_widget('window1')
        self.__dialog.connect('delete_event', self.no_quit)

        self.__init_controls()
        return

    # -- signal handlers --

    def no_quit(self, button, event = None):
        "disallow close"
        return True

    def on_close(self, widget, event = None):
        self.hide()
        self.__update_config()

        # if anyone cares notify for the changes
        if self.__callback:
            self.__callback()
        return True

    def on_video_resolution_dvd_changed(self, widget, event = None):
        text = textcombo_get_value(widget)
        if not len(text):
            return False

        self.__adapt_video_aspect_dvd(text)
        self.__adapt_video_shrink_dvd(text)
        return True

    def on_vdeo_aspect_dvd_changed(self, widget, event = None):
        text = textcombo_get_value(widget)
        if not len(text):
            return False

        self.__adapt_video_shrink_dvd(text)
        return True

    def on_dvd_changed(self, widget, event = None):
        if not self.__ignore_update:
            self.__ignore_update = True
            self.__combo_set_value('combobox_dvd_presets', [PRESET_DVD_NONE])
            self.__ignore_update = False
        return True

    def on_dvd_presets_changed(self, combo, event = None):
        self.__ignore_update = True
        value = textcombo_get_value(combo)

        if value == PRESET_DVD_QUALITY:
            self.__combo_set_value('combobox_video_format_dvd', [VIDEO_MPEG2])
            self.__combo_set_value('combobox_video_resolution_dvd', ['original'])
            self.__combo_set_value('combobox_video_aspect_dvd', ['original'])
            self.__combo_set_value('combobox_video_encoding_dvd', ['2 pass'])
            # pcm have the best quality but it's just a waste of space in most cases...
            #self.__combo_set_value('combobox_audio_format_dvd', ['AC3', 'MP2', 'PCM'])
            self.__combo_set_value('combobox_audio_format_dvd', [AUDIO_AC3, AUDIO_MP2])

        elif value == PRESET_DVD_SPEED:
            self.__combo_set_value('combobox_video_format_dvd', [VIDEO_MPEG2])
            self.__combo_set_value('combobox_video_resolution_dvd', ['original'])
            self.__combo_set_value('combobox_video_aspect_dvd', ['original'])
            self.__combo_set_value('combobox_video_encoding_dvd', ['requantize', '1 pass'])
            self.__combo_set_value('combobox_audio_format_dvd', [AUDIO_AC3, AUDIO_MP2])

        self.__ignore_update = False
        return True

    def on_file_format_changed(self, widget, event = None):
        # oops - todo: this prevents setting the mobile preset
        #if self.__ignore_update:
        #    return True

        combo_audio_format_file = self.__xml.get_widget('combobox_audio_format_file')
        combo_video_format_file = self.__xml.get_widget('combobox_video_format_file')
        combo_video_resolution_file = self.__xml.get_widget('combobox_video_resolution_file')
        combo_video_aspect_file = self.__xml.get_widget('combobox_video_aspect_file')
        combo_video_shrink = self.__xml.get_widget('combobox_video_encoding_file')
        file_format = textcombo_get_value(widget)

        audio_format_file = self.__config.get_audio_format_file()
        audio_formats_file = self.__create_audio_values(file_format)
        textcombo_setup(combo_audio_format_file, audio_formats_file, audio_format_file)

        video_format_file = self.__config.get_video_format_file()
        video_formats_file = self.__create_video_values(file_format)
        textcombo_setup(combo_video_format_file, video_formats_file, video_format_file)

        video_resolution_file = self.__config.get_video_resolution_file()
        video_resolutions_file = self.__create_video_resol_values(file_format)
        textcombo_setup(combo_video_resolution_file, video_resolutions_file, video_resolution_file)

        video_aspect_file = self.__config.get_video_aspect_file()
        video_aspects_file = self.__create_video_aspect_values(file_format)
        textcombo_setup(combo_video_aspect_file, video_aspects_file, video_aspect_file)

        video_shrink = self.__config.get_video_shrink_method_file()
        shrink_methods = self.__create_shrink_method_values(file_format)
        textcombo_setup(combo_video_shrink, shrink_methods, video_shrink)
        return True

    def on_audio_format_file_changed(self, widget, event = None):
        value = textcombo_get_value(widget)
        combo = self.__xml.get_widget('combobox_audio_quality_file')

        self.__setup_file_quality_combo(combo, value)
        self.on_file_changed(widget)
        return True

    def on_audio_quality_file_changed(self, widget, event = None):
        value = textcombo_get_value(widget)

        combo = self.__xml.get_widget('combobox_audio_format_file')
        aformat = textcombo_get_value(combo)

        if aformat == AUDIO_OGG:
            self.__config.set_ogg_quality(value)
        elif aformat == AUDIO_MP3:
            self.__config.set_mp3_quality(value)
        elif aformat == AUDIO_MP2:
            self.__config.set_mp2_quality(value)
        elif aformat == AUDIO_AC3:
            self.__config.set_ac3_quality(value)
        elif aformat == AUDIO_AAC:
            self.__config.set_aac_quality(value)

        self.on_file_changed(widget)
        return True

    def on_file_changed(self, widget, event = None):
        if not self.__ignore_update:
            self.__ignore_update = True
            self.__combo_set_value('combobox_file_presets', [PRESET_FILE_NONE])
            self.__ignore_update = False
        return True

    def on_file_presets_changed(self, combo, event = None):
        if self.__ignore_update:
            return True

        self.__ignore_update = True
        value = textcombo_get_value(combo)

        file_formats = self.__create_file_format_values(value)
        combo_file_format = self.__xml.get_widget('combobox_file_format')
        textcombo_setup(combo_file_format, file_formats, 'blah')

        if value == PRESET_FILE_QUALITY:
            self.__combo_set_value('combobox_video_format_file', [VIDEO_MPEG2, VIDEO_XVID, VIDEO_H264, VIDEO_MPEG4])
            self.__combo_set_value('combobox_video_resolution_file', ['original'])
            self.__combo_set_value('combobox_video_aspect_file', ['original'])
            self.__combo_set_value('combobox_video_encoding_file', ['2 pass'])
            self.__combo_set_value('combobox_audio_format_file', [AUDIO_AC3, AUDIO_OGG, AUDIO_MP2, AUDIO_MP3, AUDIO_AAC])

        elif value == PRESET_FILE_SPEED:
            self.__combo_set_value('combobox_video_format_file', [VIDEO_XVID, VIDEO_MPEG4, VIDEO_MPEG2])
            self.__combo_set_value('combobox_video_resolution_file', ['original'])
            self.__combo_set_value('combobox_video_aspect_file', ['original'])
            self.__combo_set_value('combobox_video_encoding_file', ['1 pass'])
            self.__combo_set_value('combobox_audio_format_file', [AUDIO_MP3, AUDIO_OGG, AUDIO_MP2, AUDIO_AC3, AUDIO_AAC])
            
        elif value == PRESET_FILE_COMPRESSION:
            self.__combo_set_value('combobox_file_format', [FILE_MATROSKA])
            self.__combo_set_value('combobox_video_format_file', [VIDEO_XVID, VIDEO_H264, VIDEO_MPEG4, VIDEO_MPEG2])
            self.__combo_set_value('combobox_video_resolution_file', ['original'])
            self.__combo_set_value('combobox_video_aspect_file', ['original'])
            self.__combo_set_value('combobox_video_encoding_file', ['2 pass'])
            self.__combo_set_value('combobox_audio_format_file', [AUDIO_OGG, AUDIO_MP3, AUDIO_MP2, AUDIO_AC3, AUDIO_AAC])

        elif value == PRESET_FILE_IPOD:
            self.__combo_set_value('combobox_file_format', [FILE_MP4])
            self.__combo_set_value('combobox_video_format_file', [VIDEO_H264, VIDEO_MPEG4])
            self.__combo_set_value('combobox_video_resolution_file', ['320x240'])
            self.__combo_set_value('combobox_video_aspect_file', ['original'])
            self.__combo_set_value('combobox_video_encoding_file', ['1 pass'])
            self.__combo_set_value('combobox_audio_format_file', [AUDIO_AAC, AUDIO_MP3])

        elif value == PRESET_FILE_MOBILE:
            self.__combo_set_value('combobox_file_format', [FILE_MOBILE]) # 3gp
            self.__combo_set_value('combobox_video_format_file', [VIDEO_MOBILE]) # mpeg-4 actually
            self.__combo_set_value('combobox_video_resolution_file', ['176x144'])
            self.__combo_set_value('combobox_video_aspect_file', ['original'])
            self.__combo_set_value('combobox_video_encoding_file', ['1 pass'])
            self.__combo_set_value('combobox_audio_format_file', [AUDIO_MOBILE]) # pcm actually

        elif value == PRESET_FILE_NONE:
            pass

        else:
            debug('SelectSettings.on_file_preset_changed()', 'unknow preset: %d' % value, error)
            return False

        self.__ignore_update = False
        return True

    def on_output_folder_clicked(self, button, event = None):
        entry_output_folder = self.__xml.get_widget('entry_output_folder')
        dir_chooser = gtk.FileChooserDialog(
            _('Select default output folder'), self.__dialog,
            gtk.FILE_CHOOSER_ACTION_CREATE_FOLDER,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
             gtk.STOCK_OK, gtk.RESPONSE_OK))
        dir_chooser.set_default_response(gtk.RESPONSE_OK)
        dir_chooser.set_filename(
            os.path.abspath(os.path.expanduser(entry_output_folder.get_text())))
        response = dir_chooser.run()
        if response == gtk.RESPONSE_OK:
            entry_output_folder.set_text(dir_chooser.get_filename())
        dir_chooser.destroy()

    # -- public interface --

    def show(self):
        if self.__restore_position_func:
            self.__restore_position_func(self.__dialog, 'settings')

        self.__dialog.show()
        return

    def hide(self):
        if self.__store_position_func:
            self.__store_position_func(self.__dialog, 'settings')

        self.__dialog.hide()
        return

    def set_store_position(self, func):
        self.__store_position_func = func
        return

    def set_restore_position(self, func):
        self.__restore_position_func = func
        return

    def set_callback(self, func):
        self.__callback = func
        return

    # -- helpers --

    def __init_controls(self):
        if not self.__config:
            return

        self.__init_dvd_tab()
        self.__init_file_tab()
        self.__init_programs_tab()
        self.__init_misc()
        return

    def __init_dvd_tab(self):
        self.__ignore_update = True

        combo_video_format_dvd = self.__xml.get_widget('combobox_video_format_dvd')
        video_format_dvd = self.__config.get_video_format_dvd()
        # todo: add mpeg1
        video_formats_dvd = self.__create_values([('MPEG-2', VIDEO_MPEG2, 'lavc')])
        textcombo_setup(combo_video_format_dvd, video_formats_dvd, video_format_dvd)

        combo_video_resolution_dvd = self.__xml.get_widget('combobox_video_resolution_dvd')
        video_resolutions_dvd = [(_('Original'), 'original'),
                                 ('720x576', '720x576'),
                                 ('704x576', '704x576'),
                                 ('352x576', '352x576'),
                                 ('352x288', '352x288')]
        video_resolution_dvd = self.__config.get_video_resolution_dvd()
        if video_format_dvd == VIDEO_MPEG1:
            video_resolutions_dvd = [('352x288', '352x288')]
            video_resolution_dvd  = '352x288'
        textcombo_setup(combo_video_resolution_dvd, video_resolutions_dvd, video_resolution_dvd)

        combo_video_shrink = self.__xml.get_widget('combobox_video_encoding_dvd')
        video_shrink = self.__config.get_video_shrink_method_dvd()
        desired = [(_('Requantize (fastest encoding)'), 'requantize', 'tcrequant'),
                   (_('1 pass reencode'), '1 pass', 'mencoder'),
                   (_('2 pass reencode (best quality)'), '2 pass', 'mencoder')]
        shrink_methods = self.__create_values(desired)
        textcombo_setup(combo_video_shrink, shrink_methods, video_shrink)

        combo_audio_format_dvd = self.__xml.get_widget('combobox_audio_format_dvd')
        audio_format_dvd = self.__config.get_audio_format_dvd()
        desired = [('MP2', AUDIO_MP2, 'mp2enc'),
                   ('AC3', AUDIO_AC3, 'ffmpeg')]
                   #('PCM', AUDIO_PCM, None)]
        audio_formats_dvd = self.__create_values(desired)
        textcombo_setup(combo_audio_format_dvd, audio_formats_dvd, audio_format_dvd)

        combo_dvd_presets = self.__xml.get_widget('combobox_dvd_presets')
        preset_dvd = self.__config.get_dvd_preset()
        dvd_preset_values = [(_('None'), PRESET_DVD_NONE),
                             (_('Best quality'), PRESET_DVD_QUALITY),
                             (_('Fastest encoding'), PRESET_DVD_SPEED)]
        bold_font = True
        textcombo_setup(combo_dvd_presets , dvd_preset_values, preset_dvd, bold_font)

        self.__ignore_update = False
        return

    def __init_file_tab(self):
        self.__ignore_update = True

        file_format = self.__config.get_file_encoding()
        combo_video_format_file = self.__xml.get_widget('combobox_video_format_file')
        video_format_file = self.__config.get_video_format_file()
        video_formats_file = self.__create_video_values(file_format)
        textcombo_setup(combo_video_format_file, video_formats_file, video_format_file)

        combo_video_resolution_file = self.__xml.get_widget('combobox_video_resolution_file')
        video_resolution_file = self.__config.get_video_resolution_file()
        video_resolutions_file = self.__create_video_resol_values(file_format)
        textcombo_setup(combo_video_resolution_file, video_resolutions_file, video_resolution_file)

        combo_video_aspect_file = self.__xml.get_widget('combobox_video_aspect_file')
        video_aspect_file = self.__config.get_video_aspect_file()
        video_aspects_file = self.__create_video_aspect_values(file_format)
        textcombo_setup(combo_video_aspect_file, video_aspects_file, video_aspect_file)

        combo_video_shrink = self.__xml.get_widget('combobox_video_encoding_file')
        video_shrink = self.__config.get_video_shrink_method_file()
        shrink_methods = self.__create_shrink_method_values(file_format)
        textcombo_setup(combo_video_shrink, shrink_methods, video_shrink)

        combo_file_format = self.__xml.get_widget('combobox_file_format')
        preset_file = self.__config.get_file_preset()
        file_formats = self.__create_file_format_values(preset_file)
        textcombo_setup(combo_file_format, file_formats, file_format)

        combo_audio_format_file = self.__xml.get_widget('combobox_audio_format_file')
        audio_format_file = self.__config.get_audio_format_file()
        audio_formats_file = self.__create_audio_values(file_format)
        textcombo_setup(combo_audio_format_file, audio_formats_file, audio_format_file)

        combo_audio_quality_file = self.__xml.get_widget('combobox_audio_quality_file')
        combo_audio_format = self.__xml.get_widget('combobox_audio_format_file')
        value = textcombo_get_value(combo_audio_format)
        self.__setup_file_quality_combo(combo_audio_quality_file, value)

        combo_file_presets = self.__xml.get_widget('combobox_file_presets')
        file_preset_values = [(_('None'), PRESET_FILE_NONE),
                              (_('Best quality'), PRESET_FILE_QUALITY),
                              (_('Fastest encoding'), PRESET_FILE_SPEED),
                              (_('Maximum compression'), PRESET_FILE_COMPRESSION)]
        if self.__tool_checker.has_tool('MP4Box') and self.__tool_checker.has_tool('faac'):
            file_preset_values.append((_('iPod'), PRESET_FILE_IPOD))
        if self.__tool_checker.has_tool('mencoder') and self.__tool_checker.has_tool('ffmpeg'):
            file_preset_values.append((_('Cell phone'), PRESET_FILE_MOBILE))
        bold_font = True
        file_preset = self.__config.get_file_preset()
        textcombo_setup(combo_file_presets , file_preset_values, file_preset, bold_font)

        self.__ignore_update = False
        return

    def __init_programs_tab(self):
        button = self.__xml.get_widget('button_ok')
        yes_pixbuf = button.render_icon(STOCK_YES, ICON_SIZE_BUTTON)
        no_pixbuf = button.render_icon(STOCK_NO, ICON_SIZE_BUTTON)

        self.__init_program_entry('mplayer', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mencoder', yes_pixbuf, no_pixbuf)
        self.__init_program('divx', 'lavc', yes_pixbuf, no_pixbuf)
        self.__init_program('xvid', 'xvid', yes_pixbuf, no_pixbuf)
        self.__init_program('x264', 'x264', yes_pixbuf, no_pixbuf)
        self.__init_program('mpeg2', 'lavc', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('lsdvd', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('oggenc', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('ffmpeg', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mkvmerge', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('avimerge', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mplex', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('dvdauthor', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('tcrequant', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('tcextract', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('title', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('lindex', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('paldump', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('lame', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mp2enc', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('spuunmux', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('spumux', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mkisofs', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('cdrecord', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('hal-device', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('mp4box', yes_pixbuf, no_pixbuf)
        self.__init_program_entry('faac', yes_pixbuf, no_pixbuf)
        return

    def __init_misc(self):
        entry_output_folder = self.__xml.get_widget('entry_output_folder')
        entry_output_folder.set_text(self.__config.get_output_dir())
        
        checkbutton_ch_rip = self.__xml.get_widget('checkbutton_chapter_rip')
        enabled = self.__config.get_chapter_rip()
        checkbutton_ch_rip.set_active(enabled)

        checkbutton_exit = self.__xml.get_widget('checkbutton_exit')
        enabled = self.__config.get_exit_winki()
        checkbutton_exit.set_active(enabled)

        checkbutton_popup = self.__xml.get_widget('checkbutton_popup')
        enabled = self.__config.get_show_popups()
        checkbutton_popup.set_active(enabled)
        return

    def __init_program_entry(self, tool_name, yes_pixbuf, no_pixbuf):
        entry = self.__xml.get_widget('entry_%s' % tool_name)
        image = self.__xml.get_widget('image_%s' % tool_name)

        lc_all_prefix = False
        location = ''
        pixbuf = no_pixbuf
        if self.__tool_checker.has_tool(tool_name):
            location = self.__tool_checker.get_tool_location(tool_name, lc_all_prefix)
            pixbuf = yes_pixbuf

        entry.set_text(location)
        image.set_from_pixbuf(pixbuf)
        return

    def __init_program(self, ui_name, tool_name, yes_pixbuf, no_pixbuf):
        image = self.__xml.get_widget('image_%s' % ui_name)

        #lc_all_prefix = False
        pixbuf = no_pixbuf
        if self.__tool_checker.has_tool(tool_name, TOOL_MENCODER):
            #location = self.__tool_checker.get_tool_location(tool_name, lc_all_prefix)
            pixbuf = yes_pixbuf

        image.set_from_pixbuf(pixbuf)
        return

    def __update_config(self):
        if not self.__config:
            assert(False)
            return

        self.__update_dvd_tab()
        self.__update_file_tab()
        self.__update_programs_tab()
        self.__update_misc()

        if not self.__config.write():
            assert(False)
        return

    def __update_dvd_tab(self):
        combo = self.__xml.get_widget('combobox_dvd_presets')
        value = textcombo_get_value(combo)
        self.__config.set_dvd_preset(value)

        combo = self.__xml.get_widget('combobox_video_format_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_video_format_dvd(value)

        combo = self.__xml.get_widget('combobox_video_resolution_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_video_resolution_dvd(value)

        combo = self.__xml.get_widget('combobox_video_aspect_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_video_aspect_dvd(value)

        combo = self.__xml.get_widget('combobox_video_encoding_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_video_shrink_method_dvd(value)

        combo = self.__xml.get_widget('combobox_audio_format_dvd')
        value = textcombo_get_value(combo)
        self.__config.set_audio_format_dvd(value)
        return

    def __update_file_tab(self):
        combo = self.__xml.get_widget('combobox_file_presets')
        value = textcombo_get_value(combo)
        self.__config.set_file_preset(value)

        combo = self.__xml.get_widget('combobox_file_format')
        value = textcombo_get_value(combo)
        self.__config.set_file_encoding(value)

        combo = self.__xml.get_widget('combobox_video_format_file')
        value = textcombo_get_value(combo)
        self.__config.set_video_format_file(value)

        combo = self.__xml.get_widget('combobox_video_resolution_file')
        value = textcombo_get_value(combo)
        self.__config.set_video_resolution_file(value)

        combo = self.__xml.get_widget('combobox_video_aspect_file')
        value = textcombo_get_value(combo)
        self.__config.set_video_aspect_file(value)

        combo = self.__xml.get_widget('combobox_video_encoding_file')
        value = textcombo_get_value(combo)
        self.__config.set_video_shrink_method_file(value)

        combo = self.__xml.get_widget('combobox_audio_quality_file')
        quality = textcombo_get_value(combo)

        combo = self.__xml.get_widget('combobox_audio_format_file')
        value = textcombo_get_value(combo)
        self.__config.set_audio_format_file(value)

        if value == AUDIO_OGG:
            self.__config.set_ogg_quality(quality)
        elif value == AUDIO_MP3:
            self.__config.set_mp3_quality(quality)
        elif value == AUDIO_MP2:
            self.__config.set_mp2_quality(quality)
        elif value == AUDIO_AC3:
            self.__config.set_ac3_quality(quality)
        elif value == AUDIO_AAC:
            self.__config.set_aac_quality(quality)
        elif value == AUDIO_MOBILE:
            pass
        else:
            debug('SelectSettings.__update_file_tab()', 'unexpected file format %s' % value, error)
        return

    def __update_programs_tab(self):
        entry = self.__xml.get_widget('entry_mplayer')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mplayer', location)

        entry = self.__xml.get_widget('entry_mencoder')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mencoder', location)

        entry = self.__xml.get_widget('entry_lsdvd')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('lsdvd', location)

        entry = self.__xml.get_widget('entry_oggenc')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('oggenc', location)

        entry = self.__xml.get_widget('entry_ffmpeg')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('ffmpeg', location)

        entry = self.__xml.get_widget('entry_mkvmerge')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mkvmerge', location)

        entry = self.__xml.get_widget('entry_avimerge')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('avimerge', location)

        entry = self.__xml.get_widget('entry_mplex')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mplex', location)

        entry = self.__xml.get_widget('entry_dvdauthor')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('dvdauthor', location)

        entry = self.__xml.get_widget('entry_tcrequant')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('tcrequant', location)

        entry = self.__xml.get_widget('entry_tcextract')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('tcextract', location)

        entry = self.__xml.get_widget('entry_title')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('title', location)

        entry = self.__xml.get_widget('entry_lindex')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('lindex', location)

        entry = self.__xml.get_widget('entry_paldump')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('paldump', location)

        entry = self.__xml.get_widget('entry_lame')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('lame', location)

        entry = self.__xml.get_widget('entry_mp2enc')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mp2enc', location)

        entry = self.__xml.get_widget('entry_spuunmux')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('spuunmux', location)

        entry = self.__xml.get_widget('entry_spumux')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('spumux', location)

        entry = self.__xml.get_widget('entry_mkisofs')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('mkisofs', location)

        entry = self.__xml.get_widget('entry_cdrecord')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('cdrecord', location)

        entry = self.__xml.get_widget('entry_hal-device')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('hal-device', location)

        entry = self.__xml.get_widget('entry_faac')
        location = entry.get_text()
        self.__tool_checker.set_tool_location('faac', location)
        return

    def __update_misc(self):
        entry_output_folder = self.__xml.get_widget('entry_output_folder')
        self.__config.set_output_dir(entry_output_folder.get_text())

        checkbutton_ch_rip = self.__xml.get_widget('checkbutton_chapter_rip')
        enabled = checkbutton_ch_rip.get_active()
        self.__config.set_chapter_rip(enabled)

        checkbutton_exit = self.__xml.get_widget('checkbutton_exit')
        enabled = checkbutton_exit.get_active()
        self.__config.set_exit_winki(enabled)

        checkbutton_popup = self.__xml.get_widget('checkbutton_popup')
        enabled = checkbutton_popup.get_active()
        self.__config.set_show_popups(enabled)
        return


    def __combo_set_value(self, combo_name, keys):
        combo = self.__xml.get_widget(combo_name)
        if not textcombo_set_value(combo, keys):
            return False

        return True

    def __create_video_values(self, file_format):
        desired = [(_('DivX'), VIDEO_MPEG4, 'lavc'),
                   (_('XviD'), VIDEO_XVID, 'xvid')]

        if file_format == FILE_MATROSKA:
            desired.append(tuple((_('MPEG-2'), VIDEO_MPEG2, 'lavc')))

        elif file_format == FILE_MOBILE:
            desired = [(_('MPEG4 for cell phone'), VIDEO_MOBILE, 'lavc')]

        else:
            # it was possible but now mkvmerge segfaults like hell with such streams
            # todo: further investigation needed
            desired.append(tuple((_('x264'), VIDEO_H264, 'x264')))

        return self.__create_values(desired)

    def __create_video_aspect_values(self, file_format):
        if file_format == FILE_MOBILE:
            desired = [(_('Original'), 'original')]

        else:
            desired = [(_('Original'), 'original'),
                       (_('16/9 (widescreen)'), '16/9'),
                       (_('4/3 (TV)'), '4/3'),
                       (_('2.11/1 (DVB)'), '2.11')]
        return desired

    def __create_shrink_method_values(self, file_format):
        if file_format == FILE_MOBILE:
            desired = [(_('1 pass reencode (faster)'), '1 pass', 'mencoder')]

        else:
            desired = [(_('1 pass reencode (faster)'), '1 pass', 'mencoder'),
                       (_('2 pass reencode (better quality)'), '2 pass', 'mencoder')]

        return self.__create_values(desired)

    def __create_video_resol_values(self, file_format):
        if file_format == FILE_MOBILE:
            desired = [(_('176x144 (cell phone)'), '176x144')]

        else:
            desired = [(_('Original'), 'original'),
                       ('720x576', '720x576'),
                       ('704x576', '704x576'),
                       ('352x576', '352x576'),
                       ('352x288', '352x288'),
                       (_('384x288 (DivX 4:3)'), '384x288'),
                       (_('512x288 (DivX 16:9)'), '512x288'),
                       (_('320x240 (iPod)'), '320x240')]

        return desired

    def __create_file_format_values(self, preset):
        if preset == PRESET_FILE_MOBILE:
            desired = [(_('3GP'), FILE_MOBILE, 'ffmpeg')]

        elif preset == PRESET_FILE_IPOD:
            desired = [(_('MP4'), FILE_MP4, 'mp4box')]

        else:
            desired = [(_('Matroska (superior)'), FILE_MATROSKA, 'mkvmerge'),
                       (_('MP4'), FILE_MP4, 'mp4box'),
                       (_('AVI (inferior)'), FILE_AVI, 'avimerge')]

        return self.__create_values(desired)

    def __create_audio_values(self, file_format):
        desired = [('MP3', AUDIO_MP3, 'lame')]
                   #('PCM', AUDIO_PCM, None)]

        if file_format == FILE_AVI:
            desired.append(tuple(('AC3', AUDIO_AC3, 'ffmpeg')))

        elif file_format == FILE_MATROSKA:
            desired.append(tuple(('AC3', AUDIO_AC3, 'ffmpeg')))
            desired.append(tuple(('OGG', AUDIO_OGG, 'oggenc')))
            desired.append(tuple(('MP2', AUDIO_MP2, 'mp2enc')))
            desired.append(tuple(('AAC', AUDIO_AAC, 'faac')))

        elif file_format == FILE_MP4:
            desired.append(tuple(('OGG', AUDIO_OGG, 'oggenc')))
            desired.append(tuple(('AAC', AUDIO_AAC, 'faac')))

        elif file_format == FILE_MOBILE:
            desired = [(_('PCM for cell phone'), AUDIO_MOBILE, 'mencoder')]

        return self.__create_values(desired)

    def __create_values(self, tool_value_list):
        result = []
        for item in tool_value_list:
            value, key, tool = item

            ttype = TOOL_EXTERNAL
            if tool in ['xvid', 'lavc', 'x264', 'toolame', 'mp3lame', 'twolame']:
                ttype = TOOL_MENCODER

            if tool:
                if self.__tool_checker.has_tool(tool, ttype):
                    result.append(tuple((value, key)))
            else:
                result.append(tuple((value, key)))

        return result

    def __setup_file_quality_combo(self, combo, value):
        combo.set_sensitive(True)

        if value == AUDIO_OGG:
            ogg_quality = self.__config.get_ogg_quality()
            audio_qualities_file = [(_('64 kb/s'), 64),
                                    (_('96 kb/s'), 96),
                                    (_('128 kb/s default'), 128),
                                    (_('160 kb/s'), 160),
                                    (_('192 kb/s'), 192),
                                    (_('224 kb/s'), 224),
                                    (_('256 kb/s'), 256),
                                    (_('320 kb/s'), 320)]
            textcombo_setup(combo, audio_qualities_file, ogg_quality)
        elif value == AUDIO_MP3:
            mp3_quality = self.__config.get_mp3_quality()
            audio_qualities_file = [(_('32 kb/s'), 32),
                                    (_('40 kb/s'), 40),
                                    (_('48 kb/s'), 48),
                                    (_('56 kb/s'), 56),
                                    (_('64 kb/s'), 64),
                                    (_('96 kb/s'), 96),
                                    (_('128 kb/s default'), 128),
                                    (_('160 kb/s'), 160),
                                    (_('192 kb/s'), 192),
                                    (_('224 kb/s'), 224),
                                    (_('256 kb/s'), 256),
                                    (_('320 kb/s'), 320)]
            textcombo_setup(combo, audio_qualities_file, mp3_quality)
        elif value == AUDIO_MP2:
            mp2_quality = self.__config.get_mp2_quality()
            audio_qualities_file = [(_('64 kb/s'), 64),
                                    (_('96 kb/s'), 96),
                                    (_('128 kb/s'), 128),
                                    (_('160 kb/s'), 160),
                                    (_('192 kb/s'), 192),
                                    (_('224 kb/s default'), 224),
                                    (_('256 kb/s'), 256),
                                    (_('320 kb/s'), 320),
                                    (_('384 kb/s'), 384),
                                    (_('512 kb/s'), 512)]
            textcombo_setup(combo, audio_qualities_file, mp2_quality)
        elif value == AUDIO_AC3:
            ac3_quality = self.__config.get_ac3_quality()
            audio_qualities_file = [(_('64 kb/s'), 64),
                                    (_('96 kb/s'), 96),
                                    (_('112 kb/s'), 112),
                                    (_('128 kb/s'), 128),
                                    (_('224 kb/s default'), 224),
                                    (_('384 kb/s'), 384),
                                    (_('512 kb/s'), 512)]
            textcombo_setup(combo, audio_qualities_file, ac3_quality)
        elif value == AUDIO_AAC:
            aac_quality = self.__config.get_aac_quality()
            audio_qualities_file = [(_('40 kb/s'), 40),
                                    (_('48 kb/s'), 48),
                                    (_('56 kb/s'), 56),
                                    (_('64 kb/s'), 64),
                                    (_('80 kb/s'), 80),
                                    (_('96 kb/s'), 96),
                                    (_('112 kb/s'), 112),
                                    (_('128 kb/s default'), 128),
                                    (_('160 kb/s'), 160),
                                    (_('192 kb/s'), 192)]
            textcombo_setup(combo, audio_qualities_file, aac_quality)
        else:
            combo.set_sensitive(False)
        return

    def __adapt_video_aspect_dvd(self, text):
        combo_video_aspect_dvd = self.__xml.get_widget('combobox_video_aspect_dvd')
        video_aspects_dvd = [('4/3', '4/3')]
        video_aspect_dvd = '4/3'
        if text == 'original' or text == '720x576':
            video_aspects_dvd = [(_('Original'), 'original'),
                                 ('16/9 (widescreen)', '16/9'),
                                 ('4/3 (TV)', '4/3')]
            video_aspect_dvd = textcombo_get_value(combo_video_aspect_dvd)
        
        textcombo_setup(combo_video_aspect_dvd, video_aspects_dvd, video_aspect_dvd)
        return

    def __adapt_video_shrink_dvd(self, text):
        combo_video_shrink = self.__xml.get_widget('combobox_video_encoding_dvd')
        video_shrink = textcombo_get_value(combo_video_shrink)
        if text == 'original':
            desired = [(_('Requantize (fastest encoding)'), 'requantize', 'tcrequant'),
                       (_('1 pass reencode'), '1 pass', 'mencoder'),
                       (_('2 pass reencode (best quality)'), '2 pass', 'mencoder')]
        else:
            desired = [(_('1 pass reencode'), '1 pass', 'mencoder'),
                       (_('2 pass reencode (best quality)'), '2 pass', 'mencoder')]
        shrink_methods = self.__create_values(desired)
        textcombo_setup(combo_video_shrink, shrink_methods, video_shrink)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
