# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_video import VIDEO_MPEG2, VIDEO_MPEG4
from winkirip.container_audio import AUDIO_AC3, AUDIO_MP2, AUDIO_PCM
from winkirip.constructor_mpeg2 import ConstructorMpeg2DVD
from winkirip.lib import debug, error

class WorkerAudioVideoDVD(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Extract audio and video'))
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create_old(self, command_list):
        shrink_method = ''
        cfg = self.get_context().get_project().get_config()

        containers = self.get_context().get_project().get_dest()
        if containers[0].get_type() == CONTAINER_DVD:
            exec_func = self.__exec_dvd
            shrink_method = cfg.get_video_shrink_method_dvd()
        elif containers[0].get_type() == CONTAINER_FILE:
            exec_func = self.__exec_file
            shrink_method = cfg.get_video_shrink_method_file()

        disc_nr = 0
        for container in containers:
            title_nr = 0
            for title in container:
                ch_start = title.get_first_chapter_id()
                ch_end = title.get_last_chapter_id()

                ui_text = '' # (_('Extracting audio and video (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
                #if shrink_method == '2 pass':
                #    ui_text = (_('Creating video index (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))

                # first pass
                ui_command = UICommand(exec_func, ui_text, True)
                ui_command.set_exec_log_callback(self.__exec_log)
                if shrink_method == '2 pass':
                    ui_command.set_data([disc_nr, title_nr, ch_start, ch_end, 1, 2])
                else:
                    ui_command.set_data([disc_nr, title_nr, ch_start, ch_end, 1, 1])
                command_list.append(ui_command)

                # second pass
                if shrink_method == '2 pass':
                    ui_text = '' # (_('Extracting audio and video (disk %d :: title %d)') % (disc_nr + 1, title_nr + 1))
                    ui_command = UICommand(exec_func, ui_text, True)
                    ui_command.set_exec_log_callback(self.__exec_log)
                    ui_command.set_data([disc_nr, title_nr, ch_start, ch_end, 2, 2])
                    command_list.append(ui_command)

                title_nr += 1
            disc_nr += 1
        return True

    # todo: implement pcm audio
    def __exec_dvd_old(self, command_data, command):
        assert(len(command_data) == 6)
        disc_nr, title_nr, ch_start, ch_end, pass_nr, pass_count = command_data

        cont = self.get_context().get_project().get_dest()[disc_nr]
        title = cont[title_nr]
        device = cont.get_device()
        video = title.get_video()[0]
        #fname = 'video/disc%d_title%d_video.mpeg' % (disc_nr + 1, title_nr + 1)
        fname = self.create_stream_name(disc_nr + 1, title_nr + 1, 'video', 'mpeg')

        seconds = 0
        for title in cont:
            seconds += title.get_length()

        disc_size = cont.get_size() * 1024 * 1024
        admin_size = disc_size * 0.05
        cfg = self.get_context().get_project().get_config()
        asize =  seconds * cfg.get_audio_bytes_per_second(AUDIO_AC3)
        vsize = -1
        vbitrate, shrink_needed = self.calc_vbitrate(disc_size, admin_size, asize, seconds, vsize)
        acodec = cfg.get_audio_format_dvd().lower()
        if acodec == 'pcm':
            debug('WorkerAudioVideoDVD.__exec_dvd', 'pcm audio not implemented yet', error)
            command.set_error(_('PCM audio not implemented yet'))
            return False

        media_title = self.get_context().get_media(disc_nr)[title_nr]
        crop = media_title.get_crop()
        resol = cfg.get_video_resolution_dvd()
        aspect = cfg.get_video_aspect_dvd()
        if resol != '720x576':
            aspect = '4/3'
        aspect, scale_expand = self.compute_scale_expand(video, crop, aspect, resol)

        tool_checker = self.get_context().get_project().get_tool_checker()

        constr = ConstructorMpeg2DVD(tool_checker)
        constr.set_source(device, title.get_id())
        constr.set_source_type(CONTAINER_DVD)
        constr.set_outname(fname)
        constr.set_aspect(aspect)
        constr.set_vbitrate(vbitrate)
        constr.set_scale_expand(scale_expand)
        constr.set_pass_count(pass_count)
        constr.set_pass_nr(pass_nr)

        success, shell_cmd = constr.construct()
        if not success:
            command.set_error(_('You can get help in the winki forum located at http://www.winki-the-ripper.de'))
            return False

        self.__exec_log_text = ('disc: %d - admin: %d - asize: %d - seconds: %d - vbitrate: %d\n' % (disc_size, admin_size, asize, seconds, vbitrate))
        self.__exec_log_text += ('%s\n' % shell_cmd)

        result = command.run_command(shell_cmd)
        if result:
            self.__update_video(disc_nr, title_nr, VIDEO_MPEG2, fname, aspect)
            self.__update_audio(disc_nr, title_nr, acodec)

        return result

    # todo: implement mpeg4 video and all supported audio types
    def __exec_file_old(self, command_data, command):
        assert(len(command_data) == 6)
        disc_nr, title_nr, ch_start, ch_end, pass_nr, pass_count = command_data

        cont = self.get_context().get_project().get_dest()[disc_nr]
        device = cont.get_device()
        title = cont[title_nr]
        video = title.get_video()[0]
        #fname = 'video/disc%d_title%d_video.mpeg' % (disc_nr + 1, title_nr + 1)
        fname = self.create_stream_name(disc_nr + 1, title_nr + 1, 'video', 'mpeg')

        seconds = 0
        for title in cont:
            seconds += title.get_length()

        disc_size = cont.get_size() * 1024 * 1024
        admin_size = disc_size * 0.05
        cfg = self.get_context().get_project().get_config()
        asize =  seconds * cfg.get_audio_bytes_per_second(AUDIO_AC3)
        vsize = -1
        vbitrate, shrink_needed = self.calc_vbitrate(disc_size, admin_size, asize, seconds, vsize)
        acodec = cfg.get_audio_format_dvd().lower()
        if acodec != 'ac3' and acodec != 'mp2':
            debug('WorkerAudioVideoDVD.__exec_file', 'pcm audio not implemented yet', error)
            command.set_error(_('PCM audio not implemented yet'))
            return False

        tool_checker = self.get_context().get_project().get_tool_checker()

        media_title = self.get_context().get_media(disc_nr)[title_nr]
        crop = media_title.get_crop()
        aspect = cfg.get_video_aspect_file()
        resol = cfg.get_video_resolution_file()
        aspect, scale_expand = self.compute_scale_expand(video, crop, aspect, resol)

        constr = ConstructorMpeg2DVD(tool_checker)
        constr.set_source(device, title.get_id())
        constr.set_source_type(CONTAINER_DVD)
        constr.set_outname(fname)
        constr.set_aspect(aspect)
        constr.set_vbitrate(vbitrate)
        constr.set_scale_expand(scale_expand)
        constr.set_pass_count(pass_count)
        constr.set_pass_nr(pass_nr)

        success, shell_cmd = constr.construct()
        if not success:
            command.set_error(_('You can get help in the winki forum located at http://www.winki-the-ripper.de'))
            return False

        self.__exec_log_text = ('disc: %d - admin: %d - asize: %d - seconds: %d - vbitrate: %d\n' % (disc_size, admin_size, asize, seconds, vbitrate))
        self.__exec_log_text += ('%s\n' % shell_cmd)

        result = command.run_command(shell_cmd)
        if result:
            self.__update_video(disc_nr, title_nr, VIDEO_MPEG2, fname, aspect)
            self.__update_audio(disc_nr, title_nr, acodec)

        return result

    def __update_video_old(self, disc_nr, title_nr, vtype, fname, aspect):
        title = self.get_context().get_media(disc_nr)[title_nr]
        video_list = title.get_video()
        assert(len(video_list) > 0)

        vobj = video_list[0]
        vobj.set_filename(fname)
        vobj.set_type(vtype)
        vobj.set_id(1)
        vobj.set_aspect(aspect.replace(':', '/'))
        return

    def __update_audio_old(self, disc_nr, title_nr, atype):
        title = self.get_context().get_media(disc_nr)[title_nr]
        audio_list = title.get_audio()

        aformat = AUDIO_PCM
        if atype == 'ac3':
            aformat = AUDIO_AC3
        elif atype == 'mp2':
            aformat = AUDIO_MP2

        for audio in audio_list:
            audio.set_format(aformat)
        return

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no audio/video information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
