# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.writer_xml import WriterXML
from winkirip.container import FILE_MATROSKA, FILE_MP4
from winkirip.container_chapter import Chapter
from winkirip.lib import sec2str

class WorkerChaptersTags(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Create chapters and tags'))
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        dest = self.get_context().get_dest()
        for disc_nr in range(len(dest)):
            ui_command = UICommand()
            ui_command.set_command(self.__execute)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_data([disc_nr])
            command_list.append(ui_command)

        return True

    def __execute(self, command_data, command):
        assert(len(command_data) == 1)
        disc_nr, = command_data

        self.__exec_log_text = ''

        cont = self.get_context().get_dest()[disc_nr]
        if cont.get_file_type() == FILE_MATROSKA:
            if not self.__create_chapters_mkv(disc_nr):
                self.__exec_log_text += 'failed to create chapters\n'
                command.set_error(_('Failed to create chapter description'))
                return False

            if not self.__create_tags_mkv(disc_nr):
                self.__exec_log_text += 'failed to create tags\n'
                command.set_error(_('Failed to create tags description'))
                return False

        elif cont.get_file_type() == FILE_MP4:
            if not self.__create_chapters_mp4(disc_nr):
                self.__exec_log_text += 'failed to create chapters\n'
                command.set_error(_('Failed to create chapter description'))
                return False

        return True

    def __create_chapters_mp4(self, disc_nr):
        cont = self.get_context().get_dest()[disc_nr]

        title_nr = 0
        for title in cont:
            fname = 'disc%d_title%d.time_codes' % (disc_nr + 1, title_nr + 1)
            try:
                f = open(fname, 'w')
            except:
                return False

            chapter_start_time = 0
            f.write('%s Title: %s\n' % (sec2str(chapter_start_time), title.get_name()))

            for chapter in title.get_chapters():
                f.write('%s %s\n' % (sec2str(chapter_start_time), chapter.get_name()))
                chapter_start_time += chapter.get_length()

            f.close()

            title.set_chapter_file(fname)
            title_nr += 1

        return True

    def __create_chapters_mkv(self, disc_nr):
        cont = self.get_context().get_dest()[disc_nr]
        title_nr = 0

        fname = 'disc%d_chapters.xml' % (disc_nr + 1)
        self.__exec_log_text += 'creating %s\n' % fname

        writer = WriterXML(fname)
        writer.xml_version({'version' : '1.0', 'encoding' : 'UTF-8'})
        writer.doctype('Chapters SYSTEM "matroskachapters.dtd"')
        writer.stag('Chapters', {})
        writer.increment()

        writer.stag('EditionEntry', {})
        writer.increment()

        t_time = 0
        for title in cont:
            writer.stag('ChapterAtom', {})
            writer.increment()

            t_name = title.get_name()
            t_start = sec2str(t_time)
            writer.ltag('ChapterTimeStart', {}, t_start)

            ch_time = t_time
            t_time += title.get_length()
            t_end = sec2str(t_time)
            writer.ltag('ChapterTimeEnd', {}, t_end)

            writer.stag('ChapterDisplay', {})
            writer.increment()

            writer.ltag('ChapterString', {}, t_name)
            # todo: this is the language for ChapterString
            writer.ltag('ChapterLanguage', {}, 'eng')

            writer.decrement()
            writer.etag('ChapterDisplay')

            # indent chapters
            writer.increment()

            chapter_list = title.get_chapters()
            for chapter in chapter_list:
                ch_name = chapter.get_name()
                ch_start = sec2str(ch_time)

                writer.stag('ChapterAtom', {})
                writer.increment()

                writer.ltag('ChapterTimeStart', {}, ch_start)

                ch_time += chapter.get_length()
                ch_end = sec2str(ch_time)
                writer.ltag('ChapterTimeEnd', {}, ch_end)

                writer.stag('ChapterDisplay', {})
                writer.increment()

                writer.ltag('ChapterString', {}, ch_name)
                # todo: this is the language for ChapterString, not the audio
                # language
                writer.ltag('ChapterLanguage', {}, 'eng')

                writer.decrement()
                writer.etag('ChapterDisplay')

                writer.decrement()
                writer.etag('ChapterAtom')

            # end indent chapters
            writer.decrement()

            # end title
            writer.decrement()
            writer.etag('ChapterAtom')

            title_nr += 1

        writer.increment()
        writer.etag('EditionEntry')

        writer.decrement()
        writer.etag('Chapters')

        writer.write()

        cont.set_chapter_file(fname)

        return True

    def __create_tags_mkv(self, disc_nr):
        cont = self.get_context().get_dest()[disc_nr]

        tags = cont.get_tags()
        if len(tags) < 1:
            return True
        
        fname = 'disc%d_tags.xml' % (disc_nr + 1)
        self.__exec_log_text += 'creating %s\n' % fname
        writer = WriterXML(fname)

        writer.xml_version({'version' : '1.0', 'encoding' : 'UTF-8'})
        writer.doctype('Tags SYSTEM "matroskatags.dtd"')

        writer.stag('Tags', {})
        writer.increment()

        writer.stag('Tag', {})
        writer.increment()

        writer.stag('Targets', {})
        writer.increment()

        # todo: start loop
        writer.ltag('TrackUID', {}, '1')
        # todo: end loop

        # todo: is just the first title enough?
        title = cont[0]
        chapter_list = title.get_chapters()
        for i in range(len(chapter_list)):
            writer.ltag('ChapterUID', {}, '%d' % (i + 1))

        writer.decrement()
        writer.etag('Targets')

        for tag_key in tags:
            # KEYWORDS: komma separated list of keywords
            # see http://www.matroska.org/technical/specs/tagging/index.html
            if tag_key == 'KEYWORDS':
                values = ''
                for i in range(len(tags[tag_key])):
                    if i == (len(tags[tag_key]) - 1):
                        values += tags[tag_key][i]
                    else:
                        values += ('%s,' % tags[tag_key][i])

                writer.stag('Simple', {})
                writer.increment()

                writer.ltag('Name', {}, tag_key)
                writer.ltag('String', {}, values)

                writer.decrement()
                writer.etag('Simple')

            else:
                for value in tags[tag_key]:
                    writer.stag('Simple', {})
                    writer.increment()

                    writer.ltag('Name', {}, tag_key)
                    writer.ltag('String', {}, value)

                    writer.decrement()
                    writer.etag('Simple')

        writer.decrement()
        writer.etag('Tag')

        writer.decrement()
        writer.etag('Tags')

        writer.write()

        cont.set_tag_file(fname)

        return True

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('exec log: no chapter information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
