# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.lib import mkdir, normalize_path, does_path_exist, unlink_dir
from winkirip.command_ui import UICommand
from winkirip.alert import AlertWarning
from os import chdir

class WorkerChDir(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Change working folder'))
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        dest = self.get_context().get_project().get_dest_dir()
        if not dest:
            return False

        dest = self.get_context().get_project().get_dest_dir()

        ui_command = UICommand()
        ui_command.set_command(self.__execute)
        ui_command.set_exec_log_callback(self.__exec_log)
        command_list.append(ui_command)
        return True

    def __execute(self, command_data, command):
        chdir(normalize_path('~'))

        dest = self.get_context().get_project().get_dest_dir()

        if does_path_exist(dest):
            primary = _('Do you want to remove %s?') % dest
            secondary = _('The destination folder already exist. Removing this folder will erase all the data contained in it and its subfolders')

            dlg_info = AlertWarning()
            dlg_info.show(primary, secondary, _('Remove'))

            if dlg_info.state != 'ok':
                return False

            self.__exec_log_text = ('rm -r %s\n' % dest)
            unlink_dir(dest)

        assert(not does_path_exist(dest))

        if not mkdir(dest):
            self.__exec_log_text = ('failed to create %s\n' % dest)
            command.set_error(_('Unable to create folder %s') % dest)
            return False

        self.__exec_log_text = ('cd %s\n\n' % dest)
        chdir('%s' % dest)
        return True

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
