# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import debug, error, unlink_dir, fix_fname

class WorkerCleanup(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Cleanup temporary files and folders'))
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        ui_text = ''
        ui_command = UICommand()
        ui_command.set_command(self.__execute)
        ui_command.set_exec_log_callback(self.__exec_log)
        command_list.append(ui_command)
        return True

    def __execute(self, command_data, command):
        # remove temp directories
        self.__exec_log_text = 'rm -rf audio/ video/ subtitles/ DISC/\n'
        map(unlink_dir, ['audio', 'video', 'subtitles', 'result', 'DISC'])

        # remove divx2pass.log variants and /tmp/source_X
        temp_list = []
        cid = 0
        for cont in self.get_context().get_dest():
            self.__exec_log_text += ('rm /tmp/source_%d\n' % cid)
            temp_list.append('/tmp/source_%d' % cid)
            cid += 1

            if cont.get_name():
                self.__exec_log_text += ('rm -rf %s\n' % fix_fname(cont.get_name()).upper())
                unlink_dir(fix_fname(cont.get_name()).upper())

            for title in cont:
                ch_start = title.get_first_chapter_id()
                ch_end = title.get_last_chapter_id()
                chapt_info = self.expand_chapter_list(ch_start, ch_end, 'divx2pass.log')
                for s, e, tname in chapt_info:
                    self.__exec_log_text += ('rm %s\n' % tname)
                    temp_list.append(tname)

        self.__exec_log_text += 'rm disc.xml\n'
        temp_list.append('disc.xml')

        self.unlink_temp(temp_list)
        return True


    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no cleanup information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
