# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import mkdir, does_path_exist, debug, error, build_path

class WorkerDirStruct(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Create working folders structure'))
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        ui_command = UICommand()
        ui_command.set_command(self.__execute)
        ui_command.set_exec_log_callback(self.__exec_log)
        command_list.append(ui_command)
        return True

    def __execute(self, command_data, command):
        self.__exec_log_text = ''
        dirs_to_create = ['audio', 'video', 'subtitles', 'result']

        self.__exec_log_text += ('mkdir audio video subtitles result\n\n')

        for mydir in dirs_to_create:
            if not self.__create_dir(mydir, command):
                return False

        return True

    def __create_dir(self, dirname, command):
        if does_path_exist(dirname):
            dest = self.get_context().get_project().get_dest_dir()
            full_name = build_path([dest, dirname])
            self.__exec_log_text += ('directory %s already exist\n' % full_name)
            debug('WorkerDirStruct.__create_dir()', '%s already exist' % full_name, error)
            command.set_error(_('Output folder %s already exist. Either delete it or specify another output folder.') % full_name)
            return False

        if not mkdir(dirname):
            self.__exec_log_text += ('failed to create directory %s\n' % dirname)
            command.set_error(_('Cannot create output folder %s') % dirname)
            return False
        return True

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
