# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import does_path_exist, debug
from os import symlink, unlink

from winkirip import lib

class WorkerLinkSource(Worker):
    def __init__(self, context, source_id):
        Worker.__init__(self, context, self.__create, _('Link source'))
        self.__dev_link = ('/tmp/source_%d' % source_id)
        self.__source_id = source_id
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        device = None
        for source_container in self.get_context().get_project().get_source():
            if source_container.get_id() == self.__source_id:
                preferre_link = False
                device = source_container.get_device(preferre_link)
                source_container.set_device_link(self.__dev_link)
                break

        if not device:
            return False

        ui_command = UICommand()
        ui_command.set_command(self.__execute)
        ui_command.set_exec_log_callback(self.__exec_log)
        ui_command.set_data([device])
        command_list.append(ui_command)
        return True

    def __execute(self, command_data, command):
        assert len(command_data) == 1
        device, = command_data

        if does_path_exist(self.__dev_link):
            debug('WorkerLinkSource.__execute()', '%s already exist: removing it' % self.__dev_link)
            self.__exec_log_text = ('rm %s\n' % self.__dev_link)
            unlink(self.__dev_link)
        # quoting isn't really needed, just to show a valid command in the log
        self.__exec_log_text = ('ln -sf %s %s\n\n' % (lib.quote_path(device),
                                                      self.__dev_link))
        symlink(device, self.__dev_link)
        return True

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            return

        fobj.write('%s' % self.__exec_log_text)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
