# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand

class WorkerPaletteDVD(Worker):
    def __init__(self, context, source_id):
        Worker.__init__(self, context, self.__create, _('Create palette'))
        self.__source_container_id = source_id
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create(self, command_list):
        vlist = self.get_video_list(self.__source_container_id)

        for video in vlist:
            disc_id = video.get_container_id()
            title_id = video.get_title_id()

            fname = 'video/disc%d_title%d.yuv' % (disc_id + 1, title_id + 1)

            ui_command = UICommand()
            ui_command.set_command(self.__execute)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_data([video, fname])
            command_list.append(ui_command)

        return True

    def __execute(self, command_data, command):
        assert(len(command_data) == 2)
        video, fname = command_data

        source_video = video.get_source()
        cont_id = source_video.get_container_id()
        source_container = self.get_source_container(cont_id)
        device = source_container.get_device()

        tool_checker = self.get_context().get_tool_checker()
        if not tool_checker.has_tool('paldump'):
            self.__exec_log_text = ('paldump not found\n')
            command.set_error(_('No %s installation found') % 'paldump')
            return False

        program = tool_checker.get_tool_location('paldump')
        shell_cmd = '%s -d %s -p %s' % (program, device, fname)

        self.__exec_log_text = ('%s\n' % shell_cmd)

        if command.run_command(shell_cmd):
            cont_id = video.get_container_id()
            title_id = video.get_title_id()
            title = self.get_dest_title(cont_id, title_id)
            title.set_palette_file(fname)
            return True

        command.set_error(_('You can get help in the winki forum located at http://www.winki-the-ripper.de'))
        return False

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no palette information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
