# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.lib import debug, error

class WorkerScaleProgress:
    def __init__(self):
        self.__command_count = 1.
        self.__current_command = -1.

        self.__chapter_count = 1.
        self.__current_chapter = -1.

        self.__estim_value = -1.
        # used just to pass once data between two member functions
        self.__cur_chunk_progress = -1.
        return

    # --- public interface ---

    def init_command_progress(self, count):
        if count < 1:
            #debug('WorkerScaleProgress.init_command_progress()', 'requested 0 commands', error)
            return False

        self.__command_count = float(count)
        self.__current_command = -1.
        return True

    def next_command_progress(self):
        if self.__current_command + 1 < self.__command_count:
            self.__current_command += 1.
        return

    def init_chapter_progress(self, count):
        if count < 1:
            #debug('WorkerScaleProgress.init_chapter_progress()', 'requested 0 chapters', error)
            return False

        self.__chapter_count = float(count)
        self.__current_chapter = -1.
        return True

    def next_chapter_progress(self):
        if self.__current_chapter + 1 < self.__chapter_count:
            self.__current_chapter += 1.
        return

    def scale_progress(self, value):
        # remember the current progress at the beginning of the first chunk for computing the initial estimation correction
        if self.__current_command == 0 and self.__current_chapter == 0 and value > 0.01 and value < 0.1 and self.__cur_chunk_progress == -1:
            self.__cur_chunk_progress = value

        command_portion = self.__current_command / self.__command_count

        chapter_portion = self.__current_chapter / self.__chapter_count
        chapter_scaled = value / self.__chapter_count
        #print 'progress(%d), total(%d), command_portion(%d), commands(%d), cur_command(%d), chapters(%d), cur_ch(%d)' % (int(value * 100), int((command_portion + (chapter_portion + chapter_scaled) / self.__command_count) * 100), int(command_portion * 100), self.__command_count, self.__current_command, self.__chapter_count, self.__current_chapter)
        return command_portion + (chapter_portion + chapter_scaled) / self.__command_count

    def scale_estimation(self, value):
        # correct the estimation if we have data about the current progress (first chapter only)
        if self.__current_command == 0 and self.__current_chapter == 0 and self.__cur_chunk_progress > -1.:
            self.__estim_value = value / (1. - self.__cur_chunk_progress)
            #print 'UPDATE: estim: %f, progress: %f, new value: %f' % (value, self.__cur_chunk_progress, self.__estim_value)
            self.__cur_chunk_progress = -2.

        # we still cannot correct the estimation
        if self.__estim_value < 0.:
            return value

        commands_left = self.__command_count - self.__current_command - 1
        chapters_left = self.__chapter_count - self.__current_chapter - 1
        chunks_left = chapters_left + commands_left * self.__chapter_count

        #print 'chunks: %d, chapters: %d, cur chunk: %d, cur chapter: %d' % (self.__command_count, self.__chapter_count, self.__current_command, self.__current_chapter)
        #print 'est: %f, corrected: %f, estim chunk: %f, cmd left: %d, chap left: %d, chunks left: %d' % (float(value), float(value + self.__estim_value * chunks_left), float(self.__estim_value), commands_left, chapters_left, chunks_left)
        #print '---'

        return value + self.__estim_value * chunks_left

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
