# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.lib import debug, error

class WriterXML:
    def __init__(self, fname):
        self.__fname = fname
        self.__lines = []
        self.__wspace = 0
        self.__wchar = '\t'
        return

    # -- public interface --

    def write(self):
        xml_file = open(self.__fname, 'w')

        for line in self.__lines:
            xml_file.write(u'%s\n' % line)

        xml_file.close()
        return

    def xml_version(self, options):
        opts = ''
        for opt in options:
            if not opt or not options[opt]:
                continue

            opts += (' %s="%s"' % (opt, options[opt]))

        line = '<?xml%s?>' % opts
        self.__lines.append(line)
        return

    def doctype(self, text):
        line = '<!DOCTYPE %s>' % text
        self.__lines.append(line)
        return

    def stag(self, name, options):
        line = self.__indent()
        
        opts = ''
        for opt in options:
            if not opt or not options[opt]:
                continue

            opts += (' %s="%s"' % (opt, options[opt]))

        line += ('<%s%s>' % (name, opts))
        self.__lines.append(line)
        return

    def etag(self, name):
        line = self.__indent()

        line += ('</%s>' % name)

        self.__lines.append(line)
        return

    def ltag(self, name, options, value):
        line = self.__indent()
        
        opts = ''
        for opt in options:
            if not opt or not options[opt]:
                continue

            opts += (' %s="%s"' % (opt, options[opt]))

        line += ('<%s%s>%s</%s>' % (name, opts, value, name))
        self.__lines.append(line)
        return

    def tag(self, name, options):
        line = self.__indent()
        
        opts = ''
        for opt in options:
            if not opt or not options[opt]:
                continue

            opts += (' %s="%s"' % (opt, options[opt]))

        line += ('<%s%s />' % (name, opts))
        self.__lines.append(line)
        return

    def increment(self, step = 1):
        self.__wspace += step
        return

    def decrement(self, step = 1):
        if (self.__wspace - step) < 0:
            debug('WriterXML.decrement()', 'wrong indentation requested', error)
            return
            
        self.__wspace -= step
        return

    # -- helpers --

    def __indent(self):
        line = ''
        for i in range(self.__wspace):
            line += self.__wchar
        return line

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
